Package: TSP
Type: Package
Title: Infrastructure for the Traveling Salesperson Problem
Version: 1.2.7
Date: 2026-03-22
Authors@R: 
  c(person("Michael", "Hahsler", 
    role = c("aut", "cre", "cph"),
	email = "mhahsler@lyle.smu.edu",
	comment = c(ORCID = "0000-0003-2716-1405")),
    person("Kurt", "Hornik", 
	role = c("aut", "cph"),
	comment = c(ORCID = "0000-0003-4198-9911"))
    )
Description: Basic infrastructure and some algorithms for the traveling
    salesperson problem (also traveling salesman problem; TSP).
    The package provides some simple algorithms and
    an interface to the Concorde TSP solver and its implementation of the
    Chained-Lin-Kernighan heuristic. The code for Concorde
    itself is not included in the package and has to be obtained separately.
    Hahsler and Hornik (2007) <doi:10.18637/jss.v023.i02>.
Classification/ACM: G.1.6, G.2.1, G.4
URL: https://github.com/mhahsler/TSP
BugReports: https://github.com/mhahsler/TSP/issues
Depends: R (>= 3.5.0)
Imports: graphics, foreach, utils, stats, grDevices
Suggests: maps, doParallel, testthat
Encoding: UTF-8
RoxygenNote: 7.3.3
License: GPL-3
Copyright: All code is Copyright (C) Michael Hahsler and Kurt Hornik.
NeedsCompilation: yes
Packaged: 2026-03-22 19:06:40 UTC; mhahsler
Author: Michael Hahsler [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2716-1405>),
  Kurt Hornik [aut, cph] (ORCID: <https://orcid.org/0000-0003-4198-9911>)
Maintainer: Michael Hahsler <mhahsler@lyle.smu.edu>
Repository: CRAN
Date/Publication: 2026-03-23 06:40:20 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:26:58 UTC; unix
Archs: TSP.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.ATSP,dist)
S3method(as.ATSP,matrix)
S3method(as.ETSP,data.frame)
S3method(as.ETSP,matrix)
S3method(as.TOUR,integer)
S3method(as.TOUR,numeric)
S3method(as.TSP,ETSP)
S3method(as.TSP,dist)
S3method(as.TSP,matrix)
S3method(as.dist,TSP)
S3method(as.matrix,ATSP)
S3method(as.matrix,ETSP)
S3method(cut_tour,TOUR)
S3method(image,ATSP)
S3method(image,ETSP)
S3method(image,TSP)
S3method(insert_dummy,ATSP)
S3method(insert_dummy,ETSP)
S3method(insert_dummy,TSP)
S3method(labels,ATSP)
S3method(labels,ETSP)
S3method(labels,TSP)
S3method(n_of_cities,ATSP)
S3method(n_of_cities,ETSP)
S3method(n_of_cities,TSP)
S3method(n_of_cities,default)
S3method(plot,ETSP)
S3method(print,ATSP)
S3method(print,ETSP)
S3method(print,TOUR)
S3method(print,TSP)
S3method(solve_TSP,ATSP)
S3method(solve_TSP,ETSP)
S3method(solve_TSP,TSP)
S3method(tour_length,ATSP)
S3method(tour_length,ETSP)
S3method(tour_length,TOUR)
S3method(tour_length,TSP)
S3method(tour_length,integer)
S3method(write_TSPLIB,ATSP)
S3method(write_TSPLIB,ETSP)
S3method(write_TSPLIB,TSP)
export(ATSP)
export(ETSP)
export(TOUR)
export(TSP)
export(as.ATSP)
export(as.ETSP)
export(as.TOUR)
export(as.TSP)
export(concorde_help)
export(concorde_path)
export(cut_tour)
export(filter_ATSP_as_TSP_dummies)
export(insert_dummy)
export(linkern_help)
export(n_of_cities)
export(read_TSPLIB)
export(reformulate_ATSP_as_TSP)
export(solve_TSP)
export(tour_length)
export(write_TSPLIB)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,gray.colors)
importFrom(graphics,image.default)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(TSP, .registration=TRUE)

# Teaspoon

## See the full documentation on the [ReadTheDocs Pages](https://permafrostnet.gitlab.io/teaspoon/source/about.html)

## [What is it?](https://permafrostnet.gitlab.io/teaspoon/source/about.html)
`tsp` ('teaspoon') is a python library designed to make working with permafrost ground temperature time series data more straightforward, efficient, and reproduceable. Some of the features include:

* Read a variety of common published data formats, datalogger outputs, and model results into a common data structure
    * GEOtop model output
    * GTN-P database export csv
    * NTGS ground temperature report csv
    * Geoprecision datalogger export
    * HoboWare datalogger export
* Export data in a variety of common formats
    * TSP-recommended csv format
    * netcdf
    * 'GTN-P'-style csv
    * 'NTGS'-style csv
* Perform common data transformations
    * Calculate daily, monthly, or yearly means, ignoring averaging periods with missing data
    * Switch between "long" and "wide" dataframes
* Visualize and explore your data with commonly used plots
    * Trumpet curves
    * Temperature-time graphs
    * Colour-contour profiles

## [Installation](https://permafrostnet.gitlab.io/teaspoon/source/install.html)

## [Usage Examples](https://permafrostnet.gitlab.io/teaspoon/source/examples.html)

## [How to contribute](https://permafrostnet.gitlab.io/teaspoon/source/contributions.html)

## Data Standard
TSP also defines a recommended csv format for ground temperature data (which can also be extended to many other kinds of permafrost data). It is described in the [DATA_STANDARD.md](./DATA_STANDARD.md) file in this directory. Files can be read (using the `TSP.to_csv` method) and written (using `read_tsp`) using the teaspoon software package. 

## Citation
If you find this software helpful, please consider using the following citation:

> Brown, N., (2022). tsp ("Teaspoon"): A library for ground temperature data. Journal of Open Source Software, 7(77), 4704, [https://doi.org/10.21105/joss.04704](https://doi.org/10.21105/joss.04704)

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Class TOUR -- Solution to a traveling salesperson problem
#'
#' Class to store the solution of a TSP. Objects of this class are returned by
#' TSP solvers in this package.  Essentially, an object of class `TOUR` is
#' a permutation vector containing the order of cities to visit.
#'
#' Since an object of class `TOUR` is an integer vector, it can be
#' subsetted as an ordinary vector or coerced to an integer vector using
#' `as.integer()`. It also contains the names of the objects as labels.
#' Additionally, `TOUR` has the following attributes: `"method"`,
#' `"tour_length"`.
#'
#' For most functions, e.g., [tour_length()] or [image.TSP()], the
#' `TSP/ATSP` object used to find the tour is still needed, since the tour
#' does not contain the distance information.
#'
#' @family TOUR
#'
#' @param x an integer permutation vector or, for the methods an object of
#' class [TOUR].
#' @param object data (an integer vector) which can be coerced to `TOUR`.
#' @param method character string; method used to create the tour.
#' @param tsp `TSP` object the tour applies to. If available then the tour
#' will include the tour length. Also the labels of the cities will be
#' available in the tour (otherwise the labels of `x` are used).
#' @param ... further arguments are passed on.
#' @author Michael Hahsler
#' @keywords classes
#' @examples
#' TOUR(1:10)
#'
#' ## calculate a tour
#' data("USCA50")
#' tour <- solve_TSP(USCA50)
#' tour
#'
#' ## get tour length directly from tour
#' tour_length(tour)
#'
#' ## get permutation vector
#' as.integer(tour)
#'
#' ## show labels
#' labels(tour)
#' @export
TOUR <- function(x, method = NA, tsp = NULL) {
  if (inherits(x, "TOUR"))
    return(x)

  x <- as.TOUR(x)
  attr(x, "method") <- as.character(method)
  if (!is.null(tsp)) {
    attr(x, "tour_length") <- tour_length(x, tsp)
    names(x) <- labels(tsp)[x]
  }

  x
}

## coercion
#' @rdname TOUR
#' @export
as.TOUR <- function(object)
  UseMethod("as.TOUR")

#' @rdname TOUR
#' @export
as.TOUR.numeric <-  function(object) {
  l <- labels(object)	    ### preserve labels
  object <- as.integer(object)
  names(object) <- l
  as.TOUR(object)
}

#' @rdname TOUR
#' @export
as.TOUR.integer <- function(object) {
  ## check tour
  if (any(object < 1) ||
      any(object > length(object)) || any(is.na(object)))
    stop("tour contains illegal elements.")

  if (any(duplicated(object)))
    stop("tour indices are not unique.")

  class(object) <- c("TOUR", class(object))
  object
}


#' @rdname TOUR
#' @export
print.TOUR <- function(x, ...) {
  cat("object of class", sQuote(class(x)[1]), "\n")
  cat("result of method",
    sQuote(attr(x, "method")),
    "for",
    length(x),
    "cities\n")
  if (!is.null(attr(x, "tour_length")))
    cat("tour length:", attr(x, "tour_length"), "\n")
  else
    cat("tour length: unknown\n")
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Read and write TSPLIB files
#'
#' Reads and writes TSPLIB format files. TSPLIB files can be used by most TSP
#' solvers. Many problems in TSPLIB format can be found in the local 
#' copy of the [TSPLIB95 problem library](https://github.com/mhahsler/TSP/tree/master/TSPLIB95).
#' 
#' 
#' In the TSPLIB format distances are represented by integer values. Therefore,
#' if `x` contains `double` values (which is normal in R) the values
#' given in `x` are multiplied by \eqn{10^{precision}} before coercion to
#' `integer`. Note that therefore all results produced by programs using
#' the TSPLIB file as input need to be divided by \eqn{10^{precision}} (i.e.,
#' the decimal point has to be shifted `precision` placed to the left).
#'
#' Currently only the following `EDGE_WEIGHT_TYPE`s are implemented:
#' `EXPLICIT`, `EUC_2D`, `EUC_3D`, `ATT` and `GEO`.
#'
#' @name TSPLIB
#' @aliases TSPLIB
#' @family TSP
#'
#' @param x an object with a TSP problem.
#' `NA`s are not allowed.
#' @param file file name or a [connection].
#' @param precision controls the number of decimal places used to represent
#' distances (see details).  If `x` already is `integer`, this
#' argument is ignored and `x` is used as is.
#' @param inf replacement value for `Inf` (TSPLIB format cannot handle
#' `Inf`). If `inf` is `NULL`, a large value of \eqn{max(x) + 2
#' range(x)} (ignoring infinite entries) is used.
#' @param neg_inf replacement value for `-Inf`.  If no value is specified,
#' a small value of \eqn{min(x) - 2 range(x)} (ignoring infinite entries) is
#' used.
#' @returns returns an object of class `TSP` or
#' `ATSP`.
#' @author Michael Hahsler
#' @references 
#' Reinelt, Gerhard. 1991. “TSPLIB—a Traveling Salesman Problem Library.” ORSA Journal on Computing 3 (4): 376–84. \doi{10.1287/ijoc.3.4.376}
#' 
#' @keywords file
#' @examples
#'
#' ## Drilling problem from TSP
#' drill <- read_TSPLIB(system.file("examples/d493.tsp", package = "TSP"))
#' drill
#' tour <- solve_TSP(drill, method = "nn", two_opt = TRUE)
#' tour
#' plot(drill, tour, cex=.6, col = "red", pch= 3, main = "TSPLIB: d493")
#'
#'
#' ## Write and read data in TSPLIB format
#' x <- data.frame(x=runif(5), y=runif(5))
#'
#' ## create TSP, ATSP and ETSP (2D)
#' tsp <- TSP(dist(x))
#' atsp <- ATSP(dist(x))
#' etsp <- ETSP(x[,1:2])
#'
#' write_TSPLIB(tsp, file="example.tsp")
#' #file.show("example.tsp")
#' r <- read_TSPLIB("example.tsp")
#' r
#'
#' write_TSPLIB(atsp, file="example.tsp")
#' #file.show("example.tsp")
#' r <- read_TSPLIB("example.tsp")
#' r
#'
#' write_TSPLIB(etsp, file="example.tsp")
#' #file.show("example.tsp")
#' r <- read_TSPLIB("example.tsp")
#' r
#'
#' ## clean up
#' unlink("example.tsp")
#' @export
read_TSPLIB <- function(file, precision = 0) {

  ## TSP or ATSP
  type <- NULL

  lines <- readLines(file)

  ## get info
  metadata <- grep(":", lines)

  info <- list()
  lapply(strsplit(lines[metadata], "[[:space:]]*:[[:space:]]*"),
    FUN = function(x) {
      x[2] <- sub("[[:space:]]*$","",x[2]) ## kill trailing spaces
      info[[toupper(x[1])]] <<- toupper(x[2])
    })

  ## check
  if(substr(info$TYPE, 1, 3) == "TSP") type <- "TSP"
  else if(substr(info$TYPE, 1, 3) == "ATS") type <- "ATSP"
  else stop ("Currently the only implemented TYPEs are TSP and ATS(P)!")

  dim <- as.integer(info$DIMENSION)

  if(info$EDGE_WEIGHT_TYPE == "EXPLICIT") {
    ## get data
    data_start <- grep("EDGE_WEIGHT_SECTION", lines, ignore.case = TRUE)
    if(length(data_start) == 0) stop("EDGE_WEIGHT_SECTION missing")

    data <- lines[(data_start+1):length(lines)]
    data <- sub("EOF", "", data, ignore.case = TRUE) ## kill optional EOF
    data <- sub("^[[:space:]]*", "", data)## kill leading spaces
    data <- strsplit(paste(data, collapse = " "), "[[:space:]]+")[[1]]

    ## remove everything after the data
    if(info$EDGE_WEIGHT_FORMAT == "FULL_MATRIX")
      data <- data[1:(dim^2)]
    else if(info$EDGE_WEIGHT_FORMAT == "UPPER_ROW"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_COL"
      || info$EDGE_WEIGHT_FORMAT == "UPPER_COL"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_ROW")
      data <- data[1:(dim*(dim-1)/2)]
    else if(info$EDGE_WEIGHT_FORMAT == "UPPER_DIAG_ROW"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_DIAG_COL"
      || info$EDGE_WEIGHT_FORMAT == "UPPER_DIAG_COL"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_DIAG_ROW")
      data <- data[1:(dim*(dim-1)/2 + dim)]

    data <- as.numeric(data)

    if(precision != 0) data <- data / 10^precision

    ## ATSP
    if(type == "ATSP") {
      if(info$EDGE_WEIGHT_FORMAT == "FULL_MATRIX"){
        ## todo: find out if FULL_MATRIX is row or column oriented?
        data <- matrix(data, ncol = dim)
      }else stop("ATSP needs EDGE_WEIGHT_FORMAT FULL_MATRIX!")

      return(ATSP(data))
    }

    ## TSP
    ## we have only symmetric data here!
    if(info$EDGE_WEIGHT_FORMAT == "FULL_MATRIX") {
      data <- as.dist(matrix(data, ncol = dim))

    }else if(info$EDGE_WEIGHT_FORMAT == "UPPER_ROW"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_COL") {
      class(data) <- "dist"
      attr(data, "Size")  <- dim
      attr(data, "Diag")  <- FALSE
      attr(data, "Upper") <- FALSE

    }else if(info$EDGE_WEIGHT_FORMAT == "UPPER_COL"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_ROW") {

      m <- matrix(NA, nrow = dim, ncol = dim)
      m[upper.tri(m, diag = FALSE)] <- data
      data <- as.dist(t(m))

    }else if(info$EDGE_WEIGHT_FORMAT == "UPPER_DIAG_ROW"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_DIAG_COL") {
      class(data) <- "dist"
      attr(data, "Size")  <- dim
      attr(data, "Diag")  <- TRUE
      attr(data, "Upper") <- FALSE

      data <- as.dist(data, diag = FALSE)

    }else if(info$EDGE_WEIGHT_FORMAT == "UPPER_DIAG_COL"
      || info$EDGE_WEIGHT_FORMAT == "LOWER_DIAG_ROW") {

      m <- matrix(NA, nrow = dim, ncol = dim)
      m[upper.tri(m, diag = TRUE)] <- data
      data <- as.dist(t(m))


    }else stop("The specified EDGE_WEIGHT_FORMAT is not implemented!")

    return(TSP(data))

  } else if (info$EDGE_WEIGHT_TYPE %in% c("EUC_2D", "EUC_3D", "ATT", "GEO")) {
    return(.read_tsplib_coord_problem(lines, dim, info$EDGE_WEIGHT_TYPE))
  }
  stop("EDGE_WEIGHT_TYPE not implemented! Implemented types are EXPLICIT, EUC_2D, EUC_3D, ATT and GEO")
}

.read_tsplib_coord_problem <- function(lines, dim, edge_weight_type) {
  coords <- .read_tsplib_node_coords(lines, dim)
  switch(edge_weight_type,
    ATT = TSP(.tsplib_att_dist(coords), labels = rownames(coords), method = "ATT"),
    GEO = TSP(.tsplib_geo_dist(coords), labels = rownames(coords), method = "GEO"),
    ETSP(coords))
}

.read_tsplib_node_coords <- function(lines, dim) {
  data_start <- grep("NODE_COORD_SECTION", lines, ignore.case = TRUE)
  if(length(data_start) == 0) stop("NODE_COORD_SECTION missing")

  data <- lines[(data_start+1):(data_start+dim)]
  data <- trimws(data)
  data <- matrix(as.numeric(unlist(strsplit(data, split="\\s+"))),
    nrow = dim, byrow = TRUE)
  rownames(data) <- as.character(data[,1]) # also parse node labels if they are not in order
  data[,-1]
}

.tsplib_att_dist <- function(coords) {
  rij <- sqrt((stats::dist(coords[, 1:2]) ^ 2) / 10.0)
  tij <- round(rij)
  tij + (tij < rij)
}

.tsplib_geo_dist <- function(coords) {
  dddmm_rad <- function(x) pi * (round(x) + 5 * (x - round(x)) / 3) / 180
  lat <- dddmm_rad(coords[, 1])
  lon <- dddmm_rad(coords[, 2])

  idx <- which(upper.tri(matrix(0, nrow = nrow(coords), ncol = nrow(coords))),
    arr.ind = TRUE)
  q1 <- cos(lon[idx[, 1]] - lon[idx[, 2]])
  q2 <- cos(lat[idx[, 1]] - lat[idx[, 2]])
  q3 <- cos(lat[idx[, 1]] + lat[idx[, 2]])
  x <- 0.5 * ((1 + q1) * q2 - (1 - q1) * q3)
  dij <- as.integer(6378.388 * acos(pmax(-1, pmin(1, x))) + 1)

  d <- matrix(0L, nrow = nrow(coords), ncol = nrow(coords),
    dimnames = list(rownames(coords), rownames(coords)))
  d[idx[, 2:1]] <- dij
  as.dist(d)
}

#' @rdname TSPLIB
#' @export
write_TSPLIB <- function(x, file, precision = 6, inf = NULL, neg_inf = NULL)
  UseMethod("write_TSPLIB")


## write a simple TSPLIB format file from an object of class TSP
## (contains a dist object or a symmetric matrix)

## TSP has data as integer
#' @rdname TSPLIB
#' @export
write_TSPLIB.TSP <- function(x, file, precision = 6,
  inf = NULL, neg_inf = NULL) {

  ## prepare data (NA, Inf)
  if(any(is.na(x))) stop("NAs not allowed!")
  x <- .replaceInf(x, inf, neg_inf)

  ## Concorde can handle UPPER_ROW and dist (lower triangle matrix)
  ## is symmetric.
  format <- "EDGE_WEIGHT_FORMAT: UPPER_ROW"

  zz <- file(file, "w")

  cat("NAME: TSP",
    "COMMENT: Generated by write_TSPLIB (R-package TSP)",
    "TYPE: TSP",
    paste("DIMENSION:", n_of_cities(x)),
    "EDGE_WEIGHT_TYPE: EXPLICIT",
    format,
    file = zz, sep = "\n")

  ## only integers can be used as weights
  if(storage.mode(x) != "integer" && precision != 0) x <- x * 10^precision

  x <- suppressWarnings(as.integer(x))
  if(any(is.na(x))) stop("Integer overflow, please reduce precision.")

  cat("EDGE_WEIGHT_SECTION", x, file = zz, sep = "\n")
  cat("EOF", file = zz, sep = "\n")

  close(zz)
}


#' @rdname TSPLIB
#' @export
write_TSPLIB.ATSP <- function(x, file, precision = 6, inf = NULL, neg_inf = NULL) {

  ## prepare data (NA, Inf)
  if(any(is.na(x))) stop("NAs not allowed!")
  x <- .replaceInf(x, inf, neg_inf)

  format <- "EDGE_WEIGHT_FORMAT: FULL_MATRIX"

  zz <- file(file, "w")

  cat("NAME: ATSP",
    "COMMENT: Generated by write_TSPLIB (R package TSP)",
    "TYPE: ATSP",
    paste("DIMENSION:", n_of_cities(x)),
    "EDGE_WEIGHT_TYPE: EXPLICIT",
    format,
    file = zz, sep = "\n")


  ## only integers can be used as weights
  if(storage.mode(x) != "integer") x <- x * 10^precision

  x <- suppressWarnings(as.integer(x))
  if(any(is.na(x))) stop("integer overflow, please reduce precision.")

  cat("EDGE_WEIGHT_SECTION", x, file = zz, sep = "\n")
  cat("EOF", file = zz, sep = "\n")

  close(zz)
}


## ETSP use data as real
#' @rdname TSPLIB
#' @export
write_TSPLIB.ETSP <- function(x, file, precision = 6,
  inf = NULL, neg_inf = NULL) {

  if(any(is.na(x))) stop("NAs are not allowed!")
  if(any(!is.finite(x))) stop("Only finite values allowed!")

  if(ncol(x) == 2) type <- "EUC_2D"
  else if(ncol(x) == 3) type <- "EUC_3D"
  else stop("Only EUC_2D and EUC_3D supported.")

  zz <- file(file, "w")

  cat("NAME: ETSP",
    "COMMENT: Generated by write_TSPLIB (R package TSP)",
    "TYPE: TSP",
    paste("DIMENSION:", n_of_cities(x)),
    paste("EDGE_WEIGHT_TYPE:", type),
    file = zz, sep = "\n")

  ## fix row names
  rownames(x) <- NULL
  x <- do.call(data.frame, lapply(1:ncol(x), FUN =
      function(i) sprintf(paste("%0.", precision, "e", sep=""), x[,i])))

  cat("NODE_COORD_SECTION", file = zz, sep = "\n")
  write.table(x, quote=FALSE, col.names = FALSE, file = zz)
  cat("EOF", file = zz, sep = "\n")

  close(zz)
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyright (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' TSP solver interface
#'
#' Common interface to all TSP solvers in this package.
#'
#' # TSP Methods
#'
#' TSP supports tour construction methods, tour improvement heuristics and 
#' an interface to Concorde (a state-of-the-art external solver).
#' Tour construction methods create a tour from scratch, typically using a 
#' construction heuristic. Tour improvement heuristics take an existing tour and
#' try to improve it. If no initial tour is provided for the improvement 
#' heuristic, then an initial tour (e.g., a random tour) are automatically created.
#' 
#' Most heuristic methods accept the following extra control parameters in addition
#' the parameters specified below:
#'
#' * "rep": an integer indicating how many replications (random restarts) should be performed.
#'  The best result is returned. The replications can be performed in parallel 
#'  (see Section Parallel Execution Support). 
#' * "two_opt": a logical indicating if two-opt refinement should be performed on the
#'   constructed tour. This is just for convenience so a constructive heuristic can be
#'   followed by two-opt refinement in a single call.
#' * "verbose": a logical. Most implementations provide verbose output to monitor progress.
#'   This argument can also be helpful to see what heuristics are used and what the
#'   default parameters are.
#'
#' ## Tour Construction Methods
#' 
#' These methods construct tours from scratch. 
#' The available tour construction methods are:
#'
#' - __"identity", "random"__ return a tour representing the order in the data
#'   (identity order) or a random order. \[TSP, ATSP\]
#'
#' - __"nearest_insertion", "farthest_insertion", "cheapest_insertion", "arbitrary_insertion"__
#'   Nearest, farthest, cheapest and
#'   arbitrary insertion algorithms for a symmetric and asymmetric TSP
#'   (Rosenkrantz et al. 1977). \[TSP, ATSP\]
#'
#'   The distances between cities are stored in a distance matrix \eqn{D} with
#'   elements \eqn{d(i,j)}.  All insertion algorithms start with a tour
#'   consisting of an arbitrary city and choose in each step a city \eqn{k} not
#'   yet on the tour. This city is inserted into the existing tour between two
#'   consecutive cities \eqn{i} and \eqn{j}, such that \deqn{d(i,k) + d(k,j) -
#'   d(i,j)} is minimized. The algorithms stops when all cities are on the tour.
#'
#'   The nearest insertion algorithm chooses city \eqn{k} in each step as the
#'   city which is \emph{nearest} to a city on the tour.
#'
#'   For farthest insertion, the city \eqn{k} is chosen in each step as the city
#'   which is \emph{farthest} to any city on the tour.
#'
#'   Cheapest insertion chooses the city \eqn{k} such that the cost of inserting
#'   the new city (i.e., the increase in the tour's length) is minimal.
#'
#'   Arbitrary insertion chooses the city \eqn{k} randomly from all cities not
#'   yet on the tour.
#'
#'   Nearest and cheapest insertion tries to build the tour using cities which
#'   fit well into the partial tour constructed so far.  The idea behind behind
#'   farthest insertion is to link cities far away into the tour fist to
#'   establish an outline of the whole tour early.
#'
#'   Additional control options:
#'   - "start" index of the first city (default: a random city).
#'
#' - __"nn", "repetitive_nn"__ Nearest neighbor and repetitive
#'   nearest neighbor algorithms for symmetric and asymmetric TSPs (Rosenkrantz
#'   et al. 1977). \[TSP, ATSP\]
#'
#'   The algorithm starts with a tour containing a random city. Then the
#'   algorithm always adds to the last city on the tour the nearest not yet
#'   visited city. The algorithm stops when all cities are on the tour.
#'
#'   Repetitive nearest neighbor constructs a nearest neighbor tour for each city
#'   as the starting point and returns the shortest tour found.
#'
#'   Additional control options:
#'   - "start" index of the first city (default: a random city).
#'
#' ## Tour Improvement Heuristics
#'
#' Tour improvement methods take a tour as the argument `tour` and try to improve the 
#' tour. If no tour is provided. A random tour is used as the initial tour.
#' 
#' The following tour improvement heuristics are available:
#'
#' - __"sa"__ Simulated Annealing for TSPs (Kirkpatrick et al, 1983) \[TSP, ATSP\]
#'
#'   A tour refinement method that uses simulated annealing with subtour 
#'   reversal as local move. The used optimizer is
#'   [stats::optim()] with method `"SANN"`. This method is 
#'   typically a lot slower than `"two_opt"` and requires parameter tuning for the 
#'   cooling schedule.
#'   
#'   __Note:__ For speed, a relatively small number of evaluations (`maxit`) is 
#'   used by default. A much larger value will result in more competitive results.
#'
#'   Additional control options:
#'   - "tour" an existing tour which should be improved.
#'     If no tour is given, a random tour is used.
#'   - "local_move" a function that creates a local move with the current 
#'      tour as the first and the TSP as the second parameter. Defaults to
#'      randomized subtour reversal.
#'   - "temp" initial temperature. Defaults to the length of the current tour divided
#'      by the number of cities.
#'   - "tmax" number of evaluations per temperature step. Default is 10.
#'   - "maxit" number of evaluations. Default is 10000.
#'   - "trace" set to 1 to print progress. 
#'   
#'   See [stats::optim()] for more details on the parameters.
#'   
#' - __"two_opt"__ Two edge exchange improvement procedure (Croes 1958). \[TSP, ATSP\]
#'
#'   This is a tour refinement procedure which systematically exchanges two edges
#'   in the graph represented by the distance matrix till no improvements are
#'   possible. Exchanging two edges is equal to reversing part of the tour. The
#'   resulting tour is called _2-optimal._
#'
#'   Additional control options:
#'   - "tour" an existing tour which should be improved.
#'     If no tour is given, a random tour is used.
#'   - "two_opt_repetitions" number of times to try with random restarts 
#'    (default: 1).
#'
#'  ## State-of-the-art Solvers Interfaces
#'
#' - __"concorde"__ Concorde algorithm (Applegate et al. 2001). \[TSP, ETSP\]
#'
#'   Concorde is an advanced exact TSP solver for _symmetric_ TSPs
#'   based on branch-and-cut.
#'   ATSPs can be solved using [reformulate_ATSP_as_TSP()] done automatically
#'   with `as_TSP = TRUE`.
#'   The program is not included in this package and
#'   has to be obtained and installed separately.
#'
#'   Additional control options:
#'   - "exe" a character string containing the path to the executable (see [Concorde]).
#'   - "clo" a character string containing command line options for
#'      Concorde, e.g., `control = list(clo = "-B -v")`. See
#'      [concorde_help()] on how to obtain a complete list of available command
#'      line options.
#'   - "precision" an integer which controls the number of decimal
#'     places used for the internal representation of distances in Concorde. The
#'     values given in `x` are multiplied by \eqn{10^{precision}} before being
#'     passed on to Concorde. Note that therefore the results produced by Concorde
#'     (especially lower and upper bounds) need to be divided by
#'     \eqn{10^{precision}} (i.e., the decimal point has to be shifted
#'     `precision` placed to the left). The interface to Concorde uses
#'     [write_TSPLIB()].
#'   - "verbose" logical; `FALSE` suppresses the output printed to the terminal.
#'
#' - __"linkern"__ Concorde's Chained Lin-Kernighan heuristic (Applegate et al. 2003). \[TSP, ETSP\]
#'
#'   The Lin-Kernighan (Lin and Kernighan 1973) heuristic uses variable \eqn{k}
#'   edge exchanges to improve an initial tour.  The program is not included in
#'   this package and has to be obtained and installed separately (see [Concorde]).
#'
#'   Additional control options: see Concorde above.
#'
#' # Parallel Execution Support
#'
#' If several repetitions are performed (this includes method
#' `"repetitive_nn"`) then \pkg{foreach} is used so they can be performed
#' in parallel on multiple cores/machines. To enable parallel execution an
#' appropriate parallel backend needs to be registered 
#' (e.g., [doParallel::registerDoParallel()]).
#' 
#' # Treatment of `NA`s and Infinite Values in `x`
#'
#' [TSP] and [ATSP] need to contain valid distances. `NA`s are not allowed. `Inf` is
#' allowed and can be used to model the missing edges in incomplete graphs
#' (i.e., the distance between the two objects is infinite) or infeasible connections.
#' Internally, `Inf` is replaced by a large value given by \eqn{max(x) + 2 range(x)}.
#' Note that the solution might still place the two objects next to each other
#' (e.g., if `x` contains several unconnected subgraphs) which results in
#' a path length of `Inf`. `-Inf` is replaced by \eqn{min(x) - 2 range(x)} and
#' can be used to encourage the solver to place two objects next to each other.
#'
#' # Solving ATSP and ETSP
#'
#' Some solvers (including Concorde) cannot directly solve [ATSP]
#' directly. `ATSP` can be reformulated as larger `TSP` and solved
#' this way. For convenience, `solve_TSP()` has an extra argument
#' `as_TSP` which can be set to `TRUE` to automatically solve the
#' `ATSP` reformulated as a `TSP` (see [reformulate_ATSP_as_TSP()]).
#'
#' Only methods "concorde" and "linkern" currently solve [ETSP]s directly.
#' For all other methods, ETSPs are converted into TSPs by creating a
#' distance matrix and then solved. Note: distance matrices can become 
#' very large leading to long memory issues and long computation times.
#'
#' @family TSP
#' @family TOUR
#'
#' @param x a TSP problem.
#' @param method method to solve the TSP (default: "arbitrary insertion" 
#' construction followed by the "two_opt" improvement heuristic.
#' @param control a list of arguments passed on to the TSP solver selected by
#' `method`.
#' @param as_TSP should the ATSP reformulated as a TSP for the solver?
#' @param ...  additional arguments are added to `control`.
#' @return An object of class [TOUR].
#' @author Michael Hahsler
#' @references
#' David Applegate, Robert Bixby, Vasek Chvatal, William Cook
#' (2001): TSP cuts which do not conform to the template paradigm,
#' Computational Combinatorial Optimization, M. Junger and D. Naddef (editors),
#' Springer.\doi{10.1007/3-540-45586-8_7}
#'
#' D. Applegate, W. Cook and A. Rohe (2003): Chained Lin-Kernighan for Large
#' Traveling Salesman Problems.  \emph{INFORMS Journal on Computing},
#' 15(1):82--92. \doi{10.1287/ijoc.15.1.82.15157}
#'
#' G.A. Croes (1958): A method for solving traveling-salesman problems.
#' \emph{Operations Research}, 6(6):791--812. \doi{10.1287/opre.6.6.791}
#'
#' Kirkpatrick, S., C. D. Gelatt, and M. P. Vecchi (1983): 
#' Optimization by Simulated Annealing. \emph{Science} 220 (4598): 671–80
#' \doi{10.1126/science.220.4598.671}
#'
#' S. Lin and B. Kernighan (1973): An effective heuristic algorithm for the
#' traveling-salesman problem. \emph{Operations Research}, 21(2): 498--516.
#' \doi{10.1287/opre.21.2.498}
#'
#' D.J. Rosenkrantz, R. E. Stearns, and Philip M. Lewis II (1977): An analysis
#' of several heuristics for the traveling salesman problem.  \emph{SIAM
#' Journal on Computing}, 6(3):563--581. \doi{10.1007/978-1-4020-9688-4_3}
#' @keywords optimize
#' @examples
#'
#' ## solve a simple Euclidean TSP (using the default method)
#' etsp <- ETSP(data.frame(x = runif(20), y = runif(20)))
#' tour <- solve_TSP(etsp)
#' tour
#' tour_length(tour)
#' plot(etsp, tour)
#'
#' ## manually run a construction followed by an improvement heuristic by 
#' ## calling solve_TSP() twice.
#' tour_constr <- solve_TSP(etsp, method = "nn")
#' tour_constr
#' 
#' tour_improved <- solve_TSP(etsp, method = "sa", tour = tour_constr)
#' tour_improved
#'
#' ## compare methods
#' data("USCA50")
#' USCA50
#' methods <- c("identity", "random", "nearest_insertion",
#'   "cheapest_insertion", "farthest_insertion", "arbitrary_insertion",
#'   "nn", "repetitive_nn", "two_opt", "sa")
#'
#' ## calculate tours
#' tours <- lapply(methods, FUN = function(m) solve_TSP(USCA50, method = m))
#' names(tours) <- methods
#'
#' ## use the external solver which has to be installed separately
#' \dontrun{
#' tours$concorde  <- solve_TSP(USCA50, method = "concorde")
#' tours$linkern  <- solve_TSP(USCA50, method = "linkern")
#' }
#'
#' ## register a parallel backend to perform repetitions in parallel
#' \dontrun{
#' library(doParallel)
#' registerDoParallel()
#' }
#'
#' ## add some tours using repetition and two_opt refinements
#' tours$'nn+two_opt' <- solve_TSP(USCA50, method = "nn", two_opt = TRUE)
#' tours$'nn+rep_10' <- solve_TSP(USCA50, method = "nn", rep = 10)
#' tours$'nn+two_opt+rep_10' <- solve_TSP(USCA50, method = "nn", two_opt = TRUE, rep = 10)
#' tours$'arbitrary_insertion+two_opt' <- solve_TSP(USCA50)
#'
#' ## show first tour
#' tours[[1]]
#'
#' ## compare tour lengths
#' opt <- 14497 # obtained by Concorde
#' tour_lengths <- sort(sapply(tours, tour_length), decreasing = TRUE)
#' dotchart(tour_lengths / opt * 100 - 100, xlab = "percent excess over optimum")
#' abline(v = 0, col = "red")
#' @export
solve_TSP <- function(x,
  method = NULL,
  control = NULL,
  ...)
  UseMethod("solve_TSP")

## TSP
#' @rdname solve_TSP
#' @export
solve_TSP.TSP <- function(x,
  method = NULL,
  control = NULL,
  ...) {
  .solve_TSP(x, method, control, ...)
}

## ATSP
#' @rdname solve_TSP
#' @export
solve_TSP.ATSP <-
  function(x,
    method = NULL,
    control = NULL,
    as_TSP = FALSE,
    ...) {
    # force as_TSP for solvers that cannot deal with ATSPs
    m <- pmatch(tolower(method), c("concorde", "linkern"))
    if (!is.na(m) && length(m) > 0L && !as_TSP) {
      warning(
        "NOTE: Solver cannot solve the ATSP directly. Reformulating ATSP as TSP. Use 'as_TSP = TRUE' to supress this warning.\n"
      )
      as_TSP <- TRUE
    }

    # reformulate ATSP as TSP
    if (as_TSP) {
      x_atsp <- x
      x <- reformulate_ATSP_as_TSP(x_atsp)
    }

    tour <- .solve_TSP(x, method, control, ...)

    if (as_TSP)
      tour <- filter_ATSP_as_TSP_dummies(tour, atsp = x_atsp)

    tour
  }

## ETSP
#' @rdname solve_TSP
#' @export
solve_TSP.ETSP <- function(x,
  method = NULL,
  control = NULL,
  ...) {
  ## all but concorde and linkern can currently only do TSP
  ## TODO: Implement insertion, NN, etc directly for ETSP to avoid creating 
  ##       a large distance matrix
  m <- pmatch(tolower(method), c("concorde", "linkern"))
  if (length(m) == 0L || is.na(m)) {
    if (nrow(x) > 10000L)
      warning("Convertion of ETSP to TSP creates a distance matrix of size O(n^2). ", 
              "This operation may be slow or you may run out of memory. ",
              "Currently, only \"concorde\" and \"linkern\" can directly work with ETSP.", 
              immediate. = TRUE)
    x <- as.TSP(x)
  }

  .solve_TSP(x, method, control, ...)
}



## Deal with Inf: punish (-)Inf with max (min) +(-) 2*range
.replaceInf <- function(x, pInf = NULL, nInf = NULL) {
  if (any(is.infinite(x))) {
    range_x <- range(x, na.rm = TRUE, finite = TRUE)

    # data with only a single non-inf value.
    diff_range <- diff(range_x)
    if (diff_range == 0) {
      if (range_x[1] == 0)
        diff_range <- 1
      else
        diff_range <- range_x[1] * 2
    }

    if (is.null(pInf))
      pInf <- range_x[2] + 2 * diff_range
    if (is.null(nInf))
      nInf <- range_x[1] - 2 * diff_range
    x[x == Inf] <- pInf
    x[x == -Inf] <- nInf
  }
  x
}

## workhorse
.solve_TSP <- function(x,
  method = NULL,
  control = NULL,
  ...) {
  ## add ... to control
  control <- c(control, list(...))

  ## methods
  methods <- c(
    "identity",
    "random",
    "nearest_insertion",
    "farthest_insertion",
    "cheapest_insertion",
    "arbitrary_insertion",
    "nn",
    "repetitive_nn",
    ### deprecate use two_opt
    "2-opt",
    "two_opt",
    "concorde",
    "linkern",
    "sa"
  )

  ## default is arbitrary_insertion + two_opt
  if (is.null(method)) {
    method <- "arbitrary_insertion"
    control$two_opt <- TRUE
  } else
    method <- match.arg(tolower(method), methods)
    
  ## no rep or two_opt for these!
  if (method == "concorde" || method == "linkern") {
    if (control$rep %||% 1L > 1L || control$two_opt %||% FALSE)
      warning("control parameters rep and two_opt not available for methods concorde and linkern and are ignored!")
    control$rep <- NULL
    control$two_opt <- NULL
  }

  ## check for NAs
  if (any(is.na(x)))
    stop("NAs not allowed!")

  ## Inf
  x_ <- .replaceInf(x)

  ## work horses
  .solve_TSP_worker <- function(x_, method, control) {
    order <- switch(
      method,
      identity = seq(n_of_cities(x_)),
      random = sample(n_of_cities(x_)),
      nearest_insertion = tsp_insertion(x_, type = "nearest", control = control),
      farthest_insertion = tsp_insertion(x_, type = "farthest", control = control),
      cheapest_insertion = tsp_insertion(x_, type = "cheapest", control = control),
      #      arbitrary_insertion = tsp_insertion(x_, type = "arbitrary", control = control),
      arbitrary_insertion = tsp_insertion_arbitrary(x_, control = control),
      nn = tsp_nn(x_, control = control),
      repetitive_nn = tsp_repetitive_nn(x_, control = control),
      two_opt = tsp_two_opt(x_, control = control),
      '2-opt' = tsp_two_opt(x_, control = control),
      concorde = tsp_concorde(x_, control = control),
      linkern = tsp_linkern(x_, control = control),
      sa = tsp_SA(x_, control = control)
    )
  
    ### do refinement two_opt
    if (!is.null(control[["two_opt"]]) && control[["two_opt"]]) {
      order <- tsp_two_opt(x_, control = c(control, list(tour = order)))
      method <- paste(method , "+two_opt", sep = "")
    }

    TOUR(order, method = method, tsp = x)
  }

  ## do rep?
  n <- control$rep %||% 1L

  ## reps are handled internally!
  if (method == "repetitive_nn")
    n <- 1L

  if (n == 1L)
    return(.solve_TSP_worker(x_, method, control))

  #l <- replicate(n, .solve_TSP_worker(x_, method, control), simplify = FALSE)
  l <-
    foreach(i = 1:n) %dopar% .solve_TSP_worker(x_, method, control)

  l <- l[[which.min(sapply(l, attr, "tour_length"))]]
  attr(l, "method") <-
    paste(attr(l, "method"), "_rep_", n, sep = "")

  return(l)
}

library(TSP)
library(testthat)


context("solve_TSP (Concorde and Linkern)")

m <- rbind(
  c(0, 1, 0, 1),
  c(1, 0, 1, Inf),
  c(0, 1, 0, 1),
  c(1, Inf, 1, 0)
)

d <- as.dist(m)
tsp <- TSP(d)
tsp

skip_if_not(
  Sys.which("concorde") != "" &&
    Sys.which("linkern") != "",
  message = "skipped test for concorde/linkern. Not installed.")

v <- FALSE
o <- solve_TSP(tsp, method="concorde", verbose = v)
expect_equivalent(tour_length(tsp, o), 4)

# large numbers should be scaled right.
expect_warning(o_large <- solve_TSP(tsp*2^15, method="concorde", verbose = v), regex = "Converting the provided distances to integers")
expect_equivalent(o, o_large)

expect_warning(o_large <- solve_TSP(tsp*10^10, method="concorde", verbose = v), regex = "Converting the provided distances to integers")
expect_equivalent(o, o_large)

# expect warning for rounding
expect_warning(o_large <- solve_TSP(tsp*2^15+0.1, method="concorde", verbose = v), regex = "Converting the provided distances to integers")
expect_equivalent(o, o_large)

# expect a warning for rounding
expect_warning(o_round <- solve_TSP(tsp/0.3, method="concorde", verbose = v), regex = "Converting the provided distances to integers")
expect_equivalent(o, o_round)

o <- solve_TSP(tsp, method="linkern", verbose = v)
expect_equivalent(tour_length(tsp, o), 4)

# test ATSP
#data <- matrix(runif(5^2), ncol = 5, dimnames = list(1:5, 1:5))
data <- structure(c(0.13930352916941, 0.897691324818879, 0.509101516567171,
  0.430898967897519, 0.141799068776891, 0.0334562903735787, 0.902805947931483,
  0.203576791565865, 0.435874363640323, 0.0641707226168364, 0.101683554705232,
  0.631239329231903, 0.555331876967102, 0.0829615572001785, 0.272443652851507,
  0.215095571940765, 0.532841097796336, 0.795302660670131, 0.43256876245141,
  0.582661165855825, 0.250269076088443, 0.164849652675912, 0.638499777996913,
  0.857200765516609, 0.0134391817264259), .Dim = c(5L, 5L), .Dimnames = list(
    c("1", "2", "3", "4", "5"), c("1", "2", "3", "4", "5")))

atsp <- ATSP(data)

## Concorde (gives conversation warning for reformulation of ATSP to TSP)
expect_warning(o1 <- solve_TSP(atsp, method = "concorde", verbose = v), regex = "Solver cannot solve the ATSP directly")
o2 <- solve_TSP(atsp, method = "concorde", as_TSP = TRUE, verbose = v)
expect_equal(length(o1), 5L)
expect_equal(length(o2), 5L)

# Concorde should find the optimal solution of 0.8082826
expect_equal(round(tour_length(o1), 7), 0.8082826)
expect_equal(round(tour_length(o2), 7), 0.8082826)

## Linkern
## warning for reformulate as TSP automatically
expect_warning(o1 <- solve_TSP(atsp, method = "linkern", verbose = v), regex = "Solver cannot solve the ATSP directly")
o2 <- solve_TSP(atsp, method = "linkern", as_TSP = TRUE, verbose = v)
expect_equal(length(o1), 5L)
expect_equal(length(o2), 5L)

# Linkern should find the optimal solution of 0.8082826
expect_equal(round(tour_length(o1), 7), 0.8082826)
expect_equal(round(tour_length(o2), 7), 0.8082826)

library(TSP)
library(testthat)

context("solve_TSP")
m <- rbind(
  c(0, 1, 0, 1),
  c(1, 0, 1, Inf),
  c(0, 1, 0, 1),
  c(1, Inf, 1, 0)
)

d <- as.dist(m)
tsp <- TSP(d)
tsp

## from matrix should give the same result
expect_equal(as.numeric(tsp), as.numeric(TSP(as.matrix(d))))


## test error on NA
tsp_na <- tsp
tsp_na[4] <- NA
expect_error(o <- solve_TSP(tsp_na))

## test Inf
methods <- c("nearest_insertion", "cheapest_insertion", "farthest_insertion",
  "arbitrary_insertion", "nn", "repetitive_nn", "two_opt", "random", "identity")

tours <- lapply(methods, FUN = function(m) solve_TSP(tsp, method = m))
names(tours) <- methods
#tours

tl <- sapply(tours, attr, "tour_length")
expect_true(all(tl == 4 | tl == Inf))

## test rep
res <- solve_TSP(tsp, rep=10)
tl <- attr("res", "tour_length")
expect_true(all(tl == 4 | tl == Inf))

## no two_opt
res <- solve_TSP(tsp, two_opt=FALSE)
tl <- attr("res", "tour_length")
expect_true(all(tl == 4 | tl == Inf))

## test special case: two cities
d <- dist(rbind(c(0,0), c(1,1)))
tsp2 <- TSP(d)
tours2 <- lapply(methods, FUN = function(m) solve_TSP(tsp2, method = m))
expect_true(all(sapply(tours2, attr, "tour_length") == as.numeric(d)*2))

## test special case: one city
tsp1 <- TSP(dist(1))
tours1 <- lapply(methods, FUN = function(m) solve_TSP(tsp1, method = m))
expect_true(all(sapply(tours1, attr, "tour_length") == 0))

## test ATSP (just for errors)

#data <- matrix(runif(5^2), ncol = 5, dimnames = list(1:5, 1:5))
data <- structure(c(0.13930352916941, 0.897691324818879, 0.509101516567171,
  0.430898967897519, 0.141799068776891, 0.0334562903735787, 0.902805947931483,
  0.203576791565865, 0.435874363640323, 0.0641707226168364, 0.101683554705232,
  0.631239329231903, 0.555331876967102, 0.0829615572001785, 0.272443652851507,
  0.215095571940765, 0.532841097796336, 0.795302660670131, 0.43256876245141,
  0.582661165855825, 0.250269076088443, 0.164849652675912, 0.638499777996913,
  0.857200765516609, 0.0134391817264259), .Dim = c(5L, 5L), .Dimnames = list(
    c("1", "2", "3", "4", "5"), c("1", "2", "3", "4", "5")))
# best solution is 0.8082826

atsp <- ATSP(data)

methods <- c("nearest_insertion", "cheapest_insertion", "farthest_insertion",
  "arbitrary_insertion", "nn", "repetitive_nn", "two_opt", "random", "identity")

tours <- lapply(methods, FUN = function(m) solve_TSP(atsp, method = m))
names(tours) <- methods

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Class TSP -- Symmetric traveling salesperson problem
#'
#' Constructor to create an instance of a symmetric traveling salesperson
#' problem (TSP) and some auxiliary methods.
#'
#' Objects of class `TSP` are internally represented as `dist`
#' objects (use [as.dist()] to get the `dist` object).
#'
#' Not permissible paths can be set to a distance of `+Inf`. `NA`s are not allowed and `-Inf` will lead
#' to the algorithm only being able to find an admissible tour, but not the best one.
#'
#' @family TSP
#'
#' @param x,object an object (currently `dist` or a symmetric matrix) to
#' be converted into a `TSP` or, for the methods, an object of class
#' `TSP`.
#' @param labels optional city labels. If not given, labels are taken from
#' `x`.
#' @param method optional name of the distance metric. If `x` is a
#' `dist` object, then the method is taken from that object.
#' @param col color scheme for image.
#' @param order order of cities for the image as an integer vector or an object
#' of class [TOUR].
#' @param ... further arguments are passed on.
#' @returns
#' - `TSP()` returns `x` as an object of class `TSP`.
#' - `n_of_cities()` returns the number of cities in `x`.
#' - `labels()` returns a vector with the names of the cities in `x`.
#' @author Michael Hahsler
#' @keywords classes
#' @examples
#' data("iris")
#' d <- dist(iris[-5])
#'
#' ## create a TSP
#' tsp <- TSP(d)
#' tsp
#'
#' ## use some methods
#' n_of_cities(tsp)
#' labels(tsp)
#' image(tsp)
#' @export
TSP <- function(x, labels = NULL, method = NULL) {
  if (inherits(x, "TSP"))
    return(x)
  x <- as.TSP(x)
  if (!is.null(labels))
    attr(x, "Labels") <- labels
  if (!is.null(method))
    attr(x, "method") <- method
  x
}

## coercion
#' @rdname TSP
#' @export
as.TSP <- function(x)
  UseMethod("as.TSP")

#' @rdname TSP
#' @export
as.TSP.dist <- function(x) {
  ## make sure we have a upper triangle matrix w/o diagonal
  x <- as.dist(x, diag = FALSE, upper = FALSE)

  ## make sure we have labels
  if (is.null(attr(x, "Labels")))
    attr(x, "Labels") <- c(1:n_of_cities(x))

  if (any(is.nan(x)))
    stop(paste(sQuote("NAs"), "not supported"))

  ## make sure data is numeric
  mode(x) <- "numeric"
  class(x) <- c("TSP", class(x))
  x
}

#' @rdname TSP
#' @export
as.TSP.matrix <- function(x) {
  if (!isSymmetric(x))
    stop("TSP requires a symmetric matrix")

  method <- attr(x, "method")
  x <- as.dist(x, diag = FALSE, upper = FALSE)
  attr(x, "method") <- method

  ## make sure we have labels
  if (is.null(attr(x, "Labels")))
    attr(x, "Labels") <- c(1:n_of_cities(x))

  if (any(is.nan(x)))
    stop(paste(sQuote("NAs"), "not supported"))

  ## make sure data is numeric
  mode(x) <- "numeric"
  class(x) <- c("TSP", class(x))
  x
}

#' @rdname TSP
#' @param m a TSP object to be converted to a [dist] object.
#' @export
as.dist.TSP <- function(m, ...) {
  class(m) <- "dist"
  as.dist(m, ...)
}


## print
#' @rdname TSP
#' @export
print.TSP <- function(x, ...) {
  method <- attr(x, "method")
  if (is.null(method))
    method <- "unknown"

  cat("object of class", sQuote(class(x)[1]), "\n")
  cat(n_of_cities(x),
    "cities",
    paste("(distance ", sQuote(method), ")", sep = ""),
    "\n")
}



## generic for n_of_cities
#' @rdname TSP
#' @export
n_of_cities <- function(x)
  UseMethod("n_of_cities")

## number of cities
#' @rdname TSP
#' @export
n_of_cities.TSP <- function(x)
  attr(x, "Size")

#' @export
n_of_cities.default <- n_of_cities.TSP

## labels
#' @rdname TSP
#' @export
labels.TSP <- function(object, ...)
  attr(object, "Labels")

## image
#' @rdname TSP
#' @export
image.TSP <- function(x, order, col = gray.colors(64), ...) {
  p <- n_of_cities(x)
  if (missing(order))
    order <- 1:p

  graphics::image.default(1:p, 1:p, as.matrix(x)[order, order],
    col = col, ...)
}

library(TSP)
library(testthat)

context("ETSP")

x <- data.frame(x = runif(20), y = runif(20), row.names = LETTERS[1:20])

## create a TSP
etsp <- ETSP(x)
etsp

## use some methods
expect_equal(n_of_cities(etsp), 20L)
expect_equal(labels(etsp), LETTERS[1:20])

## solve
tour <- solve_TSP(etsp)
tour

## compare tour_length
expect_equal(tour_length(etsp), tour_length(as.TSP(etsp)))
expect_equal(tour_length(tour, etsp), tour_length(tour, as.TSP(etsp)))

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## simulated annealing


# Subtour reversal
# FIXME: when i>j then we should revert the beginning 
# and the end of the tour instead!
tsp_SA_reversal <- function(tour, x) {
  n <- length(tour)
  # Pick two random distinct positions
  pos <- sample(1:n, 2, replace = FALSE)
  i <- pos[1]
  j <- pos[2]
  
  # Swap the cities at these positions
  new_tour <- tour
  new_tour[i:j] <- rev(tour[i:j])
  return(new_tour)
}

# A simple swap operator is commonly used for TSP
tsp_SA_swap <- function(tour, x) {
  n <- length(tour)
  # Pick two random distinct positions
  pos <- sample(1:n, 2, replace = FALSE)
  i <- pos[1]
  j <- pos[2]
  
  # Swap the cities at these positions
  new_tour <- tour
  new_tour[i] <- tour[j]
  new_tour[j] <- tour[i]
  return(new_tour)
}

tsp_SA_mixed <- function(tour, x) {
  if (stats::runif(1) > .5) tsp_SA_reversal(tour, x)
  else tsp_SA_swap(tour, x)
}

tsp_SA <- function(x, control = NULL){

  control <- .get_parameters(control, list(
    tour = NULL,
    local_move = tsp_SA_reversal,
    temp = NULL, # Initial temperature
    tmax = 10, # number of evaluations per temperature step
    maxit = 10000, # Number of iterations 
    trace = 0
  ), method = "SA")

  if (control$verbose) 
    control$trace <- 1
  
  initial_tour <- as.TOUR(control$tour %||% sample(n_of_cities(x)))
  control$temp <- control$temp %||% 
    tour_length(x, initial_tour) / n_of_cities(x)
  
  if (control$trace > 0)
    cat("Initial temperature set to:", control$temp, "\n\n")
  
  cost_function <- function(tour, x) tour_length(x, as.integer(tour))
    
  ret <- stats::optim(
    par = initial_tour,
    fn = cost_function,
    gr = control$local_move,
    method = "SANN",
    x = x, # Pass additional argument to fn and gr
    control = list(
      temp = control$temp,
      tmax = control$tmax,
      maxit = control$maxit,
      trace = control$trace
    )
  )

  TOUR(as.integer(ret$par), "sa", x)
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## heuristic to improve a tour using exchanges of 2 edges.

tsp_two_opt <- function(x, control = NULL){

  control <- .get_parameters(control, list(
    tour = NULL,
    two_opt_repetitions = 1
  ), method = "two_opt")

  if (!is.null(control$tour) && control$two_opt_repetitions > 1) {
    control$two_opt_repetitions <- 1
    warning("Doing multiple repetitions of two-opt given a fixed tour does not make sense. Doing only 1 repetition.")
    }

  ## improve a given tour or create a random tour
  ## we use a function since for rep >1 we want several random
  ## initial tours
  initial <- function() {
    if(!is.null(control$tour)) as.integer(control$tour)
    else sample(n_of_cities(x))
  }

  xx <- as.matrix(x)

  if(control$two_opt_repetitions > 1) {
    tour <- replicate(control$two_opt_repetitions, .Call(R_two_opt, xx, initial()), simplify = FALSE)
    lengths <- sapply(tour, FUN = function(t) tour_length(x, t))
    cat(lengths)
    tour <- tour[[which.min(lengths)]]
  }else tour <- .Call(R_two_opt, xx, initial())

  if (control$verbose)
    cat("Two-opt done.\n\n")

  tour
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## insert a dummy city
##generic


#' Insert dummy cities into a distance matrix
#'
#' Inserts dummy cities into a TSP problem.  A
#' dummy city has the same, constant distance (0) to all other cities and is
#' infinitely far from other dummy cities. A dummy city can be used to
#' transform a shortest Hamiltonian path problem (i.e., finding an optimal
#' linear order) into a shortest Hamiltonian cycle problem which can be solved
#' by a TSP solvers (Garfinkel 1985).
#'
#' Several dummy cities can be used together with a TSP solvers to perform
#' rearrangement clustering (Climer and Zhang 2006).
#'
#' The dummy cities are inserted after the other cities in `x`.
#'
#' A `const` of 0 is guaranteed to work if the TSP finds the optimal
#' solution. For heuristics returning suboptimal solutions, a higher
#' `const` (e.g., `2 * max(x)`) might provide better results.
#'
#' @family TSP
#'
#' @param x an object with a TSP problem.
#' @param n number of dummy cities.
#' @param const distance of the dummy cities to all other cities.
#' @param inf distance between dummy cities.
#' @param label labels for the dummy cities. If only one label is given, it is
#' reused for all dummy cities.
#' @returns returns an object of the same class as `x`.
#'
#' @author Michael Hahsler
#' @references Sharlee Climer, Weixiong Zhang (2006): Rearrangement Clustering:
#' Pitfalls, Remedies, and Applications, \emph{Journal of Machine Learning
#' Research} \bold{7}(Jun), pp. 919--943.
#'
#' R.S. Garfinkel (1985): Motivation and modelling (chapter 2). In: E. L.
#' Lawler, J. K. Lenstra, A.H.G. Rinnooy Kan, D.  B. Shmoys (eds.) The
#' traveling salesman problem - A guided tour of combinatorial optimization,
#' Wiley & Sons.
#' @keywords manip
#' @examples
#' ## Example 1: Find a short Hamiltonian path
#' set.seed(1000)
#' x <- data.frame(x = runif(20), y = runif(20), row.names = LETTERS[1:20])
#'
#' tsp <- TSP(dist(x))
#'
#' ## add a dummy city to cut the tour into a path
#' tsp <- insert_dummy(tsp, label = "cut")
#' tour <- solve_TSP(tsp)
#' tour
#'
#' plot(x)
#' lines(x[cut_tour(tour, cut = "cut"),])
#'
#'
#' ## Example 2: Rearrangement clustering of the iris dataset
#' set.seed(1000)
#' data("iris")
#' tsp <- TSP(dist(iris[-5]))
#'
#' ## insert 2 dummy cities to creates 2 clusters
#' tsp_dummy <- insert_dummy(tsp, n = 3, label = "boundary")
#'
#' ## get a solution for the TSP
#' tour <- solve_TSP(tsp_dummy)
#'
#' ## plot the reordered distance matrix with the dummy cities as lines separating
#' ## the clusters
#' image(tsp_dummy, tour)
#' abline(h = which(labels(tour)=="boundary"), col = "red")
#' abline(v = which(labels(tour)=="boundary"), col = "red")
#'
#' ## plot the original data with paths connecting the points in each cluster
#' plot(iris[,c(2,3)], col = iris[,5])
#' paths <- cut_tour(tour, cut = "boundary")
#' for(p in paths) lines(iris[p, c(2,3)])
#'
#' ## Note: The clustering is not perfect!
#' @export
insert_dummy <- function(x, n = 1, const = 0, inf = Inf, label = "dummy")
  UseMethod("insert_dummy")


## use insert dummy from ATSP
#' @rdname insert_dummy
#' @export
insert_dummy.TSP <- function(x, n = 1, const = 0, inf = Inf, label = "dummy") {
    x <- insert_dummy(ATSP(x), n, const, inf, label)
    TSP(x)
}

#' @rdname insert_dummy
#' @export
insert_dummy.ATSP <- function(x, n = 1, const = 0, inf = Inf, label = "dummy") {

    method <- attr(x, "method")

    n <- as.integer(n)
    p <- n_of_cities(x)

    if(length(label) == 1 && n > 1) label = rep(label, n)

    ## add dummy rows/columns
    x <- cbind(x, matrix(const, ncol = n, nrow = p,
            dimnames = list(NULL, label)))
    x <- rbind(x, matrix(const, ncol = p+n, nrow = n,
            dimnames = list(label, NULL)))

    ## place inf between dummies
    if(n>1) {
        x[(p+1):(p+n), (p+1):(p+n)] <- inf
        diag(x[(p+1):(p+n), (p+1):(p+n)]) <- 0
    }

    attr(x, "method") <- method
    ATSP(x)
}

#' @rdname insert_dummy
#' @export
insert_dummy.ETSP <- function(x, n = 1, const = 0, inf = Inf, label = "dummy")
  stop("Dummy cities cannot be used with ETSP! Convert the problem into a TSP.")

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## nearest neighbor algorithm

tsp_nn <- function(x, control = NULL) {
    ## parameter x comes checked from solve_TSP/solve_ATSP

    n <- n_of_cities(x)

    ## we use a matrix for now (covers TSP and ATSP)
    x <- as.matrix(x)

    control <- .get_parameters(control, list(
      start = sample(n, 1),
      verbose = FALSE
    ), method = "nn")
    start <- control$start
    if(start < 0 || start > n)
      stop(paste("illegal value for", sQuote("start")))

    placed <- logical(n)
    order <- integer(n)

    ## place first city
    current <- start
    order[1L] <- current
    placed[current] <- TRUE

    while(length(rest <- which(!placed)) > 0L) {
        ## nearest <- rest[which.min(x[current,rest])]
        ## which.min has problems with Inf
        ## so we can break ties randomly now too
        x_sub <- x[current, rest]
        current <- rest[which(x_sub == min(x_sub, na.rm = TRUE))]
        if(length(current) > 1L) current <- sample(current, 1)

        ## place city
        order[n + 1L - length(rest)] <- current
        placed[current] <- TRUE
        
        if (control$verbose && (length(order) - length(rest)) %% 1000 == 0) 
          cat("\r", "Placed: ", (length(order) - length(rest)), "/", length(order), sep = "")
    }

    if (control$verbose)
      cat("\nAll cities placed.\n\n")

    order
}

## repetitive NN

tsp_repetitive_nn <- function(x, control){
  n <- n_of_cities(x)

  control <- .get_parameters(control, list(), method = "repetitive_nn")

  #tours <- lapply(1:n, function(i) tsp_nn(x, control = list(start = i)))
  ## no backend would warn!
  i <- 0L ## for R CMD check (no global binding for i)
  suppressWarnings(
    tours <- foreach(i = 1:n) %dopar% tsp_nn(x, control = list(start = i))
  )

  if (control$verbose)
    cat("All replications are complete.\n\n")

  lengths <- sapply(tours, FUN = function(i) tour_length(x, i))
  tours[[which.min(lengths)]]
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Calculate the length of a tour
#'
#' Calculate the length of a [TOUR] for a [TSP].
#'
#' If no `tsp` is specified, then the tour length stored in `x` as
#' attribute `"tour_length"` is returned.  If `tsp` is given then the
#' tour length is recalculated using the specified TSP problem.
#'
#' If a distance in the tour is infinite, the result is also infinite. If the
#' tour contains positive and negative infinite distances then the method
#' returns `NA`.
#'
#' @family TOUR
#'
#' @param x a TSP problem or a [TOUR].
#' @param order an object of class `TOUR`
#' @param tsp as TSP object.
#' @param ... further arguments are currently unused.
#' @author Michael Hahsler
#' @keywords optimize
#' @examples
#'
#' data("USCA50")
#'
#' ## original order
#' tour_length(solve_TSP(USCA50, method="identity"))
#'
#' ## length of a manually created (random) tour
#' tour <- TOUR(sample(seq(n_of_cities(USCA50))))
#' tour
#' tour_length(tour)
#' tour_length(tour, USCA50)
#' @export
tour_length <- function(x, ...)
  UseMethod("tour_length")

#' @rdname tour_length
#' @export
tour_length.TSP <- function(x, order, ...) {
  n <- n_of_cities(x)
  if (missing(order))
    order <- 1:n

  .Call(R_tour_length_dist, x, order)
}

#' @rdname tour_length
#' @export
tour_length.ATSP <- function(x, order, ...) {
  n <- n_of_cities(x)
  if (missing(order))
    order <- 1:n

  .Call(R_tour_length_matrix, x, order)
}

### faster for small n but takes O(n^2) memory
#tour_length.ETSP <- function(x, order) tour_length(as.TSP(x), order)

#' @rdname tour_length
#' @export
tour_length.ETSP <- function(x, order, ...) {
  n <- n_of_cities(x)
  if (n != nrow(x))
    stop("x and order do not have the same number of cities!")

  if (missing(order))
    order <- 1:n

  as.numeric(sum(sapply(
    1:(n - 1),
    FUN = function(i)
      dist(x[order[c(i, i + 1)], , drop = FALSE])
  )) +
      dist(x[order[c(n, 1)], , drop = FALSE]))
}

#' @rdname tour_length
#' @export
tour_length.TOUR <- function(x, tsp = NULL, ...) {
  if (is.null(tsp)) {
    len <- attr(x, "tour_length")
    if (is.null(len))
      len <- NA
    return(len)
  }

  tour_length(x = tsp, order = x)
}

#' @rdname tour_length
#' @export
tour_length.integer <- function(x, tsp = NULL, ...) {
  if (is.null(tsp))
    return(NA)

  tour_length(x = tsp, order = x)
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Cut a tour to form a path
#'
#' Cuts a tour at a specified city to form a path.
#'
#' @family TOUR
#'
#' @param x an object of class [TOUR].
#' @param cut the index or label of the city/cities to cut the tour.
#' @param exclude_cut exclude the city where we cut? If `FALSE`, the city
#' at the cut is included in the path as the first city.
#' @return Returns a named vector with city ids forming the path. If multiple
#' cuts are used then a list with paths is returned.
#' @author Michael Hahsler
#' @keywords optimize
#' @examples
#' data("USCA50")
#'
#' ## find a path starting at Austin, TX
#' tour <- solve_TSP(USCA50)
#' path <- cut_tour(tour, cut = "Austin, TX", exclude_cut = FALSE)
#' path
#'
#' ## cut the tours at two cities
#' tour <- solve_TSP(USCA50)
#' path <- cut_tour(tour, cut = c("Austin, TX", "Cambridge, MA"), exclude_cut = FALSE)
#' path
#'
#' ## cut a tour at the largest gap using a dummy city
#' tsp <- insert_dummy(USCA50, label = "cut")
#' tour <- solve_TSP(tsp)
#'
#' ## cut tour into path at the dummy city
#' path <- cut_tour(tour, "cut")
#' path
#' @export
cut_tour <- function(x, cut, exclude_cut = TRUE)
  UseMethod("cut_tour")

#' @rdname cut_tour
#' @export
cut_tour.TOUR <- function(x, cut, exclude_cut = TRUE) {
  exclude_cut <- as.integer(exclude_cut)

  ## city label
  if (is.character(cut)) {
    cut <-
      which(as.logical(apply(
        sapply(cut, "==", labels(x)), MARGIN = 1, sum
      )))
    if (length(cut) < 1)
      stop("cut has to exist")

    ## city id
  } else {
    if (any(is.na(cut <- match(cut, x))))
      stop("cut has to exist")
  }

  if (length(cut) == 1L) {
    ## single path
    if (exclude_cut && length(x) <= 1)
      path <- integer(0)
    else
      path <-
        c(x, x)[(cut + exclude_cut):(length(x) + cut - 1L)]

  } else {
    ## multiple paths, return as a list
    path <- replicate(length(cut), integer(0))
    path[[1L]] <- c(if ((tail(cut, 1) + exclude_cut) <= length(x))
      (tail(cut, 1) + exclude_cut):length(x)
      else
        NULL,
      if (cut[1] > 1)
        1:(cut[1] - 1L)
      else
        NULL)
    for (i in seq_len(length(cut) - 1L)) {
      if ((cut[i] + exclude_cut) <= (cut[i + 1L] - 1L))
        path[[i + 1L]] <-
          (cut[i] + exclude_cut):(cut[i + 1L] - 1L)
    }

    path <- lapply(
      path,
      FUN = function(i)
        x[i]
    )
    if (exclude_cut)
      names(path) <- labels(x)[cut]
  }

  path
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyright (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



#' Using the Concorde TSP Solver
#'
#' The Concorde TSP Solver package contains several solvers.  Currently,
#' interfaces to the Concorde solver (Applegate et al. 2001), one of the most
#' advanced and fastest TSP solvers using branch-and-cut, and the Chained
#' Lin-Kernighan (Applegate et al. 2003) implementation are provided in
#' \pkg{TSP}. Concorde can solve [TSP]s and [ETSP]s directly. [ATSP]s are
#' reformulated as larger TSP's and then solved.
#'
#'
#' # Installation of Concorde
#'
#' The Concorde TSP Solver is freely available for academic research.
#' It is not included in the \pkg{TSP} R package and has
#' to be obtained separately from the
#' [Concorde download page](https://www.math.uwaterloo.ca/tsp/concorde/downloads/downloads.htm).
#' Either download the precompiled executables and place them in a suitable
#' directory (make sure they are executable), or you can get the source code and
#' compile the program on your own. \pkg{TSP} needs to know where the executables are.
#' There are two options:
#' 1. use `concorde_path()` to set the path to the
#'   directory containing the executables for concorde and linkern, or
#' 2. make
#'   sure that the executables are in the search path stored in the `PATH`
#'   environment variable (see [Sys.setenv()]).
#'
#' # Using Concorde for `solve_TSP()`
#'
#' [solve_TSP()] uses [write_TSPLIB()] to write the TSP for
#' Concorde and tries to find the appropriate `precision` value (digits
#' after the decimal point) to convert the provided distances into the needed
#' integer value range. The `precision` value can also be specified in
#' `control` in [solve_TSP()] with method Concorde. Warning
#' messages will alert the user if the conversion to integer values results
#' into rounding errors that are worse then what is specified in the
#' `precision` control parameter.
#'
#' To get a list of all available command line options which can be used via
#' the `clo` option for `solve_TSP` use `concorde_help()` and
#' `linkern_help()`.  Several options (\option{-x}, \option{-o},
#' \option{-N}, \option{-Q}) are not available via [solve_TSP()] since they
#' are used by the interface.
#'
#' If Concorde takes too long, then you can interrupt [solve_TSP()] 
#' using `Esc/CTRL-C`. On most operating systems, this will also
#' terminate the Concorde executable. If Concorde keeps running, then you can 
#' kill the 'concorde' process via your operating system.
#'
#' @family TSP
#'
#' @name Concorde
#' @aliases Concorde concorde concorde_path concorde_help linkern_help
#'
#' @param path a character string with the path to the directory where the
#' executables are installed.
#' @returns `concorde_path()` returns the path to the executable. Others functions: Nothing.
#' @author Michael Hahsler
#' @references Concorde home page,
#' \url{https://www.math.uwaterloo.ca/tsp/concorde/}
#'
#' David Applegate, Robert Bixby, Vasek Chvatal, William Cook (2001): TSP cuts
#' which do not conform to the template paradigm, Computational Combinatorial
#' Optimization, M. Junger and D. Naddef (editors), Springer-Verlag.
#'
#' David Applegate and William Cook and Andre Rohe (2003): Chained
#' Lin-Kernighan for Large Traveling Salesman Problems, \emph{INFORMS Journal
#' on Computing}, \bold{15}, 82--92.
#' @keywords documentation
#' @examples
#'
#' \dontrun{
#' ## see if Concorde is correctly installed
#' concorde_path()
#'
#'
#' ## set path to the Concorde executible if it is not in the search PATH
#' ## Example:
#' ## concorde_path("~/concorde/")
#'
#' concorde_help()
#'
#' data("USCA312")
#'
#' ## run Concorde in verbose mode (-v) with fast cuts only (-V)
#' ## Note: use the contol parameter verbose = FALSE to supress Concorde's output
#' solve_TSP(USCA312, method = "concorde", control = list(clo = "-v -V"))
#' }
#'
NULL


## prepare distances as integers in the appropriate range [0..MAX]
.prepare_dist_concorde <- function(x, MAX, precision, verbose = FALSE) {
  ## handle inf
  x <- .replaceInf(x)

  ## fix neg. values
  min_x <- min(x)
  if (min_x < 0) {
    if (verbose)
    warning(
      "pTSP contains negative distances (maybe the result of a reformulation from ATSP). Shifting distances by subtracting the minimum.", immediate. = FALSE)
    x <- x - min_x
  }

  ## get max (excluding) to check for possible integer overflows
  max_x <- max(x)
  prec <- floor(log10(MAX / max_x))
  if (any((x %% 1) != 0) || prec < 0) {
    if (prec >= precision) {
      x <- x * 10 ^ precision
    } else {
      warning(
        paste0(
          "Concorde/Linken can only handle distances represented as integers. Converting the provided distances to integers with precison ",
          prec,
          ". This may lead to rounding errors."
        ),
        immediate. = TRUE
      )
      x <- x * 10 ^ prec
    } 
  }

  storage.mode(x) <-
    "integer" ## so write.TSBLIB does not do precision changes

  x
}

## interface to the Concorde algorithm
## (can only handle TSP and no neg. distances!)

tsp_concorde <- function(x, control = NULL) {
  if (!is.null(control$exe))
    warning("exe is deprecated. Use concorde_path() instead!")

  ## get parameters
  control <- .get_parameters(
    control,
    list(
      clo = "",
      exe = .find_exe(control$exe, "concorde"),
      precision = 6,
      verbose = TRUE,
      keep_files = FALSE
    )
  )



  ## get temp files and change working directory
  wd <- tempdir()
  dir <- getwd()
  setwd(wd)
  on.exit(setwd(dir))

  ### fix for Windows by Stephen Eick
  ##temp_file <- tempfile(tmpdir = wd)
  temp_file <- basename(tempfile(tmpdir = wd))

  ## file name needs to be unique
  tmp_file_in  <- paste(temp_file, ".dat", sep = "")
  tmp_file_out <- paste(temp_file, ".sol", sep = "")

  ## check x
  if (inherits(x, "TSP")) {
    #if(n_of_cities(x) < 10) MAX <- 2^15 - 1 else MAX <- 2^31 - 1
    ### MFH: Concorde may overflow with 2^31-1
    if (n_of_cities(x) < 10)
      MAX <- 2 ^ 15 - 1
    else
      MAX <- 2 ^ 28 - 1
    x <- .prepare_dist_concorde(x, MAX, precision = control$precision, verbose = control$verbose)
    ## precision is already handled!
    write_TSPLIB(x, file = tmp_file_in, precision = 0)

  } else if (inherits(x, "ETSP")) {
    ## nothing to do!
    write_TSPLIB(x, file = tmp_file_in, precision = control$precision)
  } else
    stop("Concorde only handles TSP and ETSP.")

  ## change working directory

  ## do the call and read back result
  ## we do not check return values of Concorde since they are not
  ## very consistent
  system2(
    control$exe,
    args =  paste("-x", control$clo, "-o", tmp_file_out, tmp_file_in),
    stdout = if (control$verbose)
      ""
    else
      FALSE,
    stderr = if (control$verbose)
      ""
    else
      FALSE,
  )

  if (!file.access(tmp_file_out) == 0)
    stop(
      "Concorde has not produced a result file.\nIs concorde properly installed? (see ? Concorde)\nDid Concorde finish without an error or being interupted?"
    )
  ##else cat("Concorde done.\n")

  order <- scan(tmp_file_out, what = integer(0), quiet = TRUE)
  ## remove number of nodes and add one (result starts with 0)
  order <- order[-1] + 1L

  ## tidy up
  if (!control$keep_files)
    unlink(c(tmp_file_in, tmp_file_out))
  else
    cat("File are in:", wd, "\n\n")

  order
}

## interface to the Concorde's Chained Lin-Kernighan algorithm
## (can only handle TSP, handles neg. distances)

tsp_linkern <- function(x, control = NULL) {
  if (!is.null(control$exe))
    warning("exe is deprecated. Use concorde_path() instead!")

  ## get parameters
  control <- .get_parameters(
    control,
    list(
      exe = .find_exe(control$exe, "linkern"),
      clo = "",
      precision = 6,
      verbose = TRUE,
      keep_files = FALSE
    )
  )

  ## have to set -r for small instances <8
  if (n_of_cities(x) <= 8)
    control$clo <- paste(control$clo, "-k", n_of_cities(x))

  ## check x
  if (inherits(x, "TSP")) {
    #MAX <- 2^31 - 1
    MAX <- 2 ^ 28 - 1
    x <- .prepare_dist_concorde(x, MAX, precision = control$precision, verbose = control$verbose)

  } else if (inherits(x, "ETSP")) {
    ## nothing to do
  } else
    stop("Linkern only works for TSP and ETSP.")

  ## get temp files and change working directory
  wd <- tempdir()
  dir <- getwd()
  setwd(wd)
  on.exit(setwd(dir))

  ### fix for Windows by Stephen Eick
  ##temp_file <- tempfile(tmpdir = wd)
  temp_file <- basename(tempfile(tmpdir = wd))

  ## file name needs to be unique
  tmp_file_in  <- paste(temp_file, ".dat", sep = "")
  tmp_file_out <- paste(temp_file, ".sol", sep = "")

  write_TSPLIB(x, file = tmp_file_in, precision = 0)

  ## do the call and read back result
  ## we do not check return values of Concorde since they are not
  ## very consistent
  system2(
    control$exe,
    args =  paste("-o",
      tmp_file_out, control$clo, tmp_file_in),
    stdout = if (control$verbose)
      ""
    else
      FALSE,
    stderr = if (control$verbose)
      ""
    else
      FALSE
  )

  if (!file.access(tmp_file_out) == 0)
    stop(
      "Linkern has not produced a result file.\nIs linkern properly installed?\nDid linkern finish without an error or being interrupted?"
    )
  ##else cat("Concorde done.\n")

  order <- read.table(tmp_file_out)[, 1]
  ## remove number of nodes and add one (result starts with 0)
  order <- order + as.integer(1)

  ## tidy up
  if (!control$keep_files)
    unlink(c(tmp_file_in, tmp_file_out))
  else
    cat("File are in:", wd, "\n\n")

  order
}

## path
## path
#' @rdname Concorde
#' @export
concorde_path <- local({
  .path <- NULL
  function(path) {
    if (missing(path)) {
      if (!is.null(.path))
        return(.path)
      else {
        ## find concorde and/or linkern
        p <- dirname(Sys.which("concorde"))
        if (p == "")
          p <- dirname(Sys.which("linkern"))
        if (p == "")
          stop(
            "Can not find executables for concorde or linkern. Please install the executables or set path manually."
          )
        return(p)
      }
    } else {
      if (!is.null(path)) {
        path <- normalizePath(path) ### translate all special characters
        ex <- c(
          list.files(path, pattern = "concorde",
            ignore.case = TRUE),
          list.files(path, pattern = "linkern",
            ignore.case = TRUE)
        )
        if (length(ex) < 1)
          stop(paste("no executable (concorde and/or linkern) found in",
            path))
        cat("found:", ex, "\n")
      }
      .path <<- path

      invisible(.path)

    }
  }
})


## get help page
#' @rdname Concorde
#' @export
concorde_help <- function() {
  cat(
    "The following options can be specified in solve_TSP with method \"concorde\" using clo in control:\n\n"
  )
  system2(.find_exe(NULL, "concorde"), args = "")
}

#' @rdname Concorde
#' @export
linkern_help <- function() {
  cat(
    "The following options can be specified in solve_TSP with method \"linkern\" using clo in control:\n\n"
  )
  system2(.find_exe(NULL, "linkern"), args = "")
}



## helper to find the 'concorde' executable
.find_exe <- function(exe = NULL, prog) {
  ## if not specified
  if (is.null(exe)) {
    ## was the path set ?
    if (!is.null(concorde_path()))
      exe <-
        paste(concorde_path(), .Platform$file.sep, prog, sep = "")
    ## no, so it must be in the systems execution path
    else
      exe <- prog
  }
  exe
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## create a (asymmetric) ATSP problem


#' Class ATSP -- Asymmetric traveling salesperson problem
#'
#' Constructor to create an instance of the asymmetric traveling salesperson
#' problem (ATSP) and some auxiliary methods.
#'
#' Objects of class `ATSP` are internally represented by a matrix (use
#' `as.matrix()` to get just the matrix).
#'
#' ATSPs can be transformed into (larger) symmetric TSPs using
#' [reformulate_ATSP_as_TSP()].
#'
#' @family TSP
#'
#' @param x,object an object (a square matrix) to be converted into an
#' `ATSP` or, for the methods, an object of class `ATSP`.
#' @param labels optional city labels. If not given, labels are taken from
#' `x`.
#' @param method optional name of the distance metric.
#' @param col color scheme for image.
#' @param order order of cities as an integer vector or an object of class
#' `TOUR`.
#' @param ... further arguments are passed on.
#' @returns
#' - `ATSP()` returns `x` as an object of class `ATSP`.
#' - `n_of_cities()` returns the number of cities in `x`.
#' - `labels()` returns a vector with the names of the cities in `x`.
#' @author Michael Hahsler
#' @keywords classes
#' @examples
#' data <- matrix(runif(10^2), ncol = 10, dimnames = list(1:10, 1:10))
#'
#' atsp <- ATSP(data)
#' atsp
#'
#' ## use some methods
#' n_of_cities(atsp)
#' labels(atsp)
#'
#' ## calculate a tour
#' tour <- solve_TSP(atsp, method = "nn")
#' tour
#'
#' tour_length(tour)
#'
#' image(atsp, tour)
#' @export
ATSP <- function(x, labels = NULL, method = NULL) {
    if(inherits(x, "ATSP")) return(x)

    atsp <- as.ATSP(x)

    if(!is.null(labels)) dimnames(atsp) <- list(labels, labels)
    if(!is.null(method)) attr(atsp, "method") <- method

    atsp
}

#' @rdname ATSP
#' @export
as.ATSP <- function(x) UseMethod("as.ATSP")

#' @rdname ATSP
#' @export
as.ATSP.matrix <- function(x){
    .isSquare <- function(x) (dim(x)[1] == dim(x)[2])

    if(!.isSquare(x)) stop("ATSP requires a square matrix")

    ## check for NAs
    if(any(is.nan(x))) stop(paste(sQuote("NAs"), "not supported"))

    ## make sure we have labels
    if(is.null(dimnames(x)))
        dimnames(x) <- list(1:dim(x)[1], 1: dim(x)[1])
    if(is.null(colnames(x)))  colnames(x) <- rownames(x)
    if(is.null(rownames(x)))  rownames(x) <- colnames(x)

    ## make sure data is numeric
    mode(x) <- "numeric"
    class(x) <- c("ATSP", class(x))

    x
}

#' @rdname ATSP
#' @export
as.ATSP.dist <- function(x){
    method <- attr(x, "method")
    x <- as.ATSP(as.matrix(x))

    ## make sure data is numeric
    mode(x) <- "numeric"
    class(x) <- c("ATSP", class(x))

    attr(x, "method") <- method
    x
}


## print
#' @rdname ATSP
#' @export
print.ATSP <- function(x, ...) {
    method <- attr(x, "method")
    if(is.null(method)) method <- "unknown"

    cat("object of class", sQuote(class(x)[1]), " (asymmetric TSP) \n")
    cat(n_of_cities(x), "cities",
        paste("(distance ", sQuote(method),")", sep=""), "\n")
}


## number of cities
#' @rdname ATSP
#' @export
n_of_cities.ATSP <- function(x) nrow(x)

## labels
#' @rdname ATSP
#' @export
labels.ATSP <- function(object, ...) dimnames(object)[[1]]

## image
#' @rdname ATSP
#' @export
image.ATSP <- function(x, order, col = gray.colors(64), ...) {
    p <- n_of_cities(x)
    if(missing(order)) order <- 1:p

    graphics::image.default(1:p, 1:p, x[order, order], col = col, ...)
}

## coerce to matrix
#' @rdname ATSP
#' @export
as.matrix.ATSP <- function(x, ...){
    unclass(x)
}

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## Insertion algorithms
## (Rosenkrantz et al. 1977)

tsp_insertion <- function(x, type = "nearest", control = NULL){

  ## since sample has an annoying convenience feature for
  ## length(x) == 1
  choose1 <- function(x) if(length(x) > 1L) sample(x, 1L) else x

  ## this is slower than which.min and which.max but works also
  ## correctly for only values Inf in x and breaks ties randomly
  choose1_min <- function(x) choose1(which(x == min(x)))
  choose1_max <- function(x) choose1(which(x == max(x)))

  types <- c("nearest", "farthest", "cheapest", "arbitrary")
  type_num <- pmatch(type, types)
  if(is.na(type_num)) stop(paste("Unknown insertion type: ", sQuote(type)))

  ## x comes checked form solve_TSP/solve_ATSP
  n <- n_of_cities(x)

  ## we use a matrix for now (covers TSP and ATSP)
  asym <- inherits(x, "ATSP")
  x <- as.matrix(x)

  ## place first city
  control <- .get_parameters(control, list(
      start = sample(n, 1),
      verbose = FALSE
    ), method = paste0(types[type_num], "_insertion"))
  start <- as.integer(control$start)
  if(start < 0 || start > n)
    stop(paste("illegal value for", sQuote("start")))

  placed <- logical(n)
  placed[start] <- TRUE
  order <- c(start)
  
  ## place other cities
  while(any(placed == FALSE)) {

    ## find city to be inserted
    ks <- which(!placed)
    js <- which(placed)

    if (control$verbose && length(js) %% 100 == 0) 
      cat("\r", "Placed: ", length(js), "/", length(placed), sep = "")
    
    ## nearest / farthest
    if(type_num < 3) {
      m <- x[ks,js, drop = FALSE]
      
      ## for the asymmetric case we have to take distances
      ## from and to the city into account
      if(asym){
        m <- cbind(m, t(x)[ks,js, drop = FALSE])
      }

      ds <- apply(m, MARGIN = 1, min)

      ## nearest/farthest insertion
      winner_index <- if(type_num == 1) choose1_min(ds)
      else choose1_max(ds)

      k <- ks[winner_index]
    }

    ## cheapest
    else if(type_num == 3) {
      winner_index <- choose1_min(sapply(ks, FUN =
          function(k) min(.Call(R_insertion_cost, x, order, k)) ))
      k <- ks[winner_index]

      ## we look for the optimal insertion place for k again later
      ## this is not necessary, but it is more convenient
      ## to reuse the code for the other insertion algorithms for now.
    }

    ## random
    else if(type_num == 4) k <- choose1(ks)

    ## just in case
    else stop("unknown insertion type")

    ## do insertion
    placed[k] <- TRUE

    if(length(order) == 1) order <- append(order, k)
    else {
      pos <- choose1_min(.Call(R_insertion_cost, x, order, k))
      order <- append(order, k, after = pos)
    }
  }

  if (control$verbose)
    cat("\nAll cities placed.\n\n")

  order
}

### faster arbitrary insertion (random sampling takes care of breaking ties)
tsp_insertion_arbitrary <- function(x, control = NULL){
  ## x comes checked form solve_TSP/solve_ATSP
  n <- n_of_cities(x)

  control <- .get_parameters(control, list(), method = "arbitrary_insertion")

  ## we use a matrix for now (covers TSP and ATSP)
  x <- as.matrix(x)

  ## deal with special cases
  if(nrow(x) == 1) return(1L)
  if(nrow(x) == 2) return(sample(1:2))

  x[is.na(x)] <- Inf

  ## random order
  rorder <- sample(n)
  x <- x[rorder, rorder]

  ## FIXME: specify start city

  ## place first two cities
  order <- integer(n)
  order[1:2] <- 1:2

  ## place other cities
  for(i in 3:n) {
    pos <- which.min(.Call(R_insertion_cost, x, order[1:(i-1L)], i)) + 1L
    order[((pos):i)+1L] <- order[(pos):i]
    order[pos] <- i
    if (control$verbose && i %% 100 == 0) {
          cat("\r", "Placed: ", i, "/", length(order), sep = "")
    }
  }

  if (control$verbose)
    cat("\nAll cities placed.\n\n")

  rorder[order]
}

#' @keywords internal
#'
#' @section Key functions:
#' - [solve_TSP()]
#'
#' @importFrom stats as.dist dist
#' @importFrom utils read.table write.table head tail
#' @importFrom grDevices gray.colors
#' @importFrom graphics image.default plot polygon
#' @importFrom foreach foreach "%dopar%"
#'
#' @useDynLib TSP, .registration=TRUE
"_PACKAGE"

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyright (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Reformulate a ATSP as a symmetric TSP
#'
#' A ATSP can be formulated as a symmetric TSP by doubling the number of cities
#' (Jonker and Volgenant 1983). The solution of the TSP also represents the
#' solution of the original ATSP.
#'
#' To reformulate a [ATSP] as a [TSP], for each city a dummy city (e.g, for 'New
#' York' a dummy city 'New York*') is added. Between each city and its
#' corresponding dummy city a very small (or negative) distance with value
#' `cheap` is used.
#' To ensure that the solver places each cities always occurs in the
#' solution together with its dummy city, this cost has to be much smaller than
#' the distances in the TSP.
#' The original distances are used
#' between the cities and the dummy cities, where each city is responsible for
#' the distance going to the city and the dummy city is responsible for the
#' distance coming from the city. The distances between all cities and the
#' distances between all dummy cities are set to `infeasible`, a very
#' large value which prevents the solver from using these links.
#' We use infinite values here and [solve_TSP()] treats them appropriately.
#'
#' `filter_ATSP_as_TSP_dummies()` can be used to extract the solution for the original
#' ATSP from the tour found for an ATSP reformulated as a TSP. Note that the symmetric TSP
#' tour does not reveal the direction for the ATSP. The filter function computed the
#' tour length for both directions and returns the shorter tour.
#'
#' [solve_TSP()] has a parameter `as_TSP` which preforms the reformulation and
#' filtering the dummy cities automatically.
#'
#' **Note on performance:** Doubling the problem size is a performance issue especially
#' has a negative impact on solution quality for heuristics. It should only be used
#' together with Concorde when the optimal solution is required. Most heuristics can solve
#' ATSPs directly with good solution quality.
#' @family TSP
#'
#' @param x an [ATSP].
#' @param infeasible value for infeasible connections.
#' @param cheap value for distance between a city and its corresponding dummy
#' city.
#' @param tour a [TOUR] created for a ATSP reformulated as a TSP.
#' @param atsp the original [ATSP].
#' @return
#' `reformulate_ATSP_as_TSP()` returns a [TSP] object.
#' `filter_ATSP_as_TSP_dummies()` returns a [TOUR] object.
#' @author Michael Hahsler
#' @references Jonker, R. and Volgenant, T. (1983): Transforming asymmetric
#' into symmetric traveling salesman problems, _Operations Research
#' Letters,_ 2, 161--163.
#' @keywords optimize
#' @examples
#' data("USCA50")
#'
#' ## set the distances from anywhere to Austin to zero which makes it an ATSP
#' austin <- which(labels(USCA50) == "Austin, TX")
#' atsp <- as.ATSP(USCA50)
#' atsp[, austin] <- 0
#' atsp
#'
#' ## reformulate as a TSP (by doubling the number of cities with dummy cities marked with *)
#' tsp <- reformulate_ATSP_as_TSP(atsp)
#' tsp
#'
#' ## create tour for the TSP. You should use Concorde to find the optimal solution.
#' # tour_tsp <- solve_TSP(tsp, method = "concorde")
#' # The standard heuristic is bad for this problem. We use it here because
#' #   Concord may not be installed.
#' tour_tsp <- solve_TSP(tsp)
#' head(labels(tour_tsp), n = 10)
#' tour_tsp
#' # The tour length is -Inf since it includes cheap links
#' #  from a city to its dummy city.
#'
#' ## get the solution for the original ATSP by filtering out the dummy cities.
#' tour_atsp <- filter_ATSP_as_TSP_dummies(tour_tsp, atsp = atsp)
#' tour_atsp
#' head(labels(tour_atsp), n = 10)
#'
#' ## This process can also be done automatically by using as_TSP = TRUE:
#' # solve_TSP(atsp, method = "concorde", as_TSP = TRUE)
#'
#' ## The default heuristic can directly solve ATSPs with results close to the
#' #  optimal solution of 12715.
#' solve_TSP(atsp, control = list(rep = 10))
#' @export
reformulate_ATSP_as_TSP <-
  function(x,
    infeasible = Inf,
    cheap = -Inf) {
    if (!inherits(x, "ATSP"))
      stop("x is not an ATSP object!")

    method <- attr(x, "method")
    m <- as.matrix(x)

    ### check all but the diagonal for cheap!
    if (cheap >= min(m[-seq(1, nrow(m)^2, nrow(m)+1)]))
      stop("cheap needs to be strictly smaller than the smallest distance in the ATSP!")
    if (infeasible < max(m))
      stop("infeasible needs to be larger than the largest distance in the ATSP!")

    ## scale matrix and add cheap links
    diag(m) <- cheap

    tsp <- rbind(cbind(matrix(
      infeasible, ncol = ncol(m), nrow = nrow(m)
    ), t(m)),
      cbind(m, matrix(
        infeasible, ncol = nrow(m), nrow = ncol(m)
      )))

    ## create labels (* for virtual cities)
    lab <- c(labels(x), paste(labels(x), "*", sep = ""))
    dimnames(tsp) <- list(lab, lab)
    attr(tsp, "method") <- method

    ## return as TSP
    TSP(tsp)
  }

#' @rdname reformulate_ATSP_as_TSP
#' @export
filter_ATSP_as_TSP_dummies <- function(tour, atsp) {

  ### Note: the tour can be in reverse order.
  t1 <-
    TOUR(tour[as.integer(tour) <= n_of_cities(atsp)], method = attr(tour, "method"), tsp = atsp)
  t2 <-
    TOUR(rev(tour[as.integer(tour) <= n_of_cities(atsp)]),
      method = attr(tour, "method"),
      tsp = atsp)

  if (attr(t1, "tour_length") > attr(t2, "tour_length"))
    t2
  else
    t1
}

library("testthat")
library("TSP")

data("USCA50")

context("insert_dummy")

tsp <- insert_dummy(USCA50, label = "cut")
#labels(tsp)
expect_equal(n_of_cities(tsp), n_of_cities(USCA50) + 1L)
expect_equal(labels(tsp), c(labels(USCA50), "cut"))

tsp5 <- insert_dummy(USCA50, n = 5, label = "cut")

#labels(tsp5)
expect_equal(n_of_cities(tsp5), n_of_cities(USCA50) + 5L)
expect_equal(labels(tsp5), c(labels(USCA50), rep("cut", 5)))

context("cut_tour")

tour <- solve_TSP(tsp5)
path <- cut_tour(tour, "cut")
expect_equal(sum(sapply(path, length)), n_of_cities(USCA50))

#labels(tour)
#path

## border cases
tour <- TOUR(1:10)
expect_equal(cut_tour(tour, "1"), 2:10)
expect_equal(cut_tour(tour, 1), 2:10)
expect_equal(cut_tour(tour, "10"), 1:9)
expect_equal(cut_tour(tour, 10), 1:9)

path <- cut_tour(tour, 1:3)
expect_equal(length(path), 3L)
expect_equal(sum(sapply(path, length)), length(tour) - 3L)
#path

path <- cut_tour(tour, 1:3, exclude_cut = FALSE)
expect_equal(length(path), 3L)
expect_equal(sum(sapply(path, length)), length(tour))
#path

path <- cut_tour(tour, 8:10)
expect_equal(length(path), 3L)
expect_equal(sum(sapply(path, length)), length(tour) - 3L)
#path

path <- cut_tour(tour, 8:10, exclude_cut = FALSE)
expect_equal(length(path), 3L)
expect_equal(sum(sapply(path, length)), length(tour))
#path

tour <- TOUR(1)
path <- cut_tour(tour, 1)
expect_equal(length(path), 0L)
#path

path <- cut_tour(tour, 1, exclude_cut = FALSE)
expect_equal(length(path), 1L)
#path

context("reformulate")
## reformulate
atsp <- as.ATSP(USCA50)
expect_equal(tour_length(USCA50), tour_length(atsp))

tsp <- reformulate_ATSP_as_TSP(atsp, cheap = 0)
expect_equal(n_of_cities(atsp)*2, n_of_cities(tsp))

## only Concorde guarantees to find the optimal solution
skip_if_not(
  Sys.which("concorde") != "" &&
    Sys.which("linkern") != "",
  message = "skipped test for concorde/linkern. Not installed.")

v <- FALSE

tour_tsp <- solve_TSP(tsp, method = "concorde", verbose = v)
tour_atsp <- filter_ATSP_as_TSP_dummies(tour_tsp, atsp)

expect_equal(length(tour_atsp), n_of_cities(USCA50))
expect_equal(tour_length(tour_tsp), tour_length(tour_atsp))

library(TSP)
library(testthat)

context("TSPLIB")

set.seed(1234)
x <- data.frame(x=runif(5), y=runif(5))

## create TSP, ATSP and ETSP (2D)
d <- round(dist(x), 3)

## TSP
tsp <- TSP(d)
tsp

write_TSPLIB(tsp, file="example.tsp", precision = 6)
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp", precision = 6)
expect_equivalent(tsp, r)

## ATSP
atsp <- ATSP(d)
atsp

write_TSPLIB(atsp, file="example.tsp", precision = 6)
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp", precision = 6)
expect_equivalent(atsp, r)

## ETSP (2D)
etsp <- ETSP(round(x[,1:2], 3))
etsp

write_TSPLIB(etsp, file="example.tsp", precision = 6)
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp", precision = 6)
expect_equivalent(etsp, r)

## Infinity
d[2] <- Inf
tsp <- TSP(d)
write_TSPLIB(tsp, file="example.tsp", precision = 6)
r <- read_TSPLIB("example.tsp", precision = 6)
expect_equivalent(tsp[-2], r[-2])
expect_gt(r[2], range(tsp, finite = TRUE)[2])

## ATT
writeLines(c(
  "NAME: ATT_EXAMPLE",
  "TYPE: TSP",
  "DIMENSION: 4",
  "EDGE_WEIGHT_TYPE: ATT",
  "NODE_COORD_SECTION",
  "1 0 0",
  "2 3 4",
  "3 6 8",
  "4 9 12",
  "EOF"
), con = "att-example.tsp")

r <- read_TSPLIB("att-example.tsp")
expected <- matrix(c(
  0, 2, 4, 5,
  2, 0, 2, 4,
  4, 2, 0, 2,
  5, 4, 2, 0
), nrow = 4, byrow = TRUE)
expected <- TSP(expected, labels = as.character(1:4), method = "ATT")
expect_equivalent(r, expected)

## GEO
writeLines(c(
  "NAME: GEO_EXAMPLE",
  "TYPE: TSP",
  "DIMENSION: 3",
  "EDGE_WEIGHT_TYPE: GEO",
  "NODE_COORD_SECTION",
  "1 48.12 16.22",
  "2 46.38 14.18",
  "3 48.18 14.17",
  "EOF"
), con = "geo-example.tsp")

r <- read_TSPLIB("geo-example.tsp")
expected <- matrix(c(
  0, 234, 155,
  234, 0, 186,
  155, 186, 0
), nrow = 3, byrow = TRUE)
expected <- TSP(expected, labels = as.character(1:3), method = "GEO")
expect_equivalent(r, expected)

## GEO with indented coordinates
writeLines(c(
  "NAME: GEO_INDENTED",
  "TYPE: TSP",
  "DIMENSION: 3",
  "EDGE_WEIGHT_TYPE: GEO",
  "NODE_COORD_SECTION",
  " 1 48.12 16.22",
  " 2 46.38 14.18",
  " 3 48.18 14.17",
  " EOF"
), con = "geo-example.tsp")

r <- read_TSPLIB("geo-example.tsp")
expected <- TSP(expected, labels = as.character(1:3), method = "GEO")
expect_equivalent(r, expected)

## clean up
unlink("example.tsp")
unlink("att-example.tsp")
unlink("geo-example.tsp")

#######################################################################
# TSP - Traveling Salesperson Problem
# Copyright (C) 2011 Michael Hahsler and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Class ETSP -- Euclidean traveling salesperson problem
#'
#' Constructor to create an instance of a Euclidean traveling salesperson
#' problem (TSP) represented by city coordinates and some auxiliary methods.
#'
#' Objects of class `ETSP` are internally represented as a `matrix`
#' objects (use `as.matrix()` to get the `matrix` object).
#'
#' @family TSP
#'
#' @param x,object an object (data.frame or matrix) to be converted into a
#' `ETSP` or, for the methods, an object of class `ETSP`.
#' @param labels optional city labels. If not given, labels are taken from
#' `x`.
#' @param col color scheme for image.
#' @param order order of cities for the image as an integer vector or an object
#' of class [TOUR].
#' @param tour,y a tour to be visualized.
#' @param tour_lty,tour_col line type and color for tour.
#' @param labels logical; plot city labels.
#' @param ... further arguments are passed on.
#' @returns
#' - `ETSP()` returns `x` as an object of class `ETSP`.
#' - `n_of_cities()` returns the number of cities in `x`.
#' - `labels()` returns a vector with the names of the cities in `x`.
#' @author Michael Hahsler
#' @keywords classes
#' @examples
#' ## create a random ETSP
#' n <- 20
#' x <- data.frame(x = runif(n), y = runif(n), row.names = LETTERS[1:n])
#' etsp <- ETSP(x)
#' etsp
#'
#' ## use some methods
#' n_of_cities(etsp)
#' labels(etsp)
#'
#' ## plot ETSP and solution
#' tour <- solve_TSP(etsp)
#' tour
#'
#' plot(etsp, tour, tour_col = "red")
#'
#' # plot with custom labels
#' plot(etsp, tour, tour_col = "red", labels = FALSE)
#' text(etsp, paste("City", rownames(etsp)), pos = 1)
#' @export
ETSP <- function(x, labels = NULL) {
  if(inherits(x, "ETSP")) return(x)
  x <- as.ETSP(x)
  if(!is.null(labels)) rownames(x) <- labels
  x
}

## coercion
#' @rdname ETSP
#' @export
as.ETSP <- function(x) UseMethod("as.ETSP")


#' @rdname ETSP
#' @export
as.ETSP.matrix <- function(x){
  mode(x) <- "numeric"
  if(is.null(rownames(x))) rownames(x) <- 1:nrow(x)

  class(x) <- c("ETSP", class(x))
  x
}

#' @rdname ETSP
#' @export
as.ETSP.data.frame <- function(x){
  as.ETSP(as.matrix(x))
}

#' @rdname ETSP
#' @export
as.TSP.ETSP <- function(x) TSP(dist(x))

#' @rdname ETSP
#' @export
as.matrix.ETSP <- function(x, ...) {
  unclass(x)
}

## print
#' @rdname ETSP
#' @export
print.ETSP <- function(x, ...) {
  cat("object of class", sQuote(class(x)[1]), "\n")
  cat(n_of_cities(x), "cities", "(Euclidean TSP)\n")
}


## number of cities
#' @rdname ETSP
#' @export
n_of_cities.ETSP <- function(x) nrow(x)

## labels
#' @rdname ETSP
#' @export
labels.ETSP <- function(object, ...) rownames(object)

## image
#' @rdname ETSP
#' @export
image.ETSP <- function(x, order, col = gray.colors(64), ...) {
  p <- n_of_cities(x)
  if(missing(order)) order <- 1:p

  x <- as.TSP(x)

  graphics::image.default(1:p, 1:p, as.matrix(x)[order, order],
    col = col, ...)
}

#' @rdname ETSP
#' @importFrom graphics text
#' @export
plot.ETSP <- function(x, y = NULL, tour = NULL, tour_lty = 2, tour_col = 2, labels = TRUE, ...) {
  x <- as.matrix(x)
  plot(x, y = NULL, ...)
  if(!is.null(y)) tour <- TOUR(y)
  if(!is.null(tour)) polygon(x[tour,], lty = tour_lty, border = tour_col)
  if(labels) text(x, y = NULL, labels = rownames(x), pos = 3)
}

#' USCA312/USCA50 -- 312/50 cities in the US and Canada
#'
#' The `USCA312` dataset contains the distances between 312 cities in the
#' US and Canada as an object of class `TSP`.  `USCA50` is a subset
#' of `USCA312` containing only the first 50 cities.
#'
#' The `USCA312_GPS` dataset contains the location (long/lat) of the 312
#' cities.
#'
#'
#' @name USCA
#' @aliases USCA312 USCA312_GPS USCA50
#' @docType data
#' @format `USCA312` and `USCA50` are objects of class `TSP`.
#' `USCA312_GPS` is a data.frame with city name, long and lat.
#' @author Michael Hahsler
#' @source John Burkardt, CITIES -- City Distance Datasets, Florida State
#' University, Department of Scientific Computing
#' @keywords datasets
#' @examples
#' data("USCA312")
#'
#' ## calculate a tour
#' tour <- solve_TSP(USCA312)
#' tour
#'
#' # Visualize the tour if package maps is installed
#' if(require("maps")) {
#'
#'   library(maps)
#'   data("USCA312_GPS")
#'   head(USCA312_GPS)
#'
#'   plot((USCA312_GPS[, c("long", "lat")]), cex = .3)
#'   map("world", col = "gray", add = TRUE)
#'   polygon(USCA312_GPS[, c("long", "lat")][tour,], border = "red")
#' }
NULL

%\documentclass[10pt,a4paper,fleqn]{article}
\documentclass[nojss]{jss}

\usepackage[english]{babel}

%% my packages
%\usepackage{a4wide}
%\usepackage[round,longnamesfirst]{natbib}
%\usepackage{hyperref}
%
\usepackage{amsmath}
\usepackage{amsfonts}
%
%\newcommand{\strong}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\class}[1]{\mbox{\textsf{#1}}}
\newcommand{\func}[1]{\mbox{\texttt{#1()}}}
%\newcommand{\code}[1]{\mbox{\texttt{#1}}}
%\newcommand{\pkg}[1]{\strong{#1}}
\newcommand{\samp}[1]{`\mbox{\texttt{#1}}'}
%\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\set}[1]{\mathcal{#1}}

%\usepackage{Sweave}
%% \VignetteIndexEntry{Introduction to TSP}

\author{Michael Hahsler\\Southern Methodist University \And
    Kurt Hornik\\Wirtschaftsuniversit\"at Wien}
\title{\pkg{TSP} -- Infrastructure for the Traveling
	Salesperson Problem}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Michael Hahsler, Kurt Hornik} %% comma-separated
\Plaintitle{TSP -- Infrastructure for the Traveling
	        Salesperson Problem} %% without formatting
\Shorttitle{Infrastructure for the TSP} %% a short title (if necessary)

%% an abstract and keywords
\Abstract{ The traveling salesperson problem (also known as traveling salesman problem or TSP) is a
well known and important combinatorial optimization problem.  The goal
is to find the shortest tour that visits each city in a given list
exactly once and then returns to the starting city.  Despite this
simple problem statement, solving the TSP is difficult since it
belongs to the class of NP-complete problems.  The importance of the
TSP arises besides from its theoretical appeal from the variety of its
applications.  Typical applications in operations research include
vehicle routing, computer wiring, cutting wallpaper and job
sequencing.  The main application in statistics is combinatorial data
analysis, e.g., reordering rows and columns of data matrices or
identifying clusters.  In this paper we introduce the
\proglang{R}~package \pkg{TSP} which provides a basic infrastructure
for handling and solving the traveling salesperson problem.  The
package features S3 classes for specifying a TSP and its (possibly
optimal) solution as well as several heuristics to find good
solutions. In addition, it provides an interface to \emph{Concorde},
one of the best exact TSP solvers currently available.  }
\Keywords{combinatorial optimization, traveling salesman problem, \proglang{R}}
\Plainkeywords{combinatorial optimization, traveling salesman problem, R} %% without formatting
%% The address of (at least) one author should be given
%% in the following format:
\Address{
      Michael Hahsler\\
	  Engineering Management, Information, and Systems\\
	  Lyle School of Engineering\\
	  Southern Methodist University\\
	  P.O. Box 750123 \\
	  Dallas, TX 75275-0123\\
	  E-mail: \email{mhahsler@lyle.smu.edu}\\

      Kurt Hornik\\
	  Department of Finance, Accounting and Statistics\\
	  Wirtschaftsuniversit\"at Wien\\
	  1090 Wien, Austria\\
	  E-mail: \email{kurt.hornik@wu-wien.ac.at}
}



\begin{document}

<<echo=FALSE>>=
options(width = 75, useFancyQuotes=FALSE, prompt="R> ")
### for sampling
set.seed(1234)
@

%\title{Introduction to \pkg{TSP} -- Infrastructure for the Traveling
%    Salesperson Problem}
%\author{Michael Hahsler and Kurt Hornik}
\maketitle
\sloppy

%\abstract{
%  The traveling salesperson (or, salesman) problem (TSP) is a
%  well known and important combinatorial optimization problem.  The goal
%  is to find the shortest tour that visits each city in a given list
%  exactly once and then returns to the starting city.  Despite this
%  simple problem statement, solving the TSP is difficult since it
%  belongs to the class of NP-complete problems.  The importance of the
%  TSP arises besides from its theoretical appeal from the variety of its
%  applications.  Typical applications in operations research include
%  vehicle routing, computer wiring, cutting wallpaper and job
%  sequencing.  The main application in statistics is combinatorial data
%  analysis, e.g., reordering rows and columns of data matrices or
%  identifying clusters.  In this paper we introduce the
%  \proglang{R}~package \pkg{TSP} which provides a basic infrastructure
%  for handling and solving the traveling salesperson problem.  The
%  package features S3 classes for specifying a TSP and its (possibly
%  optimal) solution as well as several heuristics to find good
%  solutions. In addition, it provides an interface to \emph{Concorde},
%  one of the best exact TSP solvers currently available.  }

\section{Introduction}

The traveling salesperson problem~\citep[TSP;][]{Lawler1985, Gutin2002}
is a well known and important combinatorial optimization problem. The
goal is to find the shortest tour that visits each city in a given list
exactly once and then returns to the starting city.  Formally, the TSP
can be stated as follows.  The distances between $n$ cities are stored
in a distance matrix $\mathbf{D}$ with elements $d_{ij}$ where $i,j =
1\dots n$ and the diagonal elements $d_{ii}$ are zero.  A \emph{tour}
can be represented by a cyclic permutation $\pi$ of $\{1, 2,\dots, n\}$
where $\pi(i)$ represents the city that follows city $i$ on the
tour. The traveling salesperson problem is then the optimization problem
to find a permutation $\pi$ that minimizes the \emph{length of the tour}
denoted by

\begin{equation}
    \sum_{i=1}^n d_{i\pi(i)}.
\end{equation}
% see Lenstra & Kan 1975: Some simple Applications to the TSP

For this minimization task, the tour length of $(n-1)!$ permutation
vectors have to be compared.  This results in a problem which is very
hard to solve and in fact known to be NP-complete~\citep{Johnson1985a}.
However, solving TSPs is an important part of applications in many areas
including vehicle routing, computer wiring, machine sequencing and
scheduling, frequency assignment in communication
networks~\citep{Lenstra1975, Punnen2002}.
%and structuring of
%matrices~\citep{Lenstra1975, Punnen2002}.
Applications in statistical data analysis include ordering and
clustering objects.  For example, data analysis applications in
psychology ranging from profile smoothing to finding an order in
developmental data are presented by~\cite{Hubert1978}.  Clustering and
ordering using TSP solvers is currently becoming popular in
biostatistics.  For example, \cite{Ray2007} describe an application for
ordering genes and \cite{Johnson2006} use a TSP solver for clustering
proteins.

In this paper we give a very brief overview of the TSP and introduce the
\proglang{R}~package \pkg{TSP} which provides an infrastructure for
handling and solving TSPs.  The paper is organized as follows.  In
Section~\ref{sec:TSP} we briefly present important aspects of the TSP
including different problem formulations and approaches to solve TSPs.
In Section~\ref{sec:infrastructure} we give an overview of the
infrastructure implemented in \pkg{TSP} and the basic usage.  In
Section~\ref{sec:examples}, several examples are used to illustrate the
package's capabilities.  Section~\ref{sec:conclusion} concludes the
paper.

A previous version of this manuscript was published in the \emph{Journal
  of Statistical Software} \citep{Hahsler+Hornik2007}.

\section{Theory}
\label{sec:TSP}

In this section, we briefly summarize some aspects of the TSP which are
important for the implementation of the \pkg{TSP} package described in
this paper.  For a complete treatment of all aspects of the TSP, we
refer the interested reader to the classic book edited by
\cite{Lawler1985} and the more modern book edited by \cite{Gutin2002}.

It has to be noted that in this paper, following the origin of the TSP,
the term \emph{distance} is used.  Distance is used here interchangeably
with dissimilarity or cost and, unless explicitly stated, no
restrictions to measures which obey the triangle inequality are made.
An important distinction can be made between the symmetric TSP and the
more general asymmetric TSP.  For the symmetric case (normally referred
to as just \emph{TSP}), for all distances in $\mathbf{D}$ the equality
$d_{ij} = d_{ji}$ holds, i.e., it does not matter if we travel from $i$
to $j$ or the other way round, the distance is the same. In the
asymmetric case (called \emph{ATSP}), the distances are not equal for
all pairs of cities.  Problems of this kind arise when we do not deal
with spatial distances between cities but, e.g., with the cost or
necessary time associated with traveling between locations, where the
price for the plane ticket between two cities may be different depending
on which way we go.

\subsection{Different formulations of the TSP}
\label{sec:formulations}

Other than the permutation problem in the introduction, the TSP can also
be formulated as a graph theoretic problem. Here the TSP is formulated
by means of a complete graph $G = (V, E)$, where the cities correspond
to the node set $V = \{1,2,\ldots,n\}$ and each edge $e_i \in E$ has an
associated weight $w_i$ representing the distance between the nodes it
connects.  If the graph is not complete, the missing edges can be
replaced by edges with very large distances.  The goal is to find a
\emph{Hamiltonian cycle}, i.e., a cycle which visits each node in the
graph exactly once, with the least weight in the
graph~\citep{Hoffman1985}. This formulation naturally leads to
procedures involving minimum spanning trees for tour construction or
edge exchanges to improve existing tours.

TSPs can also be represented as integer and linear programming
problems~\citep[see, e.g.,][]{Punnen2002}.  The \emph{integer programming (IP)
formulation} is based on the assignment problem with additional
constraint of no sub-tours:


\[
\begin{array}{rl}
    \text{Minimize }     & \sum_{i=1}^n\sum_{j=1}^n{d_{ij}x_{ij}}
               %         = \mathrm{trace}(\mathbf{D}^T\mathbf{X})
                        \\[3mm]
    \text{Subject to }  & \sum_{i=1}^n{x_{ij}=1}, \quad j=1,\ldots,n, \\
                        & \sum_{j=1}^n{x_{ij}=1}, \quad i=1,\ldots,n, \\
                        & x_{ij} = 0 \text{ or } 1 \\
                        & \text{no sub-tours allowed} \\
\end{array}
\]

The solution matrix $\mathbf{X} = (x_{ij})$ of the assignment problem
represents a tour or a collection of sub-tour (several unconnected cycles) where
only edges which corresponding to elements $x_{ij} = 1$ are on the tour or a
sub-tour. The additional restriction that no sub-tours are allowed (called
\emph{sub-tour elimination constraints}) restrict the solution to only proper
tours. Unfortunately, the number of sub-tour elimination constraints grows
exponentially with the number of cities which leads to an extremely hard
problem.

The \emph{linear programming (LP) formulation} of the TSP is given by:
\[
\begin{array}{rl}
    \text{Minimize }     & \sum_{i=1}^m{w_ix_i}  = \mathbf{w}^T\mathbf{x}\\[3mm]
    \text{Subject to }   & \mathbf{x} \in \mathcal{S} \\
\end{array}
\]
where $m$ is the number of edges $e_i$ in $G$, $w_i \in \mathbf{w}$ is the
weight of edge $e_i$ and $\mathbf{x}$ is the incidence vector indicating the
presence or absence of each edge in the tour. Again, the constraints given by
$\mathbf{x} \in \mathcal{S}$ are problematic since they have to contain the set
of incidence vectors of all possible Hamiltonian cycles in $G$ which amounts to
a direct search of all $(n-1)!$ possibilities and thus in general is
infeasible. However, relaxed versions of the linear programming problem with
removed integrality and sub-tour elimination constraints are extensively used by
modern TSP solvers where such a partial description of constraints is used and
improved iteratively in a branch-and-bound approach.

\subsection{Useful manipulations of the distance matrix}
\label{sec:manipulations}

Sometimes it is useful to transform the distance matrix $\mathbf{D} = (d_{ij})$
of a TSP into a different matrix $\mathbf{D'} = (d'_{ij})$ which has the same
optimal solution.  Such a transformation requires that for any
Hamiltonian cycle $H$ in a graph represented by its distance matrix $\mathbf{D}$
the equality
\begin{equation*}
\sum_{i,j \in H}{d_{ij}} = \alpha \sum_{i,j \in H}{d'_{ij}} + \beta
\end{equation*}
holds for suitable $\alpha > 0$ and $\beta \in \mathbb{R}$.  From the
equality we see that additive and multiplicative constants leave the
optimal solution invariant. This property is useful to rescale
distances, e.g., for many solvers, distances in the interval $[0, 1]$
have to be converted into integers from~1 to a maximal value.

A different manipulation is to reformulate an asymmetric TSP as a symmetric TSP.
This is possible by doubling the number of cities~\citep{Jonker1983}. For each
city a dummy city is added. Between each city and its corresponding dummy city
a very small value (e.g., $-\infty$) is used.  This makes sure that each
city always occurs in the solution together with its dummy city. The original
distances are used between the cities and the dummy cities, where each city is
responsible for the distance going to the city and the dummy city is
responsible for the distance coming from the city. The distances between all
cities and the distances between all dummy cities are set to a very large
value (e.g., $\infty$) which makes these edges infeasible. An example for
equivalent formulations as an asymmetric TSP (to the left) and
a symmetric TSP (to the right) for
three cities is:

\begin{equation*}
\begin{pmatrix}
    0       &d_{12}    &d_{13}    \\
    d_{21} &0          &d_{23}    \\
    d_{31} &d_{32}    &0          \\
\end{pmatrix}
\Longleftrightarrow
\begin{pmatrix}
  0         &\infty     &\infty     & -\infty       &d_{21}    &d_{31}    \\
  \infty    &0          &\infty     & d_{12} &-\infty          &d_{31}    \\
  \infty    &\infty     &0          & d_{13} &d_{23}    &-\infty          \\
  -\infty         &d_{12}    &d_{13}    & 0       &\infty    &\infty      \\
  d_{21}   &-\infty          &d_{23}    & \infty &0         &\infty       \\
  d_{31}   &d_{32}    &-\infty          & \infty &\infty   &0             \\
\end{pmatrix}
\end{equation*}

Instead of the infinity values suitably large negative and positive values can
be used.  The new symmetric TSP can be solved using techniques for symmetric
TSPs which are currently far more advanced than techniques for ATSPs.
Removing the dummy cities from the resulting tour gives the  solution for the
original ATSP.


\subsection{Finding exact solutions for the TSP}
\label{sec:exact}

Finding the exact solution to a TSP with $n$ cities requires to check $(n-1)!$
possible tours. To evaluate all possible tours is infeasible for even small TSP
instances. To find the optimal tour
\cite{Held1962} presented the following
\emph{dynamic programming} formulation: Given a subset of city indices
(excluding the first city)
$S \subset \{2, 3, \dots, n\}$ and $l \in S$, let $d^*(S, l)$ denote the length of the
shortest path from city $1$ to city $l$, visiting all cities in $S$ in-between.
For $S = \{l\}$, $d^*(S,l)$ is defined as $d_{1l}$. The shortest path for
larger sets with $|S| > 1$ is

\begin{equation}
    d^*(S,l) = \mathrm{min}_{m \in S\setminus\{l\}}\Bigl( d^*(S\setminus\{l\},m) + d_{ml}\Bigl).
    %\text{ for } |S| > 1.
\end{equation}

Finally, the minimal tour length for a complete tour which includes returning
to city $1$ is

\begin{equation}
    d^{**} = \mathrm{min}_{l \in \{2,3,\dots,n\}}\Bigl( d^*(\{2,3,\dots,n\}, l)
    + d_{l1}\Bigl).
\end{equation}

Using the last two equations, the quantities $d^*(S,l)$ can be computed
recursively and the minimal tour length $d^{**}$ can be found.  In a
second step, the optimal permutation $\pi = \{1, i_2, i_3,\dots,i_n\}$
of city indices $1$ through $n$ can be computed in reverse order,
starting with $i_n$ and working successively back to $i_2$. The
procedure exploits the fact that a permutation $\pi$ can only be
optimal, if

\begin{equation}
    d^{**} = d^*(\{2,3,\dots,n\}, i_n) + d_{i_n1}
\end{equation}
and, for $2 \le p \le n-1$,
\begin{equation}
    d^*(\{i_2, i_3,\dots, i_p, i_{p+1}\}, i_{p+1}) =
    d^*(\{i_2,i_3,\dots,i_p\}, i_p) + d_{i_pi_{p+1}}.
\end{equation}

The space complexity of storing the values for all $d^*(S,l)$ is
$(n-1)2^{n-2}$ which severely restricts the dynamic programming
algorithm to TSP problems of small sizes.  However, for very small TSP
instances this approach is fast and efficient.
%\marginpar{time complexity if order $n^22^n$. Check!}

A different method, which can deal with larger instances, uses a relaxation of
the linear programming problem presented in Section~\ref{sec:formulations} and
iteratively tightens the relaxation till a solution is found. This general
method for solving linear programming problems with complex and large
inequality systems is called \emph{cutting plane method} and was introduced by
\cite{Dantzig1954}.

Each iteration begins with using instead of the original linear
inequality description $\mathcal{S}$ the relaxation $A\mathbf{x} \le b$,
where the polyhedron $P$ defined by the relaxation contains
$\mathcal{S}$ and is bounded.  The optimal solution $\mathbf{x}^*$ of
the relaxed problem can be obtained using standard linear programming
solvers. If the $\mathbf{x}^*$ found belongs to $\mathcal{S}$, the
optimal solution of the original problem is obtained, otherwise, a
linear inequality can be found which is satisfied by all points in
$\mathcal{S}$ but violated by $\mathbf{x}^*$.  Such an inequality is
called a cutting plane or cut.  A family of such cutting planes can be
added to the inequality system $A\mathbf{x} \le b$ to obtain a tighter
relaxation for the next iteration.

If no further cutting planes can be found or the improvement in the
objective function due to adding cuts gets very small, the problem is
branched into two sub-problems which can be minimized
separately. Branching is done iteratively which leads to a binary tree
of sub-problems. Each sub-problem is either solved without further
branching or is found to be irrelevant because its relaxed version
already produces a longer path than a solution of another sub-problem.
This method is called \emph{branch-and-cut}~\citep{Padberg1990} which is
a variation of the well known \emph{branch-and-bound}~\citep{Land1960}
procedure.

The initial polyhedron $P$ used by \cite{Dantzig1954} contains all
vectors $\mathbf{x}$ for which all $x_e \in \mathbf{x}$ satisfy $0 \le
x_e \le 1$ and in the resulting tour each city is linked to exactly two
other cities.  Various separation algorithms for finding subsequent cuts
to prevent sub-tours (\emph{sub-tour elimination inequalities}) and to
ensure an integer solution \citep[\emph{Gomory cuts;}][]{Gomory1963}
were developed over time. The currently most efficient implementation of
this method is \emph{Concorde} described in~\cite{Applegate2000}.

\subsection{Heuristics for the TSP}
\label{sec:heuristics}

The NP-completeness of the TSP already makes it more time efficient for
small-to-medium size TSP instances to rely on heuristics in case a good
but not necessarily optimal solution is sufficient.  TSP heuristics
typically fall into two groups, tour construction heuristics which
create tours from scratch and tour improvement heuristics which use
simple local search heuristics to improve existing tours.

In the following we will only discuss heuristics available in \pkg{TSP}, for a
comprehensive overview of the multitude of TSP heuristics including an
experimental comparison, we refer the reader to the book chapter by
\cite{Johnson2002}.

\subsubsection{Tour construction heuristics}
The implemented tour construction heuristics are the nearest neighbor algorithm
and the insertion algorithms.

\paragraph{Nearest neighbor algorithm.}

The nearest neighbor algorithm~\citep{Rosenkrantz1977} follows a very
simple greedy procedure:  The algorithm starts with a tour containing a
randomly chosen city and then always adds to the last city in the tour
the nearest not yet visited city. The algorithm stops when all cities
are on the tour.

An extension to this algorithm is to repeat it with each city as the
starting point and then return the best tour found.  This heuristic is
called repetitive nearest neighbor.

\paragraph{Insertion algorithms.}

All insertion algorithms~\citep{Rosenkrantz1977} start with a tour
consisting of an arbitrary city and then choose in each step a city $k$
not yet on the tour.  This city is inserted into the existing tour
between two consecutive cities $i$ and $j$, such that the insertion cost
(i.e., the increase in the tour's length)
$$d(i,k) + d(k,j) - d(i,j)$$
is minimized.  The algorithms stop when all cities are on the tour.

The insertion algorithms differ in the way the city to be inserted next
is chosen.  The following variations are implemented:

\begin{description}
\item[Nearest insertion] The city $k$ is chosen in each step as
the city which is nearest to a city on the tour.

\item[Farthest insertion] The city $k$ is chosen in each step as
the city which is farthest from any of the cities on the tour.

\item[Cheapest insertion] The city $k$ is chosen in each step
such that the cost of inserting the new city is minimal.

\item[Arbitrary insertion] The city $k$ is chosen randomly from all cities
not yet on the tour.

\end{description}


The nearest and cheapest insertion algorithms correspond to the minimum
spanning tree algorithm by \cite{Prim1957}. Adding a city to a partial tour
corresponds to adding an edge to a partial spanning tree. For TSPs with
distances obeying the triangular inequality, the equality to minimum spanning
trees provides a theoretical upper bound for the two algorithms of twice the
optimal tour length.

The idea behind the farthest insertion algorithm is to link cities far
outside into the tour first to establish an outline of the whole tour
early. With this change, the algorithm cannot be directly related to
generating a minimum spanning tree and thus the upper bound stated above
cannot be guaranteed.  However, it can was shown that the algorithm
generates tours which approach $2/3$ times the optimal tour
length~\citep{Johnson1985}.

\subsubsection{Tour improvement heuristics}

Tour improvement heuristics are simple local search heuristics which try to
improve an initial tour. A comprehensive treatment of the topic
can be found in the book chapter by \cite{Rego2002}.

\paragraph{$k$-Opt heuristics.} The idea is to define a neighborhood structure
on the set of all admissible tours. Typically, a tour $t'$ is a neighbor of
another tour $t$ if $t'$ can be obtained from $t$ by deleting $k$ edges
and replacing them by a set of different feasible edges (a $k$-Opt move).  In
such a structure, the tour can iteratively be improved by always moving from
one tour to its best neighbor till no further improvement is possible. The
resulting tour represents a local optimum which is called $k$-optimal.

Typically, $2$-Opt~\citep{Croes1958} and $3$-Opt~\citep{Lin1965} heuristics
are used in practice.

\paragraph{Lin-Kernighan heuristic.}

This heuristic~\citep{Lin1973} does not use a fixed value for $k$ for
its $k$-Opt moves, but tries to find the best choice of $k$ for each
move.  The heuristic uses the fact that each $k$-Opt move can be
represented as a sequence of $2$-Opt moves.  It builds up a sequence of
$2$-Opt moves, checking after each additional move whether a stopping
rule is met. Then the part of the sequence which gives the best
improvement is used. This is equivalent to a choice of one $k$-Opt move
with variable $k$. Such moves are used till a local optimum is reached.

By using full backtracking, the optimal solution can always be found, but the
running time would be immense. Therefore, only limited backtracking is allowed
in the procedure, which helps to find better local optima or even the optimal
solution. Further improvements to the procedure are described
by \cite{Lin1973}.


\section{Computational infrastructure: the TSP package}
\label{sec:infrastructure}

In package~\pkg{TSP}, a traveling salesperson problem is
defined by an object of class \class{TSP} (symmetric) or \class{ATSP}
(asymmetric). \func{solve\_TSP}
is used to find a solution, which is represented by
an object of class \class{TOUR}. Figure~\ref{fig:overview} gives an overview
of this infrastructure.

\begin{figure}
    \centering
    \includegraphics[width=14cm]{overview}
    \caption{An overview of the classes in \pkg{TSP}.}
    \label{fig:overview}
\end{figure}


\class{TSP} objects can be created from a distance matrix (a
\class{dist} object) or a symmetric matrix using the creator function
\func{TSP} or coercion with \func{as.TSP}.  Similarly, \class{ATSP}
objects are created by \func{ATSP} or \func{as.ATSP} from square
matrices representing the distances.  In the creation process, labels
are taken and stored as city names in the object or can be explicitly
given as arguments to the creator functions.  Several methods are
defined for the classes:
\begin{itemize}
 \item \func{print} displays basic information about the problem (number
  of cities and the distance measure employed).
 \item \func{n\_of\_cities} returns the number of cities.
 \item \func{labels} returns the city names.
 \item \func{image} produces a shaded matrix plot of the distances
  between cities. The order of the cities can be specified as the
  argument \code{order}.
\end{itemize}

Internally, an object of class \class{TSP} is a \class{dist} object with
an additional class attribute and, therefore, if needed, can be coerced
to \class{dist} or to a matrix.  An \class{ATSP} object is represented
as a square matrix.  Obviously, asymmetric TSPs are more general than
symmetric TSPs, hence, symmetric TSPs can also be represented as
asymmetric TSPs.  To formulate an asymmetric TSP as a symmetric TSP with
double the number of cities (see Section \ref{sec:manipulations}),
\func{reformulate\_ATSP\_as\_TSP} is provided.  This function creates
the necessary dummy cities and adapts the distance matrix accordingly.

A popular format to save TSP descriptions to disk which is supported by
most TSP solvers is the format used by \emph{TSPLIB}, a library of
sample instances of the TSP maintained by \cite{Reinelt2004}. The
\pkg{TSP} package provides \func{read\_TSPLIB} and \func{write\_TSPLIB}
to read and save symmetric and asymmetric TSPs in TSPLIB format.

Class \class{TOUR} represents a solution to a TSP by an integer
permutation vector containing the ordered indices and labels of the
cities to visit.  In addition, it stores an attribute indicating the
length of the tour.  Again, suitable \func{print} and \func{labels}
methods are provided.  The raw permutation vector (i.e., the order in
which cities are visited) can be obtained from a tour using
\func{as.integer}.  With \func{cut\_tour}, a circular tour can be split
at a specified city resulting in a path represented by a vector of city
indices.

The length of a tour can always be calculated using \func{tour\_length}
and specifying a TSP and a tour. Instead of the tour, an integer
permutation vector calculated outside the \pkg{TSP} package can be used
as long as it has the correct length.

All TSP solvers in \pkg{TSP} can be used with the simple common interface:

\begin{quote}
\code{solve\_TSP(x, method, control)}
\end{quote}
where \code{x} is the TSP to be solved, \code{method} is a character
string indicating the method used to solve the TSP and \code{control}
can contain a list with additional information used by the solver.
The available algorithms are shown in Table~\ref{tab:methods}.

\begin{table}
    \caption{Available algorithms in \pkg{TSP}.}
    \label{tab:methods}
    \centering
    \begin{tabular}{llc}
    \hline
    \textbf{Algorithm} & \textbf{Method argument} & \textbf{Applicable to} \\
    \hline
    Nearest neighbor algorithm & \code{"nn"} & TSP/ATSP \\
    Repetitive nearest neighbor algorithm & \code{"repetitive\_nn"}
            & TSP/ATSP \\
    Nearest insertion & \code{"nearest\_insertion"} & TSP/ATSP  \\
    Farthest insertion & \code{"farthest\_insertion"} & TSP/ATSP  \\
    Cheapest insertion & \code{"cheapest\_insertion"} & TSP/ATSP  \\
    Arbitrary insertion & \code{"arbitrary\_insertion"} & TSP/ATSP  \\
    Concorde TSP solver & \code{"concorde"} & TSP  \\
    2-Opt improvement heuristic &  \code{"two\_opt"}  & TSP/ATSP \\
    Chained Lin-Kernighan &  \code{"linkern"}  & TSP \\
\hline
\end{tabular}
\end{table}

All algorithms except the Concorde TSP solver and the Chained
Lin-Kernighan heuristic (a Lin-Kernighan variation described in
\cite{Applegate2003}) are included in the package and distributed under
the GNU Public License (GPL). For the Concorde TSP solver and the
Chained Lin-Kernighan heuristic only a simple interface (using
\func{write\_TSPLIB}, calling the executable and reading back the
resulting tour) is included in \pkg{TSP}.  The executable itself is part
of the Concorde distribution, has to be installed separately and is
governed by a different license which allows only for academic use.  The
interfaces are included since
Concorde~\citep{Applegate2000,Applegate2006} is currently one of the
best implementations for solving symmetric TSPs based on the
branch-and-cut method discussed in section~\ref{sec:exact}.  In May
2004, Concorde was used to find the optimal solution for the TSP of
visiting all 24,978 cities in Sweden.  The computation was carried out
on a cluster with 96 Xeon 2.8 GHz nodes and took in total almost 100 CPU
years.

\section{Examples}
\label{sec:examples}

In this section we provide some examples for the use of
package~\pkg{TSP}.  We start with a simple example of how to use the
interface of the TSP solver to compare different heuristics.  Then we
show how to solve related tasks, using the Hamiltonian shortest path
problem as an example.  Finally, we give an example of clustering using
the \pkg{TSP} package.  An additional application can be found
in package \pkg{seriation}~\citep{Hahsler+Buchta+Hornik:2006} which uses
the TSP solvers from \pkg{TSP} to order (seriate) objects given a proximity
matrix.

\subsection{Comparing some heuristics}

In the following example, we use several heuristics to find a short path
in the \code{USCA50} data set which contains the distances between the
first 50 cities in the \code{USCA312} data set. The \code{USCA312} data
set contains the distances between 312 cities in the USA and Canada
coded as a symmetric TSP.  The smaller data set is used here, since some
of the heuristic solvers employed are rather slow.

<<>>=
library("TSP")
data("USCA50")
USCA50
@


We calculate tours using different heuristics and store the results in
the list \code{tours}. As an example, we show the first tour which
displays the method employed, the number of cities involved and the tour
length.  All tour lengths are compared using the dot chart in
Figure~\ref{fig:dotchart}.  For the chart, we add a point for the
optimal solution which has a tour length of 14497. The optimal solution
can be found using Concorde (\code{method = "concorde"}).  It is omitted
here, since Concorde has to be installed separately.

<<echo=FALSE>>=
set.seed(1234)
@


<<fig=TRUE, include=FALSE, label=dotchart_USCA50, width=7, height=4>>=
methods <- c("nearest_insertion", "farthest_insertion", "cheapest_insertion",
    "arbitrary_insertion", "nn", "repetitive_nn", "two_opt")

tours <- sapply(methods, FUN = function(m) solve_TSP(USCA50, method = m),
                simplify = FALSE)
## tours$concorde  <- solve_TSP(tsp, method = "concorde")

tours[[1]]

dotchart(sort(c(sapply(tours, tour_length), optimal = 14497)),
xlab = "tour length", xlim = c(0, 20000))
@

\begin{figure}
\centering
\includegraphics[width=11cm, trim=0 10 0 0]{TSP-dotchart_USCA50}
\caption{Comparison of the tour lengths for the USCA50 data set.}
\label{fig:dotchart}
\end{figure}


\subsection{Finding the shortest Hamiltonian path}

The problem of finding the shortest Hamiltonian path through a graph
(i.e., a path which visits each node in the graph exactly once) can be
transformed into the TSP with cities and distances representing the
graphs vertices and edge weights, respectively~\citep{Garfinkel1985}.

Finding the shortest Hamiltonian path through all cities disregarding
the endpoints can be achieved by inserting a `dummy city' which has a
distance of zero to all other cities. The position of this city in the
final tour represents the cutting point for the path. In the following
we use a heuristic to find a short path in the \code{USCA312} data set.
Inserting dummy cities is performed in \pkg{TSP} by
\func{insert\_dummy}.

<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
library("TSP")
data("USCA312")
tsp <- insert_dummy(USCA312, label = "cut")
tsp
@

The TSP now contains an additional dummy city and we can try to solve
this TSP.

<<>>=
tour <- solve_TSP(tsp, method="farthest_insertion")
tour
@

Since the dummy city has distance zero to all other cities, the path length is
equal to the tour length reported above. The path starts with the first city in
the list after the `dummy' city  and ends with the city right before it.
We use \func{cut\_tour} to create a path and show the
first and last 6 cities on it.

<<>>=
path <- cut_tour(tour, "cut")
head(labels(path))
tail(labels(path))
@

The tour found in the example results in a path from Lihue on Hawaii to
Prince Rupert in British Columbia. Such a tour can also be visualized
using a scatter plot with a map from package \pkg{maps}. Note that the \code{if} statement and the \code{plot} in the \code{else} part in the following is not needed and only checks if the map package is installed when building this document.

<<fig=TRUE, include=FALSE, label=map1, width=8, height=8>>=
if(require(maps)) {
  library("maps")

  data("USCA312_GPS")

  plot_path <- function(path) {
    plot((USCA312_GPS[, c("long", "lat")]), cex = .3, col = "red")
    map("world", col = "gray", add = TRUE)
    lines(USCA312_GPS[, c("long", "lat")][path,], col = "black")
    points(USCA312_GPS[c(head(path, 1), tail(path, 1)),  c("long", "lat")], pch = 19, col = "black")
  }

  plot_path(path)

} else {
    plot(NA, xlim= c(0,1), ylim = c(0,1))
    text(.5, .5, "Suggested packages not available")
}
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map1}
\caption{A ``short'' Hamiltonian path for the USCA312 dataset.}
\label{fig:map1}
\end{figure}

The map containing the path is presented in Figure~\ref{fig:map1}.  It
has to be mentioned that the path found by the used heuristic is
considerable longer than the optimal path found by Concorde with a
length of $34928$, illustrating the power of modern TSP algorithms.

For the following two examples, we indicate how the distance matrix
between cities can be modified to solve related shortest Hamiltonian
path problems.  These examples serve as illustrations of how
modifications can be made to transform different problems into a TSP.

The first problem is to find the shortest Hamiltonian path starting with
a given city. In this case, all distances to the selected city are set
to zero, forcing the evaluation of all possible paths starting with this
city and disregarding the way back from the final city in the tour.  By
modifying the distances the symmetric TSP is changed into an asymmetric
TSP (ATSP) since the distances between the starting city and all other
cities are no longer symmetric.

As an example, we choose New York as the starting city.  We transform
the data set into an ATSP and set the column corresponding to New York
to zero before solving it.  Thus, the distance to return from the last
city in the path to New York does not contribute to the path length.  We
use the nearest neighbor heuristic to calculate an initial tour which is
then improved using $2$-Opt moves and cut at New York to create a path.


<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
atsp <- as.ATSP(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
atsp[, ny] <- 0
initial_tour <- solve_TSP(atsp, method="nn")
initial_tour
tour <- solve_TSP(atsp, method ="two_opt", control = list(tour = initial_tour))
tour
path <- cut_tour(tour, ny, exclude_cut = FALSE)

head(labels(path))
tail(labels(path))
@
<<fig=TRUE, include=FALSE, label=map2, width=8, height=8>>=
plot_path(path)
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map2}
\caption{A Hamiltonian path for the USCA312 dataset starting in New York.}
\label{fig:map2}
\end{figure}

The found path is presented in Figure~\ref{fig:map2}. It begins with New
York and cities in New Jersey and ends in a city in Manitoba, Canada.
%The path shows the typical behavior
%of the nearest neighbor heuristic with first connecting the cities close by and
%then making rather big ``jumps'' for the final cities.

Concorde and many advanced TSP solvers can only solve symmetric TSPs. To
use these solvers, we can formulate the ATSP as a TSP using
\func{reformulate\_ATSP\_as\_TSP} which introduces a dummy city for each
city (see Section~\ref{sec:manipulations}).

<<>>=
tsp <- reformulate_ATSP_as_TSP(atsp)
tsp
@

After finding a tour for the TSP, the dummy cities are removed again giving the
tour for the original ATSP.
Note that the tour needs to be reversed if the
dummy cities appear before and not after the original cities in the solution of
the TSP.  The following code is not executed here, since it takes several
minutes to execute and Concorde has to be installed separately.  Concorde finds
the optimal solution with a length of $36091$.


<<eval=FALSE>>=
tour <- solve_TSP(tsp, method = "concorde")
tour <- as.TOUR(tour[tour <= n_of_cities(atsp)])
@


Finding the shortest Hamiltonian path which ends in a given city can be
achieved likewise by setting the row in the distance matrix which
corresponds to this city to zero.

For finding the shortest Hamiltonian path we can also restrict both end
points.  This problem can be transformed to a TSP by replacing the two
cities by a single city which contains the distances from the start
point in the columns and the distances to the end point in the
rows. Obviously this is again an asymmetric TSP.

For the following example, we are only interested in paths starting in
New York and ending in Los Angeles. Therefore, we remove the two cities
from the distance matrix, create an asymmetric TSP and insert a dummy
city called \code{"LA/NY"}. The distances from this dummy city are
replaced by the distances from New York and the distances towards are
replaced by the distances towards Los Angeles.


<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
m <- as.matrix(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
la <- which(labels(USCA312) == "Los Angeles, CA")

atsp <- ATSP(m[-c(ny,la), -c(ny,la)])
atsp <- insert_dummy(atsp, label = "LA/NY")

la_ny <- which(labels(atsp) == "LA/NY")
atsp[la_ny, ] <- c(m[-c(ny,la), ny], 0)
atsp[, la_ny] <- c(m[la, -c(ny,la)], 0)
@


We use the nearest insertion heuristic.
<<>>=
tour <- solve_TSP(atsp, method ="nearest_insertion")
tour

path_labels <- c("New York, NY",
    labels(cut_tour(tour, la_ny)), "Los Angeles, CA")
path_ids <- match(path_labels, labels(USCA312))

head(path_labels)
tail(path_labels)
@
<<fig=TRUE, include=FALSE, label=map3, width=8, height=8>>=
plot_path(path_ids)
@


The path jumps from New York to cities in Ontario and it passes through
cities in California and Nevada before ending in Los Angeles. The path
displayed in Figure~\ref{fig:map3} contains multiple crossings which
indicate that the solution is suboptimal. The optimal solution generated
by reformulating the problem as a TSP and using Concorde has only a tour
length of $38489$.

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map3}
\caption{A Hamiltonian path for the USCA312 dataset starting in New York
  and ending in Los Angles.}
\label{fig:map3}
\end{figure}


\subsection{Rearrangement clustering}
Solving a TSP to obtain a clustering was suggested several times in the
literature \citep[see, e.g.,][]{Lenstra1974, Alpert1997, Johnson2004}.  The idea
is that objects in clusters are visited in consecutive order and from one
cluster to the next larger ``jumps'' are necessary.  \cite{Climer2006} call
this type of clustering \emph{rearrangement clustering} and suggest to
automatically find the cluster boundaries of $k$ clusters by adding $k$
\emph{dummy cities} which have constant distance $c$ to all other cities and
are infinitely far from each other. In the optimal
solution of the TSP, the dummy cities must separate the most distant cities and
thus represent optimal boundaries for $k$ clusters.

For the following example, we use the well known iris data set. Since we
know that the dataset contains three classes denoted by the variable
\code{Species}, we insert three dummy cities into the TSP for the iris
data set and perform rearrangement clustering using the default method
(nearest insertion algorithm).  Note that this algorithm does not find
the optimal solution and it is not guaranteed that the dummy cities will
present the best cluster boundaries.
%\marginpar{FIXME: Was sagt Concorde dazu?}

<<echo=FALSE>>=
set.seed(4444)
@
<<>>=
data("iris")
tsp <- TSP(dist(iris[-5]), labels = iris[, "Species"])
tsp_dummy <- insert_dummy(tsp, n = 3, label = "boundary")
tour <- solve_TSP(tsp_dummy)
@

Next, we plot the TSP's permuted distance matrix using shading to
represent distances. The result is displayed as
Figure~\ref{fig:clustering}. Lighter areas represent larger
distances. The additional red lines represent the positions of the dummy
cities in the tour, which mark the cluster boundaries obtained.


<<fig=TRUE, include=FALSE, label=clustering>>=
## plot the distance matrix
image(tsp_dummy, tour, xlab = "objects", ylab ="objects")

## draw lines where the dummy cities are located
abline(h = which(labels(tour)=="boundary"), col = "red")
abline(v = which(labels(tour)=="boundary"), col = "red")
@


\begin{figure}
\centering
\includegraphics[width=9cm, height=9cm, trim=0 20 0 0]{TSP-clustering}
\caption{Result of rearrangement clustering using three dummy cities and the
nearest insertion algorithm on the iris data set.}
\label{fig:clustering}
\end{figure}


One pair of red horizontal and vertical lines exactly separates the
darker from lighter areas.  The second pair occurs inside the larger
dark block.  We can look at how well the partitioning obtained fits the
structure in the data given by the species field in the data set.  Since
we used the species as the city labels in the TSP, the labels in the
tour represent the partitioning with the dummy cities named `boundary'
separating groups.  The result can be summarized based on the run length
encoding of the obtained tour labels:

<<>>=
out <- rle(labels(tour))
data.frame(Species = out$values,
           Lenghts = out$lengths,
           Pos = cumsum(out$lengths))
@


One boundary perfectly splits the iris data set into a group containing
only examples of species `Setosa' and a second group containing examples
for `Virginica' and `Versicolor'.  However, the second boundary only
separates several examples of species `Virginica' from other examples of
the same species.  Even in the optimal tour found by Concorde, this
problem occurs. The reason why the rearrangement clustering fails to
split the data into three groups is the closeness between the groups
`Virginica' and `Versicolor'.  To inspect this problem further, we can
project the data points on the first two principal components of the
data set and add the path segments which resulted from solving the TSP.

<<fig=TRUE, include=FALSE, label=clustering2>>=
prc <- prcomp(iris[1:4])
plot(prc$x, pch = as.numeric(iris[,5]), col =  as.numeric(iris[,5]))

paths <- cut_tour(tour, cut = "boundary")
for(p in paths) lines(prc$x[p, ])
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 20 0 0]{TSP-clustering2}
\caption{The 3 path segments representing a rearrangement clustering of the
iris data set.  The data points are projected on the set's first two principal
components.  The three species are represented by different markers and
colors.}
\label{fig:clustering2}
\end{figure}

The result in shown in Figure~\ref{fig:clustering2}.  The three species
are identified by different markers and all points connected by a single
path represent a cluster found.  Clearly, the two groups to the right
side of the plot are too close to be separated correctly by using just
the distances between individual points.  This problem is similar to the
\emph{chaining effect} known from hierarchical clustering using the
single-linkage method.

\section{Conclusion}
\label{sec:conclusion}

In this paper we presented the R extension package \pkg{TSP} which
implements an infrastructure to handle and solve TSPs. The package
introduces classes for problem descriptions (\class{TSP} and
\class{ATSP}) and for the solution (\class{TOUR}).  Together with a
simple interface for solving TSPs, it allows for an easy and transparent
usage of the package.

With the interface to Concorde, \pkg{TSP} also can use a state of the
art implementation which efficiently computes exact solutions using
branch-and-cut.

\section*{Acknowledgments}

The authors of this paper want to thank Roger Bivand for providing the
code to correctly draw tours and paths on a projected map.


%
%\bibliographystyle{abbrvnat}
\bibliography{TSP}
%
\end{document}

%\documentclass[10pt,a4paper,fleqn]{article}
\documentclass[nojss]{jss}

\usepackage[english]{babel}

%% my packages
%\usepackage{a4wide}
%\usepackage[round,longnamesfirst]{natbib}
%\usepackage{hyperref}
%
\usepackage{amsmath}
\usepackage{amsfonts}
%
%\newcommand{\strong}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\class}[1]{\mbox{\textsf{#1}}}
\newcommand{\func}[1]{\mbox{\texttt{#1()}}}
%\newcommand{\code}[1]{\mbox{\texttt{#1}}}
%\newcommand{\pkg}[1]{\strong{#1}}
\newcommand{\samp}[1]{`\mbox{\texttt{#1}}'}
%\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\set}[1]{\mathcal{#1}}

%\usepackage{Sweave}
%% \VignetteIndexEntry{Introduction to TSP}

\author{Michael Hahsler\\Southern Methodist University \And
    Kurt Hornik\\Wirtschaftsuniversit\"at Wien}
\title{\pkg{TSP} -- Infrastructure for the Traveling
	Salesperson Problem}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Michael Hahsler, Kurt Hornik} %% comma-separated
\Plaintitle{TSP -- Infrastructure for the Traveling
	        Salesperson Problem} %% without formatting
\Shorttitle{Infrastructure for the TSP} %% a short title (if necessary)

%% an abstract and keywords
\Abstract{ The traveling salesperson problem (also known as traveling salesman problem or TSP) is a
well known and important combinatorial optimization problem.  The goal
is to find the shortest tour that visits each city in a given list
exactly once and then returns to the starting city.  Despite this
simple problem statement, solving the TSP is difficult since it
belongs to the class of NP-complete problems.  The importance of the
TSP arises besides from its theoretical appeal from the variety of its
applications.  Typical applications in operations research include
vehicle routing, computer wiring, cutting wallpaper and job
sequencing.  The main application in statistics is combinatorial data
analysis, e.g., reordering rows and columns of data matrices or
identifying clusters.  In this paper we introduce the
\proglang{R}~package \pkg{TSP} which provides a basic infrastructure
for handling and solving the traveling salesperson problem.  The
package features S3 classes for specifying a TSP and its (possibly
optimal) solution as well as several heuristics to find good
solutions. In addition, it provides an interface to \emph{Concorde},
one of the best exact TSP solvers currently available.  }
\Keywords{combinatorial optimization, traveling salesman problem, \proglang{R}}
\Plainkeywords{combinatorial optimization, traveling salesman problem, R} %% without formatting
%% The address of (at least) one author should be given
%% in the following format:
\Address{
      Michael Hahsler\\
	  Engineering Management, Information, and Systems\\
	  Lyle School of Engineering\\
	  Southern Methodist University\\
	  P.O. Box 750123 \\
	  Dallas, TX 75275-0123\\
	  E-mail: \email{mhahsler@lyle.smu.edu}\\

      Kurt Hornik\\
	  Department of Finance, Accounting and Statistics\\
	  Wirtschaftsuniversit\"at Wien\\
	  1090 Wien, Austria\\i
	  E-mail: \email{kurt.hornik@wu-wien.ac.at}\\
	  URL: \url{http://statmath.wu.ac.at/~hornik/}
}



\begin{document}

<<echo=FALSE>>=
options(width = 75, useFancyQuotes=FALSE, prompt="R> ")
### for sampling
set.seed(1234)
@

%\title{Introduction to \pkg{TSP} -- Infrastructure for the Traveling
%    Salesperson Problem}
%\author{Michael Hahsler and Kurt Hornik}
\maketitle
\sloppy

%\abstract{
%  The traveling salesperson (or, salesman) problem (TSP) is a
%  well known and important combinatorial optimization problem.  The goal
%  is to find the shortest tour that visits each city in a given list
%  exactly once and then returns to the starting city.  Despite this
%  simple problem statement, solving the TSP is difficult since it
%  belongs to the class of NP-complete problems.  The importance of the
%  TSP arises besides from its theoretical appeal from the variety of its
%  applications.  Typical applications in operations research include
%  vehicle routing, computer wiring, cutting wallpaper and job
%  sequencing.  The main application in statistics is combinatorial data
%  analysis, e.g., reordering rows and columns of data matrices or
%  identifying clusters.  In this paper we introduce the
%  \proglang{R}~package \pkg{TSP} which provides a basic infrastructure
%  for handling and solving the traveling salesperson problem.  The
%  package features S3 classes for specifying a TSP and its (possibly
%  optimal) solution as well as several heuristics to find good
%  solutions. In addition, it provides an interface to \emph{Concorde},
%  one of the best exact TSP solvers currently available.  }

\section{Introduction}

The traveling salesperson problem~\citep[TSP;][]{Lawler1985, Gutin2002}
is a well known and important combinatorial optimization problem. The
goal is to find the shortest tour that visits each city in a given list
exactly once and then returns to the starting city.  Formally, the TSP
can be stated as follows.  The distances between $n$ cities are stored
in a distance matrix $\mathbf{D}$ with elements $d_{ij}$ where $i,j =
1\dots n$ and the diagonal elements $d_{ii}$ are zero.  A \emph{tour}
can be represented by a cyclic permutation $\pi$ of $\{1, 2,\dots, n\}$
where $\pi(i)$ represents the city that follows city $i$ on the
tour. The traveling salesperson problem is then the optimization problem
to find a permutation $\pi$ that minimizes the \emph{length of the tour}
denoted by

\begin{equation}
    \sum_{i=1}^n d_{i\pi(i)}.
\end{equation}
% see Lenstra & Kan 1975: Some simple Applications to the TSP

For this minimization task, the tour length of $(n-1)!$ permutation
vectors have to be compared.  This results in a problem which is very
hard to solve and in fact known to be NP-complete~\citep{Johnson1985a}.
However, solving TSPs is an important part of applications in many areas
including vehicle routing, computer wiring, machine sequencing and
scheduling, frequency assignment in communication
networks~\citep{Lenstra1975, Punnen2002}.
%and structuring of
%matrices~\citep{Lenstra1975, Punnen2002}.
Applications in statistical data analysis include ordering and
clustering objects.  For example, data analysis applications in
psychology ranging from profile smoothing to finding an order in
developmental data are presented by~\cite{Hubert1978}.  Clustering and
ordering using TSP solvers is currently becoming popular in
biostatistics.  For example, \cite{Ray2007} describe an application for
ordering genes and \cite{Johnson2006} use a TSP solver for clustering
proteins.

In this paper we give a very brief overview of the TSP and introduce the
\proglang{R}~package \pkg{TSP} which provides an infrastructure for
handling and solving TSPs.  The paper is organized as follows.  In
Section~\ref{sec:TSP} we briefly present important aspects of the TSP
including different problem formulations and approaches to solve TSPs.
In Section~\ref{sec:infrastructure} we give an overview of the
infrastructure implemented in \pkg{TSP} and the basic usage.  In
Section~\ref{sec:examples}, several examples are used to illustrate the
package's capabilities.  Section~\ref{sec:conclusion} concludes the
paper.

A previous version of this manuscript was published in the \emph{Journal
  of Statistical Software} \citep{TSP:Hahsler+Hornik2007}.

\section{Theory}
\label{sec:TSP}

In this section, we briefly summarize some aspects of the TSP which are
important for the implementation of the \pkg{TSP} package described in
this paper.  For a complete treatment of all aspects of the TSP, we
refer the interested reader to the classic book edited by
\cite{Lawler1985} and the more modern book edited by \cite{Gutin2002}.

It has to be noted that in this paper, following the origin of the TSP,
the term \emph{distance} is used.  Distance is used here interchangeably
with dissimilarity or cost and, unless explicitly stated, no
restrictions to measures which obey the triangle inequality are made.
An important distinction can be made between the symmetric TSP and the
more general asymmetric TSP.  For the symmetric case (normally referred
to as just \emph{TSP}), for all distances in $\mathbf{D}$ the equality
$d_{ij} = d_{ji}$ holds, i.e., it does not matter if we travel from $i$
to $j$ or the other way round, the distance is the same. In the
asymmetric case (called \emph{ATSP}), the distances are not equal for
all pairs of cities.  Problems of this kind arise when we do not deal
with spatial distances between cities but, e.g., with the cost or
necessary time associated with traveling between locations, where the
price for the plane ticket between two cities may be different depending
on which way we go.

\subsection{Different formulations of the TSP}
\label{sec:formulations}

Other than the permutation problem in the introduction, the TSP can also
be formulated as a graph theoretic problem. Here the TSP is formulated
by means of a complete graph $G = (V, E)$, where the cities correspond
to the node set $V = \{1,2,\ldots,n\}$ and each edge $e_i \in E$ has an
associated weight $w_i$ representing the distance between the nodes it
connects.  If the graph is not complete, the missing edges can be
replaced by edges with very large distances.  The goal is to find a
\emph{Hamiltonian cycle}, i.e., a cycle which visits each node in the
graph exactly once, with the least weight in the
graph~\citep{Hoffman1985}. This formulation naturally leads to
procedures involving minimum spanning trees for tour construction or
edge exchanges to improve existing tours.

TSPs can also be represented as integer and linear programming
problems~\citep[see, e.g.,][]{Punnen2002}.  The \emph{integer programming (IP)
formulation} is based on the assignment problem with additional
constraint of no sub-tours:


\[
\begin{array}{rl}
    \text{Minimize }     & \sum_{i=1}^n\sum_{j=1}^n{d_{ij}x_{ij}}
               %         = \mathrm{trace}(\mathbf{D}^T\mathbf{X})
                        \\[3mm]
    \text{Subject to }  & \sum_{i=1}^n{x_{ij}=1}, \quad j=1,\ldots,n, \\
                        & \sum_{j=1}^n{x_{ij}=1}, \quad i=1,\ldots,n, \\
                        & x_{ij} = 0 \text{ or } 1 \\
                        & \text{no sub-tours allowed} \\
\end{array}
\]

The solution matrix $\mathbf{X} = (x_{ij})$ of the assignment problem
represents a tour or a collection of sub-tour (several unconnected cycles) where
only edges which corresponding to elements $x_{ij} = 1$ are on the tour or a
sub-tour. The additional restriction that no sub-tours are allowed (called
\emph{sub-tour elimination constraints}) restrict the solution to only proper
tours. Unfortunately, the number of sub-tour elimination constraints grows
exponentially with the number of cities which leads to an extremely hard
problem.

The \emph{linear programming (LP) formulation} of the TSP is given by:
\[
\begin{array}{rl}
    \text{Minimize }     & \sum_{i=1}^m{w_ix_i}  = \mathbf{w}^T\mathbf{x}\\[3mm]
    \text{Subject to }   & \mathbf{x} \in \mathcal{S} \\
\end{array}
\]
where $m$ is the number of edges $e_i$ in $G$, $w_i \in \mathbf{w}$ is the
weight of edge $e_i$ and $\mathbf{x}$ is the incidence vector indicating the
presence or absence of each edge in the tour. Again, the constraints given by
$\mathbf{x} \in \mathcal{S}$ are problematic since they have to contain the set
of incidence vectors of all possible Hamiltonian cycles in $G$ which amounts to
a direct search of all $(n-1)!$ possibilities and thus in general is
infeasible. However, relaxed versions of the linear programming problem with
removed integrality and sub-tour elimination constraints are extensively used by
modern TSP solvers where such a partial description of constraints is used and
improved iteratively in a branch-and-bound approach.

\subsection{Useful manipulations of the distance matrix}
\label{sec:manipulations}

Sometimes it is useful to transform the distance matrix $\mathbf{D} = (d_{ij})$
of a TSP into a different matrix $\mathbf{D'} = (d'_{ij})$ which has the same
optimal solution.  Such a transformation requires that for any
Hamiltonian cycle $H$ in a graph represented by its distance matrix $\mathbf{D}$
the equality
\begin{equation*}
\sum_{i,j \in H}{d_{ij}} = \alpha \sum_{i,j \in H}{d'_{ij}} + \beta
\end{equation*}
holds for suitable $\alpha > 0$ and $\beta \in \mathbb{R}$.  From the
equality we see that additive and multiplicative constants leave the
optimal solution invariant. This property is useful to rescale
distances, e.g., for many solvers, distances in the interval $[0, 1]$
have to be converted into integers from~1 to a maximal value.

A different manipulation is to reformulate an asymmetric TSP as a symmetric TSP.
This is possible by doubling the number of cities~\citep{Jonker1983}. For each
city a dummy city is added. Between each city and its corresponding dummy city
a very small value (e.g., $-\infty$) is used.  This makes sure that each
city always occurs in the solution together with its dummy city. The original
distances are used between the cities and the dummy cities, where each city is
responsible for the distance going to the city and the dummy city is
responsible for the distance coming from the city. The distances between all
cities and the distances between all dummy cities are set to a very large
value (e.g., $\infty$) which makes these edges infeasible. An example for
equivalent formulations as an asymmetric TSP (to the left) and
a symmetric TSP (to the right) for
three cities is:

\begin{equation*}
\begin{pmatrix}
    0       &d_{12}    &d_{13}    \\
    d_{21} &0          &d_{23}    \\
    d_{31} &d_{32}    &0          \\
\end{pmatrix}
\Longleftrightarrow
\begin{pmatrix}
  0         &\infty     &\infty     & -\infty       &d_{21}    &d_{31}    \\
  \infty    &0          &\infty     & d_{12} &-\infty          &d_{31}    \\
  \infty    &\infty     &0          & d_{13} &d_{23}    &-\infty          \\
  -\infty         &d_{12}    &d_{13}    & 0       &\infty    &\infty      \\
  d_{21}   &-\infty          &d_{23}    & \infty &0         &\infty       \\
  d_{31}   &d_{32}    &-\infty          & \infty &\infty   &0             \\
\end{pmatrix}
\end{equation*}

Instead of the infinity values suitably large negative and positive values can
be used.  The new symmetric TSP can be solved using techniques for symmetric
TSPs which are currently far more advanced than techniques for ATSPs.
Removing the dummy cities from the resulting tour gives the  solution for the
original ATSP.


\subsection{Finding exact solutions for the TSP}
\label{sec:exact}

Finding the exact solution to a TSP with $n$ cities requires to check $(n-1)!$
possible tours. To evaluate all possible tours is infeasible for even small TSP
instances. To find the optimal tour
\cite{Held1962} presented the following
\emph{dynamic programming} formulation: Given a subset of city indices
(excluding the first city)
$S \subset \{2, 3, \dots, n\}$ and $l \in S$, let $d^*(S, l)$ denote the length of the
shortest path from city $1$ to city $l$, visiting all cities in $S$ in-between.
For $S = \{l\}$, $d^*(S,l)$ is defined as $d_{1l}$. The shortest path for
larger sets with $|S| > 1$ is

\begin{equation}
    d^*(S,l) = \mathrm{min}_{m \in S\setminus\{l\}}\Bigl( d^*(S\setminus\{l\},m) + d_{ml}\Bigl).
    %\text{ for } |S| > 1.
\end{equation}

Finally, the minimal tour length for a complete tour which includes returning
to city $1$ is

\begin{equation}
    d^{**} = \mathrm{min}_{l \in \{2,3,\dots,n\}}\Bigl( d^*(\{2,3,\dots,n\}, l)
    + d_{l1}\Bigl).
\end{equation}

Using the last two equations, the quantities $d^*(S,l)$ can be computed
recursively and the minimal tour length $d^{**}$ can be found.  In a
second step, the optimal permutation $\pi = \{1, i_2, i_3,\dots,i_n\}$
of city indices $1$ through $n$ can be computed in reverse order,
starting with $i_n$ and working successively back to $i_2$. The
procedure exploits the fact that a permutation $\pi$ can only be
optimal, if

\begin{equation}
    d^{**} = d^*(\{2,3,\dots,n\}, i_n) + d_{i_n1}
\end{equation}
and, for $2 \le p \le n-1$,
\begin{equation}
    d^*(\{i_2, i_3,\dots, i_p, i_{p+1}\}, i_{p+1}) =
    d^*(\{i_2,i_3,\dots,i_p\}, i_p) + d_{i_pi_{p+1}}.
\end{equation}

The space complexity of storing the values for all $d^*(S,l)$ is
$(n-1)2^{n-2}$ which severely restricts the dynamic programming
algorithm to TSP problems of small sizes.  However, for very small TSP
instances this approach is fast and efficient.
%\marginpar{time complexity if order $n^22^n$. Check!}

A different method, which can deal with larger instances, uses a relaxation of
the linear programming problem presented in Section~\ref{sec:formulations} and
iteratively tightens the relaxation till a solution is found. This general
method for solving linear programming problems with complex and large
inequality systems is called \emph{cutting plane method} and was introduced by
\cite{Dantzig1954}.

Each iteration begins with using instead of the original linear
inequality description $\mathcal{S}$ the relaxation $A\mathbf{x} \le b$,
where the polyhedron $P$ defined by the relaxation contains
$\mathcal{S}$ and is bounded.  The optimal solution $\mathbf{x}^*$ of
the relaxed problem can be obtained using standard linear programming
solvers. If the $\mathbf{x}^*$ found belongs to $\mathcal{S}$, the
optimal solution of the original problem is obtained, otherwise, a
linear inequality can be found which is satisfied by all points in
$\mathcal{S}$ but violated by $\mathbf{x}^*$.  Such an inequality is
called a cutting plane or cut.  A family of such cutting planes can be
added to the inequality system $A\mathbf{x} \le b$ to obtain a tighter
relaxation for the next iteration.

If no further cutting planes can be found or the improvement in the
objective function due to adding cuts gets very small, the problem is
branched into two sub-problems which can be minimized
separately. Branching is done iteratively which leads to a binary tree
of sub-problems. Each sub-problem is either solved without further
branching or is found to be irrelevant because its relaxed version
already produces a longer path than a solution of another sub-problem.
This method is called \emph{branch-and-cut}~\citep{Padberg1990} which is
a variation of the well known \emph{branch-and-bound}~\citep{Land1960}
procedure.

The initial polyhedron $P$ used by \cite{Dantzig1954} contains all
vectors $\mathbf{x}$ for which all $x_e \in \mathbf{x}$ satisfy $0 \le
x_e \le 1$ and in the resulting tour each city is linked to exactly two
other cities.  Various separation algorithms for finding subsequent cuts
to prevent sub-tours (\emph{sub-tour elimination inequalities}) and to
ensure an integer solution \citep[\emph{Gomory cuts;}][]{Gomory1963}
were developed over time. The currently most efficient implementation of
this method is \emph{Concorde} described in~\cite{Applegate2000}.

\subsection{Heuristics for the TSP}
\label{sec:heuristics}

The NP-completeness of the TSP already makes it more time efficient for
small-to-medium size TSP instances to rely on heuristics in case a good
but not necessarily optimal solution is sufficient.  TSP heuristics
typically fall into two groups, tour construction heuristics which
create tours from scratch and tour improvement heuristics which use
simple local search heuristics to improve existing tours.

In the following we will only discuss heuristics available in \pkg{TSP}, for a
comprehensive overview of the multitude of TSP heuristics including an
experimental comparison, we refer the reader to the book chapter by
\cite{Johnson2002}.

\subsubsection{Tour construction heuristics}
The implemented tour construction heuristics are the nearest neighbor algorithm
and the insertion algorithms.

\paragraph{Nearest neighbor algorithm.}

The nearest neighbor algorithm~\citep{Rosenkrantz1977} follows a very
simple greedy procedure:  The algorithm starts with a tour containing a
randomly chosen city and then always adds to the last city in the tour
the nearest not yet visited city. The algorithm stops when all cities
are on the tour.

An extension to this algorithm is to repeat it with each city as the
starting point and then return the best tour found.  This heuristic is
called repetitive nearest neighbor.

\paragraph{Insertion algorithms.}

All insertion algorithms~\citep{Rosenkrantz1977} start with a tour
consisting of an arbitrary city and then choose in each step a city $k$
not yet on the tour.  This city is inserted into the existing tour
between two consecutive cities $i$ and $j$, such that the insertion cost
(i.e., the increase in the tour's length)
$$d(i,k) + d(k,j) - d(i,j)$$
is minimized.  The algorithms stop when all cities are on the tour.

The insertion algorithms differ in the way the city to be inserted next
is chosen.  The following variations are implemented:

\begin{description}
\item[Nearest insertion] The city $k$ is chosen in each step as
the city which is nearest to a city on the tour.

\item[Farthest insertion] The city $k$ is chosen in each step as
the city which is farthest from any of the cities on the tour.

\item[Cheapest insertion] The city $k$ is chosen in each step
such that the cost of inserting the new city is minimal.

\item[Arbitrary insertion] The city $k$ is chosen randomly from all cities
not yet on the tour.

\end{description}


The nearest and cheapest insertion algorithms correspond to the minimum
spanning tree algorithm by \cite{Prim1957}. Adding a city to a partial tour
corresponds to adding an edge to a partial spanning tree. For TSPs with
distances obeying the triangular inequality, the equality to minimum spanning
trees provides a theoretical upper bound for the two algorithms of twice the
optimal tour length.

The idea behind the farthest insertion algorithm is to link cities far
outside into the tour first to establish an outline of the whole tour
early. With this change, the algorithm cannot be directly related to
generating a minimum spanning tree and thus the upper bound stated above
cannot be guaranteed.  However, it can was shown that the algorithm
generates tours which approach $2/3$ times the optimal tour
length~\citep{Johnson1985}.

\subsubsection{Tour improvement heuristics}

Tour improvement heuristics are simple local search heuristics which try to
improve an initial tour. A comprehensive treatment of the topic
can be found in the book chapter by \cite{Rego2002}.

\paragraph{$k$-Opt heuristics.} The idea is to define a neighborhood structure
on the set of all admissible tours. Typically, a tour $t'$ is a neighbor of
another tour $t$ if $t'$ can be obtained from $t$ by deleting $k$ edges
and replacing them by a set of different feasible edges (a $k$-Opt move).  In
such a structure, the tour can iteratively be improved by always moving from
one tour to its best neighbor till no further improvement is possible. The
resulting tour represents a local optimum which is called $k$-optimal.

Typically, $2$-Opt~\citep{Croes1958} and $3$-Opt~\citep{Lin1965} heuristics
are used in practice.

\paragraph{Lin-Kernighan heuristic.}

This heuristic~\citep{Lin1973} does not use a fixed value for $k$ for
its $k$-Opt moves, but tries to find the best choice of $k$ for each
move.  The heuristic uses the fact that each $k$-Opt move can be
represented as a sequence of $2$-Opt moves.  It builds up a sequence of
$2$-Opt moves, checking after each additional move whether a stopping
rule is met. Then the part of the sequence which gives the best
improvement is used. This is equivalent to a choice of one $k$-Opt move
with variable $k$. Such moves are used till a local optimum is reached.

By using full backtracking, the optimal solution can always be found, but the
running time would be immense. Therefore, only limited backtracking is allowed
in the procedure, which helps to find better local optima or even the optimal
solution. Further improvements to the procedure are described
by \cite{Lin1973}.


\section{Computational infrastructure: the TSP package}
\label{sec:infrastructure}

In package~\pkg{TSP}, a traveling salesperson problem is
defined by an object of class \class{TSP} (symmetric) or \class{ATSP}
(asymmetric). \func{solve\_TSP}
is used to find a solution, which is represented by
an object of class \class{TOUR}. Figure~\ref{fig:overview} gives an overview
of this infrastructure.

\begin{figure}
    \centering
    \includegraphics[width=14cm]{overview}
    \caption{An overview of the classes in \pkg{TSP}.}
    \label{fig:overview}
\end{figure}


\class{TSP} objects can be created from a distance matrix (a
\class{dist} object) or a symmetric matrix using the creator function
\func{TSP} or coercion with \func{as.TSP}.  Similarly, \class{ATSP}
objects are created by \func{ATSP} or \func{as.ATSP} from square
matrices representing the distances.  In the creation process, labels
are taken and stored as city names in the object or can be explicitly
given as arguments to the creator functions.  Several methods are
defined for the classes:
\begin{itemize}
 \item \func{print} displays basic information about the problem (number
  of cities and the distance measure employed).
 \item \func{n\_of\_cities} returns the number of cities.
 \item \func{labels} returns the city names.
 \item \func{image} produces a shaded matrix plot of the distances
  between cities. The order of the cities can be specified as the
  argument \code{order}.
\end{itemize}

Internally, an object of class \class{TSP} is a \class{dist} object with
an additional class attribute and, therefore, if needed, can be coerced
to \class{dist} or to a matrix.  An \class{ATSP} object is represented
as a square matrix.  Obviously, asymmetric TSPs are more general than
symmetric TSPs, hence, symmetric TSPs can also be represented as
asymmetric TSPs.  To formulate an asymmetric TSP as a symmetric TSP with
double the number of cities (see Section \ref{sec:manipulations}),
\func{reformulate\_ATSP\_as\_TSP} is provided.  This function creates
the necessary dummy cities and adapts the distance matrix accordingly.

A popular format to save TSP descriptions to disk which is supported by
most TSP solvers is the format used by \emph{TSPLIB}, a library of
sample instances of the TSP maintained by \cite{Reinelt2004}. The
\pkg{TSP} package provides \func{read\_TSPLIB} and \func{write\_TSPLIB}
to read and save symmetric and asymmetric TSPs in TSPLIB format.

Class \class{TOUR} represents a solution to a TSP by an integer
permutation vector containing the ordered indices and labels of the
cities to visit.  In addition, it stores an attribute indicating the
length of the tour.  Again, suitable \func{print} and \func{labels}
methods are provided.  The raw permutation vector (i.e., the order in
which cities are visited) can be obtained from a tour using
\func{as.integer}.  With \func{cut\_tour}, a circular tour can be split
at a specified city resulting in a path represented by a vector of city
indices.

The length of a tour can always be calculated using \func{tour\_length}
and specifying a TSP and a tour. Instead of the tour, an integer
permutation vector calculated outside the \pkg{TSP} package can be used
as long as it has the correct length.

All TSP solvers in \pkg{TSP} can be used with the simple common interface:

\begin{quote}
\code{solve\_TSP(x, method, control)}
\end{quote}
where \code{x} is the TSP to be solved, \code{method} is a character
string indicating the method used to solve the TSP and \code{control}
can contain a list with additional information used by the solver.
The available algorithms are shown in Table~\ref{tab:methods}.

\begin{table}
    \caption{Available algorithms in \pkg{TSP}.}
    \label{tab:methods}
    \centering
    \begin{tabular}{llc}
    \hline
    \textbf{Algorithm} & \textbf{Method argument} & \textbf{Applicable to} \\
    \hline
    Nearest neighbor algorithm & \code{"nn"} & TSP/ATSP \\
    Repetitive nearest neighbor algorithm & \code{"repetitive\_nn"}
            & TSP/ATSP \\
    Nearest insertion & \code{"nearest\_insertion"} & TSP/ATSP  \\
    Farthest insertion & \code{"farthest\_insertion"} & TSP/ATSP  \\
    Cheapest insertion & \code{"cheapest\_insertion"} & TSP/ATSP  \\
    Arbitrary insertion & \code{"arbitrary\_insertion"} & TSP/ATSP  \\
    Concorde TSP solver & \code{"concorde"} & TSP  \\
    2-Opt improvement heuristic &  \code{"two\_opt"}  & TSP/ATSP \\
    Chained Lin-Kernighan &  \code{"linkern"}  & TSP \\
\hline
\end{tabular}
\end{table}

All algorithms except the Concorde TSP solver and the Chained
Lin-Kernighan heuristic (a Lin-Kernighan variation described in
\cite{Applegate2003}) are included in the package and distributed under
the GNU Public License (GPL). For the Concorde TSP solver and the
Chained Lin-Kernighan heuristic only a simple interface (using
\func{write\_TSPLIB}, calling the executable and reading back the
resulting tour) is included in \pkg{TSP}.  The executable itself is part
of the Concorde distribution, has to be installed separately and is
governed by a different license which allows only for academic use.  The
interfaces are included since
Concorde~\citep{Applegate2000,Applegate2006} is currently one of the
best implementations for solving symmetric TSPs based on the
branch-and-cut method discussed in section~\ref{sec:exact}.  In May
2004, Concorde was used to find the optimal solution for the TSP of
visiting all 24,978 cities in Sweden.  The computation was carried out
on a cluster with 96 Xeon 2.8 GHz nodes and took in total almost 100 CPU
years.

\section{Examples}
\label{sec:examples}

In this section we provide some examples for the use of
package~\pkg{TSP}.  We start with a simple example of how to use the
interface of the TSP solver to compare different heuristics.  Then we
show how to solve related tasks, using the Hamiltonian shortest path
problem as an example.  Finally, we give an example of clustering using
the \pkg{TSP} package.  An additional application can be found
in package \pkg{seriation}~\citep{TSP:Hahsler+Buchta+Hornik:2006} which uses
the TSP solvers from \pkg{TSP} to order (seriate) objects given a proximity
matrix.

\subsection{Comparing some heuristics}

In the following example, we use several heuristics to find a short path
in the \code{USCA50} data set which contains the distances between the
first 50 cities in the \code{USCA312} data set. The \code{USCA312} data
set contains the distances between 312 cities in the USA and Canada
coded as a symmetric TSP.  The smaller data set is used here, since some
of the heuristic solvers employed are rather slow.

<<>>=
library("TSP")
data("USCA50")
USCA50
@


We calculate tours using different heuristics and store the results in
the list \code{tours}. As an example, we show the first tour which
displays the method employed, the number of cities involved and the tour
length.  All tour lengths are compared using the dot chart in
Figure~\ref{fig:dotchart}.  For the chart, we add a point for the
optimal solution which has a tour length of 14497. The optimal solution
can be found using Concorde (\code{method = "concorde"}).  It is omitted
here, since Concorde has to be installed separately.

<<echo=FALSE>>=
set.seed(1234)
@


<<fig=TRUE, include=FALSE, label=dotchart_USCA50, width=7, height=4>>=
methods <- c("nearest_insertion", "farthest_insertion", "cheapest_insertion",
    "arbitrary_insertion", "nn", "repetitive_nn", "two_opt")

tours <- sapply(methods, FUN = function(m) solve_TSP(USCA50, method = m),
                simplify = FALSE)
## tours$concorde  <- solve_TSP(tsp, method = "concorde")

tours[[1]]

dotchart(sort(c(sapply(tours, tour_length), optimal = 14497)),
xlab = "tour length", xlim = c(0, 20000))
@

\begin{figure}
\centering
\includegraphics[width=11cm, trim=0 10 0 0]{TSP-dotchart_USCA50}
\caption{Comparison of the tour lengths for the USCA50 data set.}
\label{fig:dotchart}
\end{figure}


\subsection{Finding the shortest Hamiltonian path}

The problem of finding the shortest Hamiltonian path through a graph
(i.e., a path which visits each node in the graph exactly once) can be
transformed into the TSP with cities and distances representing the
graphs vertices and edge weights, respectively~\citep{Garfinkel1985}.

Finding the shortest Hamiltonian path through all cities disregarding
the endpoints can be achieved by inserting a `dummy city' which has a
distance of zero to all other cities. The position of this city in the
final tour represents the cutting point for the path. In the following
we use a heuristic to find a short path in the \code{USCA312} data set.
Inserting dummy cities is performed in \pkg{TSP} by
\func{insert\_dummy}.

<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
library("TSP")
data("USCA312")
tsp <- insert_dummy(USCA312, label = "cut")
tsp
@

The TSP now contains an additional dummy city and we can try to solve
this TSP.

<<>>=
tour <- solve_TSP(tsp, method="farthest_insertion")
tour
@

Since the dummy city has distance zero to all other cities, the path length is
equal to the tour length reported above. The path starts with the first city in
the list after the `dummy' city  and ends with the city right before it.
We use \func{cut\_tour} to create a path and show the
first and last 6 cities on it.

<<>>=
path <- cut_tour(tour, "cut")
head(labels(path))
tail(labels(path))
@

The tour found in the example results in a path from Lihue on Hawaii to
Prince Rupert in British Columbia. Such a tour can also be visualized
using a scatter plot with a map from package \pkg{maps}. Note that the \code{if} statement and the \code{plot} in the \code{else} part in the following is not needed and only checks if the map package is installed when building this document.

<<fig=TRUE, include=FALSE, label=map1, width=8, height=8>>=
if(require(maps)) {
  library("maps")

  data("USCA312_GPS")

  plot_path <- function(path) {
    plot((USCA312_GPS[, c("long", "lat")]), cex = .3, col = "red")
    map("world", col = "gray", add = TRUE)
    lines(USCA312_GPS[, c("long", "lat")][path,], col = "black")
    points(USCA312_GPS[c(head(path, 1), tail(path, 1)),  c("long", "lat")], pch = 19, col = "black")
  }

  plot_path(path)

} else {
    plot(NA, xlim= c(0,1), ylim = c(0,1))
    text(.5, .5, "Suggested packages not available")
}
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map1}
\caption{A ``short'' Hamiltonian path for the USCA312 dataset.}
\label{fig:map1}
\end{figure}

The map containing the path is presented in Figure~\ref{fig:map1}.  It
has to be mentioned that the path found by the used heuristic is
considerable longer than the optimal path found by Concorde with a
length of $34928$, illustrating the power of modern TSP algorithms.

For the following two examples, we indicate how the distance matrix
between cities can be modified to solve related shortest Hamiltonian
path problems.  These examples serve as illustrations of how
modifications can be made to transform different problems into a TSP.

The first problem is to find the shortest Hamiltonian path starting with
a given city. In this case, all distances to the selected city are set
to zero, forcing the evaluation of all possible paths starting with this
city and disregarding the way back from the final city in the tour.  By
modifying the distances the symmetric TSP is changed into an asymmetric
TSP (ATSP) since the distances between the starting city and all other
cities are no longer symmetric.

As an example, we choose New York as the starting city.  We transform
the data set into an ATSP and set the column corresponding to New York
to zero before solving it.  Thus, the distance to return from the last
city in the path to New York does not contribute to the path length.  We
use the nearest neighbor heuristic to calculate an initial tour which is
then improved using $2$-Opt moves and cut at New York to create a path.


<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
atsp <- as.ATSP(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
atsp[, ny] <- 0
initial_tour <- solve_TSP(atsp, method="nn")
initial_tour
tour <- solve_TSP(atsp, method ="two_opt", control = list(tour = initial_tour))
tour
path <- cut_tour(tour, ny, exclude_cut = FALSE)

head(labels(path))
tail(labels(path))
@
<<fig=TRUE, include=FALSE, label=map2, width=8, height=8>>=
plot_path(path)
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map2}
\caption{A Hamiltonian path for the USCA312 dataset starting in New York.}
\label{fig:map2}
\end{figure}

The found path is presented in Figure~\ref{fig:map2}. It begins with New
York and cities in New Jersey and ends in a city in Manitoba, Canada.
%The path shows the typical behavior
%of the nearest neighbor heuristic with first connecting the cities close by and
%then making rather big ``jumps'' for the final cities.

Concorde and many advanced TSP solvers can only solve symmetric TSPs. To
use these solvers, we can formulate the ATSP as a TSP using
\func{reformulate\_ATSP\_as\_TSP} which introduces a dummy city for each
city (see Section~\ref{sec:manipulations}).

<<>>=
tsp <- reformulate_ATSP_as_TSP(atsp)
tsp
@

After finding a tour for the TSP, the dummy cities are removed again giving the
tour for the original ATSP.
Note that the tour needs to be reversed if the
dummy cities appear before and not after the original cities in the solution of
the TSP.  The following code is not executed here, since it takes several
minutes to execute and Concorde has to be installed separately.  Concorde finds
the optimal solution with a length of $36091$.


<<eval=FALSE>>=
tour <- solve_TSP(tsp, method = "concorde")
tour <- as.TOUR(tour[tour <= n_of_cities(atsp)])
@


Finding the shortest Hamiltonian path which ends in a given city can be
achieved likewise by setting the row in the distance matrix which
corresponds to this city to zero.

For finding the shortest Hamiltonian path we can also restrict both end
points.  This problem can be transformed to a TSP by replacing the two
cities by a single city which contains the distances from the start
point in the columns and the distances to the end point in the
rows. Obviously this is again an asymmetric TSP.

For the following example, we are only interested in paths starting in
New York and ending in Los Angeles. Therefore, we remove the two cities
from the distance matrix, create an asymmetric TSP and insert a dummy
city called \code{"LA/NY"}. The distances from this dummy city are
replaced by the distances from New York and the distances towards are
replaced by the distances towards Los Angeles.


<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
m <- as.matrix(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
la <- which(labels(USCA312) == "Los Angeles, CA")

atsp <- ATSP(m[-c(ny,la), -c(ny,la)])
atsp <- insert_dummy(atsp, label = "LA/NY")

la_ny <- which(labels(atsp) == "LA/NY")
atsp[la_ny, ] <- c(m[-c(ny,la), ny], 0)
atsp[, la_ny] <- c(m[la, -c(ny,la)], 0)
@


We use the nearest insertion heuristic.
<<>>=
tour <- solve_TSP(atsp, method ="nearest_insertion")
tour

path_labels <- c("New York, NY",
    labels(cut_tour(tour, la_ny)), "Los Angeles, CA")
path_ids <- match(path_labels, labels(USCA312))

head(path_labels)
tail(path_labels)
@
<<fig=TRUE, include=FALSE, label=map3, width=8, height=8>>=
plot_path(path_ids)
@


The path jumps from New York to cities in Ontario and it passes through
cities in California and Nevada before ending in Los Angeles. The path
displayed in Figure~\ref{fig:map3} contains multiple crossings which
indicate that the solution is suboptimal. The optimal solution generated
by reformulating the problem as a TSP and using Concorde has only a tour
length of $38489$.

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 30 0 0]{TSP-map3}
\caption{A Hamiltonian path for the USCA312 dataset starting in New York
  and ending in Los Angles.}
\label{fig:map3}
\end{figure}


\subsection{Rearrangement clustering}
Solving a TSP to obtain a clustering was suggested several times in the
literature \citep[see, e.g.,][]{Lenstra1974, Alpert1997, Johnson2004}.  The idea
is that objects in clusters are visited in consecutive order and from one
cluster to the next larger ``jumps'' are necessary.  \cite{Climer2006} call
this type of clustering \emph{rearrangement clustering} and suggest to
automatically find the cluster boundaries of $k$ clusters by adding $k$
\emph{dummy cities} which have constant distance $c$ to all other cities and
are infinitely far from each other. In the optimal
solution of the TSP, the dummy cities must separate the most distant cities and
thus represent optimal boundaries for $k$ clusters.

For the following example, we use the well known iris data set. Since we
know that the dataset contains three classes denoted by the variable
\code{Species}, we insert three dummy cities into the TSP for the iris
data set and perform rearrangement clustering using the default method
(nearest insertion algorithm).  Note that this algorithm does not find
the optimal solution and it is not guaranteed that the dummy cities will
present the best cluster boundaries.
%\marginpar{FIXME: Was sagt Concorde dazu?}

<<echo=FALSE>>=
set.seed(4444)
@
<<>>=
data("iris")
tsp <- TSP(dist(iris[-5]), labels = iris[, "Species"])
tsp_dummy <- insert_dummy(tsp, n = 3, label = "boundary")
tour <- solve_TSP(tsp_dummy)
@

Next, we plot the TSP's permuted distance matrix using shading to
represent distances. The result is displayed as
Figure~\ref{fig:clustering}. Lighter areas represent larger
distances. The additional red lines represent the positions of the dummy
cities in the tour, which mark the cluster boundaries obtained.


<<fig=TRUE, include=FALSE, label=clustering>>=
## plot the distance matrix
image(tsp_dummy, tour, xlab = "objects", ylab ="objects")

## draw lines where the dummy cities are located
abline(h = which(labels(tour)=="boundary"), col = "red")
abline(v = which(labels(tour)=="boundary"), col = "red")
@


\begin{figure}
\centering
\includegraphics[width=9cm, height=9cm, trim=0 20 0 0]{TSP-clustering}
\caption{Result of rearrangement clustering using three dummy cities and the
nearest insertion algorithm on the iris data set.}
\label{fig:clustering}
\end{figure}


One pair of red horizontal and vertical lines exactly separates the
darker from lighter areas.  The second pair occurs inside the larger
dark block.  We can look at how well the partitioning obtained fits the
structure in the data given by the species field in the data set.  Since
we used the species as the city labels in the TSP, the labels in the
tour represent the partitioning with the dummy cities named `boundary'
separating groups.  The result can be summarized based on the run length
encoding of the obtained tour labels:

<<>>=
out <- rle(labels(tour))
data.frame(Species = out$values,
           Lenghts = out$lengths,
           Pos = cumsum(out$lengths))
@


One boundary perfectly splits the iris data set into a group containing
only examples of species `Setosa' and a second group containing examples
for `Virginica' and `Versicolor'.  However, the second boundary only
separates several examples of species `Virginica' from other examples of
the same species.  Even in the optimal tour found by Concorde, this
problem occurs. The reason why the rearrangement clustering fails to
split the data into three groups is the closeness between the groups
`Virginica' and `Versicolor'.  To inspect this problem further, we can
project the data points on the first two principal components of the
data set and add the path segments which resulted from solving the TSP.

<<fig=TRUE, include=FALSE, label=clustering2>>=
prc <- prcomp(iris[1:4])
plot(prc$x, pch = as.numeric(iris[,5]), col =  as.numeric(iris[,5]))

paths <- cut_tour(tour, cut = "boundary")
for(p in paths) lines(prc$x[p, ])
@

\begin{figure}
\centering
\includegraphics[width=10cm, trim=0 20 0 0]{TSP-clustering2}
\caption{The 3 path segments representing a rearrangement clustering of the
iris data set.  The data points are projected on the set's first two principal
components.  The three species are represented by different markers and
colors.}
\label{fig:clustering2}
\end{figure}

The result in shown in Figure~\ref{fig:clustering2}.  The three species
are identified by different markers and all points connected by a single
path represent a cluster found.  Clearly, the two groups to the right
side of the plot are too close to be separated correctly by using just
the distances between individual points.  This problem is similar to the
\emph{chaining effect} known from hierarchical clustering using the
single-linkage method.

\section{Conclusion}
\label{sec:conclusion}

In this paper we presented the R extension package \pkg{TSP} which
implements an infrastructure to handle and solve TSPs. The package
introduces classes for problem descriptions (\class{TSP} and
\class{ATSP}) and for the solution (\class{TOUR}).  Together with a
simple interface for solving TSPs, it allows for an easy and transparent
usage of the package.

With the interface to Concorde, \pkg{TSP} also can use a state of the
art implementation which efficiently computes exact solutions using
branch-and-cut.

\section*{Acknowledgments}

The authors of this paper want to thank Roger Bivand for providing the
code to correctly draw tours and paths on a projected map.


%
%\bibliographystyle{abbrvnat}
\bibliography{TSP}
%
\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsp_concorde.R
\name{Concorde}
\alias{Concorde}
\alias{concorde}
\alias{concorde_path}
\alias{concorde_help}
\alias{linkern_help}
\title{Using the Concorde TSP Solver}
\usage{
concorde_path(path)

concorde_help()

linkern_help()
}
\arguments{
\item{path}{a character string with the path to the directory where the
executables are installed.}
}
\value{
\code{concorde_path()} returns the path to the executable. Others functions: Nothing.
}
\description{
The Concorde TSP Solver package contains several solvers.  Currently,
interfaces to the Concorde solver (Applegate et al. 2001), one of the most
advanced and fastest TSP solvers using branch-and-cut, and the Chained
Lin-Kernighan (Applegate et al. 2003) implementation are provided in
\pkg{TSP}. Concorde can solve \link{TSP}s and \link{ETSP}s directly. \link{ATSP}s are
reformulated as larger TSP's and then solved.
}
\section{Installation of Concorde}{
The Concorde TSP Solver is freely available for academic research.
It is not included in the \pkg{TSP} R package and has
to be obtained separately from the
\href{https://www.math.uwaterloo.ca/tsp/concorde/downloads/downloads.htm}{Concorde download page}.
Either download the precompiled executables and place them in a suitable
directory (make sure they are executable), or you can get the source code and
compile the program on your own. \pkg{TSP} needs to know where the executables are.
There are two options:
\enumerate{
\item use \code{concorde_path()} to set the path to the
directory containing the executables for concorde and linkern, or
\item make
sure that the executables are in the search path stored in the \code{PATH}
environment variable (see \code{\link[=Sys.setenv]{Sys.setenv()}}).
}
}

\section{Using Concorde for \code{solve_TSP()}}{
\code{\link[=solve_TSP]{solve_TSP()}} uses \code{\link[=write_TSPLIB]{write_TSPLIB()}} to write the TSP for
Concorde and tries to find the appropriate \code{precision} value (digits
after the decimal point) to convert the provided distances into the needed
integer value range. The \code{precision} value can also be specified in
\code{control} in \code{\link[=solve_TSP]{solve_TSP()}} with method Concorde. Warning
messages will alert the user if the conversion to integer values results
into rounding errors that are worse then what is specified in the
\code{precision} control parameter.

To get a list of all available command line options which can be used via
the \code{clo} option for \code{solve_TSP} use \code{concorde_help()} and
\code{linkern_help()}.  Several options (\option{-x}, \option{-o},
\option{-N}, \option{-Q}) are not available via \code{\link[=solve_TSP]{solve_TSP()}} since they
are used by the interface.

If Concorde takes too long, then you can interrupt \code{\link[=solve_TSP]{solve_TSP()}}
using \code{Esc/CTRL-C}. On most operating systems, this will also
terminate the Concorde executable. If Concorde keeps running, then you can
kill the 'concorde' process via your operating system.
}

\examples{

\dontrun{
## see if Concorde is correctly installed
concorde_path()


## set path to the Concorde executible if it is not in the search PATH
## Example:
## concorde_path("~/concorde/")

concorde_help()

data("USCA312")

## run Concorde in verbose mode (-v) with fast cuts only (-V)
## Note: use the contol parameter verbose = FALSE to supress Concorde's output
solve_TSP(USCA312, method = "concorde", control = list(clo = "-v -V"))
}

}
\references{
Concorde home page,
\url{https://www.math.uwaterloo.ca/tsp/concorde/}

David Applegate, Robert Bixby, Vasek Chvatal, William Cook (2001): TSP cuts
which do not conform to the template paradigm, Computational Combinatorial
Optimization, M. Junger and D. Naddef (editors), Springer-Verlag.

David Applegate and William Cook and Andre Rohe (2003): Chained
Lin-Kernighan for Large Traveling Salesman Problems, \emph{INFORMS Journal
on Computing}, \bold{15}, 82--92.
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{documentation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETSP.R
\name{ETSP}
\alias{ETSP}
\alias{as.ETSP}
\alias{as.ETSP.matrix}
\alias{as.ETSP.data.frame}
\alias{as.TSP.ETSP}
\alias{as.matrix.ETSP}
\alias{print.ETSP}
\alias{n_of_cities.ETSP}
\alias{labels.ETSP}
\alias{image.ETSP}
\alias{plot.ETSP}
\title{Class ETSP -- Euclidean traveling salesperson problem}
\usage{
ETSP(x, labels = NULL)

as.ETSP(x)

\method{as.ETSP}{matrix}(x)

\method{as.ETSP}{data.frame}(x)

\method{as.TSP}{ETSP}(x)

\method{as.matrix}{ETSP}(x, ...)

\method{print}{ETSP}(x, ...)

\method{n_of_cities}{ETSP}(x)

\method{labels}{ETSP}(object, ...)

\method{image}{ETSP}(x, order, col = gray.colors(64), ...)

\method{plot}{ETSP}(x, y = NULL, tour = NULL, tour_lty = 2, tour_col = 2, labels = TRUE, ...)
}
\arguments{
\item{x, object}{an object (data.frame or matrix) to be converted into a
\code{ETSP} or, for the methods, an object of class \code{ETSP}.}

\item{labels}{logical; plot city labels.}

\item{...}{further arguments are passed on.}

\item{order}{order of cities for the image as an integer vector or an object
of class \link{TOUR}.}

\item{col}{color scheme for image.}

\item{tour, y}{a tour to be visualized.}

\item{tour_lty, tour_col}{line type and color for tour.}
}
\value{
\itemize{
\item \code{ETSP()} returns \code{x} as an object of class \code{ETSP}.
\item \code{n_of_cities()} returns the number of cities in \code{x}.
\item \code{labels()} returns a vector with the names of the cities in \code{x}.
}
}
\description{
Constructor to create an instance of a Euclidean traveling salesperson
problem (TSP) represented by city coordinates and some auxiliary methods.
}
\details{
Objects of class \code{ETSP} are internally represented as a \code{matrix}
objects (use \code{as.matrix()} to get the \code{matrix} object).
}
\examples{
## create a random ETSP
n <- 20
x <- data.frame(x = runif(n), y = runif(n), row.names = LETTERS[1:n])
etsp <- ETSP(x)
etsp

## use some methods
n_of_cities(etsp)
labels(etsp)

## plot ETSP and solution
tour <- solve_TSP(etsp)
tour

plot(etsp, tour, tour_col = "red")

# plot with custom labels
plot(etsp, tour, tour_col = "red", labels = FALSE)
text(etsp, paste("City", rownames(etsp)), pos = 1)
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solve_TSP.R
\name{solve_TSP}
\alias{solve_TSP}
\alias{solve_TSP.TSP}
\alias{solve_TSP.ATSP}
\alias{solve_TSP.ETSP}
\title{TSP solver interface}
\usage{
solve_TSP(x, method = NULL, control = NULL, ...)

\method{solve_TSP}{TSP}(x, method = NULL, control = NULL, ...)

\method{solve_TSP}{ATSP}(x, method = NULL, control = NULL, as_TSP = FALSE, ...)

\method{solve_TSP}{ETSP}(x, method = NULL, control = NULL, ...)
}
\arguments{
\item{x}{a TSP problem.}

\item{method}{method to solve the TSP (default: "arbitrary insertion"
construction followed by the "two_opt" improvement heuristic.}

\item{control}{a list of arguments passed on to the TSP solver selected by
\code{method}.}

\item{...}{additional arguments are added to \code{control}.}

\item{as_TSP}{should the ATSP reformulated as a TSP for the solver?}
}
\value{
An object of class \link{TOUR}.
}
\description{
Common interface to all TSP solvers in this package.
}
\section{TSP Methods}{
TSP supports tour construction methods, tour improvement heuristics and
an interface to Concorde (a state-of-the-art external solver).
Tour construction methods create a tour from scratch, typically using a
construction heuristic. Tour improvement heuristics take an existing tour and
try to improve it. If no initial tour is provided for the improvement
heuristic, then an initial tour (e.g., a random tour) are automatically created.

Most heuristic methods accept the following extra control parameters in addition
the parameters specified below:
\itemize{
\item "rep": an integer indicating how many replications (random restarts) should be performed.
The best result is returned. The replications can be performed in parallel
(see Section Parallel Execution Support).
\item "two_opt": a logical indicating if two-opt refinement should be performed on the
constructed tour. This is just for convenience so a constructive heuristic can be
followed by two-opt refinement in a single call.
\item "verbose": a logical. Most implementations provide verbose output to monitor progress.
This argument can also be helpful to see what heuristics are used and what the
default parameters are.
}
\subsection{Tour Construction Methods}{

These methods construct tours from scratch.
The available tour construction methods are:
\itemize{
\item \strong{"identity", "random"} return a tour representing the order in the data
(identity order) or a random order. [TSP, ATSP]
\item \strong{"nearest_insertion", "farthest_insertion", "cheapest_insertion", "arbitrary_insertion"}
Nearest, farthest, cheapest and
arbitrary insertion algorithms for a symmetric and asymmetric TSP
(Rosenkrantz et al. 1977). [TSP, ATSP]

The distances between cities are stored in a distance matrix \eqn{D} with
elements \eqn{d(i,j)}.  All insertion algorithms start with a tour
consisting of an arbitrary city and choose in each step a city \eqn{k} not
yet on the tour. This city is inserted into the existing tour between two
consecutive cities \eqn{i} and \eqn{j}, such that \deqn{d(i,k) + d(k,j) -
  d(i,j)} is minimized. The algorithms stops when all cities are on the tour.

The nearest insertion algorithm chooses city \eqn{k} in each step as the
city which is \emph{nearest} to a city on the tour.

For farthest insertion, the city \eqn{k} is chosen in each step as the city
which is \emph{farthest} to any city on the tour.

Cheapest insertion chooses the city \eqn{k} such that the cost of inserting
the new city (i.e., the increase in the tour's length) is minimal.

Arbitrary insertion chooses the city \eqn{k} randomly from all cities not
yet on the tour.

Nearest and cheapest insertion tries to build the tour using cities which
fit well into the partial tour constructed so far.  The idea behind behind
farthest insertion is to link cities far away into the tour fist to
establish an outline of the whole tour early.

Additional control options:
\itemize{
\item "start" index of the first city (default: a random city).
}
\item \strong{"nn", "repetitive_nn"} Nearest neighbor and repetitive
nearest neighbor algorithms for symmetric and asymmetric TSPs (Rosenkrantz
et al. 1977). [TSP, ATSP]

The algorithm starts with a tour containing a random city. Then the
algorithm always adds to the last city on the tour the nearest not yet
visited city. The algorithm stops when all cities are on the tour.

Repetitive nearest neighbor constructs a nearest neighbor tour for each city
as the starting point and returns the shortest tour found.

Additional control options:
\itemize{
\item "start" index of the first city (default: a random city).
}
}
}

\subsection{Tour Improvement Heuristics}{

Tour improvement methods take a tour as the argument \code{tour} and try to improve the
tour. If no tour is provided. A random tour is used as the initial tour.

The following tour improvement heuristics are available:
\itemize{
\item \strong{"sa"} Simulated Annealing for TSPs (Kirkpatrick et al, 1983) [TSP, ATSP]

A tour refinement method that uses simulated annealing with subtour
reversal as local move. The used optimizer is
\code{\link[stats:optim]{stats::optim()}} with method \code{"SANN"}. This method is
typically a lot slower than \code{"two_opt"} and requires parameter tuning for the
cooling schedule.

\strong{Note:} For speed, a relatively small number of evaluations (\code{maxit}) is
used by default. A much larger value will result in more competitive results.

Additional control options:
\itemize{
\item "tour" an existing tour which should be improved.
If no tour is given, a random tour is used.
\item "local_move" a function that creates a local move with the current
tour as the first and the TSP as the second parameter. Defaults to
randomized subtour reversal.
\item "temp" initial temperature. Defaults to the length of the current tour divided
by the number of cities.
\item "tmax" number of evaluations per temperature step. Default is 10.
\item "maxit" number of evaluations. Default is 10000.
\item "trace" set to 1 to print progress.
}

See \code{\link[stats:optim]{stats::optim()}} for more details on the parameters.
\item \strong{"two_opt"} Two edge exchange improvement procedure (Croes 1958). [TSP, ATSP]

This is a tour refinement procedure which systematically exchanges two edges
in the graph represented by the distance matrix till no improvements are
possible. Exchanging two edges is equal to reversing part of the tour. The
resulting tour is called \emph{2-optimal.}

Additional control options:
\itemize{
\item "tour" an existing tour which should be improved.
If no tour is given, a random tour is used.
\item "two_opt_repetitions" number of times to try with random restarts
(default: 1).
}
}
}

\subsection{State-of-the-art Solvers Interfaces}{
\itemize{
\item \strong{"concorde"} Concorde algorithm (Applegate et al. 2001). [TSP, ETSP]

Concorde is an advanced exact TSP solver for \emph{symmetric} TSPs
based on branch-and-cut.
ATSPs can be solved using \code{\link[=reformulate_ATSP_as_TSP]{reformulate_ATSP_as_TSP()}} done automatically
with \code{as_TSP = TRUE}.
The program is not included in this package and
has to be obtained and installed separately.

Additional control options:
\itemize{
\item "exe" a character string containing the path to the executable (see \link{Concorde}).
\item "clo" a character string containing command line options for
Concorde, e.g., \code{control = list(clo = "-B -v")}. See
\code{\link[=concorde_help]{concorde_help()}} on how to obtain a complete list of available command
line options.
\item "precision" an integer which controls the number of decimal
places used for the internal representation of distances in Concorde. The
values given in \code{x} are multiplied by \eqn{10^{precision}} before being
passed on to Concorde. Note that therefore the results produced by Concorde
(especially lower and upper bounds) need to be divided by
\eqn{10^{precision}} (i.e., the decimal point has to be shifted
\code{precision} placed to the left). The interface to Concorde uses
\code{\link[=write_TSPLIB]{write_TSPLIB()}}.
\item "verbose" logical; \code{FALSE} suppresses the output printed to the terminal.
}
\item \strong{"linkern"} Concorde's Chained Lin-Kernighan heuristic (Applegate et al. 2003). [TSP, ETSP]

The Lin-Kernighan (Lin and Kernighan 1973) heuristic uses variable \eqn{k}
edge exchanges to improve an initial tour.  The program is not included in
this package and has to be obtained and installed separately (see \link{Concorde}).

Additional control options: see Concorde above.
}
}
}

\section{Parallel Execution Support}{
If several repetitions are performed (this includes method
\code{"repetitive_nn"}) then \pkg{foreach} is used so they can be performed
in parallel on multiple cores/machines. To enable parallel execution an
appropriate parallel backend needs to be registered
(e.g., \code{\link[doParallel:registerDoParallel]{doParallel::registerDoParallel()}}).
}

\section{Treatment of \code{NA}s and Infinite Values in \code{x}}{
\link{TSP} and \link{ATSP} need to contain valid distances. \code{NA}s are not allowed. \code{Inf} is
allowed and can be used to model the missing edges in incomplete graphs
(i.e., the distance between the two objects is infinite) or infeasible connections.
Internally, \code{Inf} is replaced by a large value given by \eqn{max(x) + 2 range(x)}.
Note that the solution might still place the two objects next to each other
(e.g., if \code{x} contains several unconnected subgraphs) which results in
a path length of \code{Inf}. \code{-Inf} is replaced by \eqn{min(x) - 2 range(x)} and
can be used to encourage the solver to place two objects next to each other.
}

\section{Solving ATSP and ETSP}{
Some solvers (including Concorde) cannot directly solve \link{ATSP}
directly. \code{ATSP} can be reformulated as larger \code{TSP} and solved
this way. For convenience, \code{solve_TSP()} has an extra argument
\code{as_TSP} which can be set to \code{TRUE} to automatically solve the
\code{ATSP} reformulated as a \code{TSP} (see \code{\link[=reformulate_ATSP_as_TSP]{reformulate_ATSP_as_TSP()}}).

Only methods "concorde" and "linkern" currently solve \link{ETSP}s directly.
For all other methods, ETSPs are converted into TSPs by creating a
distance matrix and then solved. Note: distance matrices can become
very large leading to long memory issues and long computation times.
}

\examples{

## solve a simple Euclidean TSP (using the default method)
etsp <- ETSP(data.frame(x = runif(20), y = runif(20)))
tour <- solve_TSP(etsp)
tour
tour_length(tour)
plot(etsp, tour)

## manually run a construction followed by an improvement heuristic by 
## calling solve_TSP() twice.
tour_constr <- solve_TSP(etsp, method = "nn")
tour_constr

tour_improved <- solve_TSP(etsp, method = "sa", tour = tour_constr)
tour_improved

## compare methods
data("USCA50")
USCA50
methods <- c("identity", "random", "nearest_insertion",
  "cheapest_insertion", "farthest_insertion", "arbitrary_insertion",
  "nn", "repetitive_nn", "two_opt", "sa")

## calculate tours
tours <- lapply(methods, FUN = function(m) solve_TSP(USCA50, method = m))
names(tours) <- methods

## use the external solver which has to be installed separately
\dontrun{
tours$concorde  <- solve_TSP(USCA50, method = "concorde")
tours$linkern  <- solve_TSP(USCA50, method = "linkern")
}

## register a parallel backend to perform repetitions in parallel
\dontrun{
library(doParallel)
registerDoParallel()
}

## add some tours using repetition and two_opt refinements
tours$'nn+two_opt' <- solve_TSP(USCA50, method = "nn", two_opt = TRUE)
tours$'nn+rep_10' <- solve_TSP(USCA50, method = "nn", rep = 10)
tours$'nn+two_opt+rep_10' <- solve_TSP(USCA50, method = "nn", two_opt = TRUE, rep = 10)
tours$'arbitrary_insertion+two_opt' <- solve_TSP(USCA50)

## show first tour
tours[[1]]

## compare tour lengths
opt <- 14497 # obtained by Concorde
tour_lengths <- sort(sapply(tours, tour_length), decreasing = TRUE)
dotchart(tour_lengths / opt * 100 - 100, xlab = "percent excess over optimum")
abline(v = 0, col = "red")
}
\references{
David Applegate, Robert Bixby, Vasek Chvatal, William Cook
(2001): TSP cuts which do not conform to the template paradigm,
Computational Combinatorial Optimization, M. Junger and D. Naddef (editors),
Springer.\doi{10.1007/3-540-45586-8_7}

D. Applegate, W. Cook and A. Rohe (2003): Chained Lin-Kernighan for Large
Traveling Salesman Problems.  \emph{INFORMS Journal on Computing},
15(1):82--92. \doi{10.1287/ijoc.15.1.82.15157}

G.A. Croes (1958): A method for solving traveling-salesman problems.
\emph{Operations Research}, 6(6):791--812. \doi{10.1287/opre.6.6.791}

Kirkpatrick, S., C. D. Gelatt, and M. P. Vecchi (1983):
Optimization by Simulated Annealing. \emph{Science} 220 (4598): 671–80
\doi{10.1126/science.220.4598.671}

S. Lin and B. Kernighan (1973): An effective heuristic algorithm for the
traveling-salesman problem. \emph{Operations Research}, 21(2): 498--516.
\doi{10.1287/opre.21.2.498}

D.J. Rosenkrantz, R. E. Stearns, and Philip M. Lewis II (1977): An analysis
of several heuristics for the traveling salesman problem.  \emph{SIAM
Journal on Computing}, 6(3):563--581. \doi{10.1007/978-1-4020-9688-4_3}
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()}

Other TOUR: 
\code{\link{TOUR}()},
\code{\link{cut_tour}()},
\code{\link{tour_length}()}
}
\author{
Michael Hahsler
}
\concept{TOUR}
\concept{TSP}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TSPLIB.R
\name{TSPLIB}
\alias{TSPLIB}
\alias{read_TSPLIB}
\alias{write_TSPLIB}
\alias{write_TSPLIB.TSP}
\alias{write_TSPLIB.ATSP}
\alias{write_TSPLIB.ETSP}
\title{Read and write TSPLIB files}
\usage{
read_TSPLIB(file, precision = 0)

write_TSPLIB(x, file, precision = 6, inf = NULL, neg_inf = NULL)

\method{write_TSPLIB}{TSP}(x, file, precision = 6, inf = NULL, neg_inf = NULL)

\method{write_TSPLIB}{ATSP}(x, file, precision = 6, inf = NULL, neg_inf = NULL)

\method{write_TSPLIB}{ETSP}(x, file, precision = 6, inf = NULL, neg_inf = NULL)
}
\arguments{
\item{file}{file name or a \link{connection}.}

\item{precision}{controls the number of decimal places used to represent
distances (see details).  If \code{x} already is \code{integer}, this
argument is ignored and \code{x} is used as is.}

\item{x}{an object with a TSP problem.
\code{NA}s are not allowed.}

\item{inf}{replacement value for \code{Inf} (TSPLIB format cannot handle
\code{Inf}). If \code{inf} is \code{NULL}, a large value of \eqn{max(x) + 2
range(x)} (ignoring infinite entries) is used.}

\item{neg_inf}{replacement value for \code{-Inf}.  If no value is specified,
a small value of \eqn{min(x) - 2 range(x)} (ignoring infinite entries) is
used.}
}
\value{
returns an object of class \code{TSP} or
\code{ATSP}.
}
\description{
Reads and writes TSPLIB format files. TSPLIB files can be used by most TSP
solvers. Many problems in TSPLIB format can be found in the local
copy of the \href{https://github.com/mhahsler/TSP/tree/master/TSPLIB95}{TSPLIB95 problem library}.
}
\details{
In the TSPLIB format distances are represented by integer values. Therefore,
if \code{x} contains \code{double} values (which is normal in R) the values
given in \code{x} are multiplied by \eqn{10^{precision}} before coercion to
\code{integer}. Note that therefore all results produced by programs using
the TSPLIB file as input need to be divided by \eqn{10^{precision}} (i.e.,
the decimal point has to be shifted \code{precision} placed to the left).

Currently only the following \code{EDGE_WEIGHT_TYPE}s are implemented:
\code{EXPLICIT}, \code{EUC_2D}, \code{EUC_3D}, \code{ATT} and \code{GEO}.
}
\examples{

## Drilling problem from TSP
drill <- read_TSPLIB(system.file("examples/d493.tsp", package = "TSP"))
drill
tour <- solve_TSP(drill, method = "nn", two_opt = TRUE)
tour
plot(drill, tour, cex=.6, col = "red", pch= 3, main = "TSPLIB: d493")


## Write and read data in TSPLIB format
x <- data.frame(x=runif(5), y=runif(5))

## create TSP, ATSP and ETSP (2D)
tsp <- TSP(dist(x))
atsp <- ATSP(dist(x))
etsp <- ETSP(x[,1:2])

write_TSPLIB(tsp, file="example.tsp")
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp")
r

write_TSPLIB(atsp, file="example.tsp")
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp")
r

write_TSPLIB(etsp, file="example.tsp")
#file.show("example.tsp")
r <- read_TSPLIB("example.tsp")
r

## clean up
unlink("example.tsp")
}
\references{
Reinelt, Gerhard. 1991. “TSPLIB—a Traveling Salesman Problem Library.” ORSA Journal on Computing 3 (4): 376–84. \doi{10.1287/ijoc.3.4.376}
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{file}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ATSP.R
\name{ATSP}
\alias{ATSP}
\alias{as.ATSP}
\alias{as.ATSP.matrix}
\alias{as.ATSP.dist}
\alias{print.ATSP}
\alias{n_of_cities.ATSP}
\alias{labels.ATSP}
\alias{image.ATSP}
\alias{as.matrix.ATSP}
\title{Class ATSP -- Asymmetric traveling salesperson problem}
\usage{
ATSP(x, labels = NULL, method = NULL)

as.ATSP(x)

\method{as.ATSP}{matrix}(x)

\method{as.ATSP}{dist}(x)

\method{print}{ATSP}(x, ...)

\method{n_of_cities}{ATSP}(x)

\method{labels}{ATSP}(object, ...)

\method{image}{ATSP}(x, order, col = gray.colors(64), ...)

\method{as.matrix}{ATSP}(x, ...)
}
\arguments{
\item{x, object}{an object (a square matrix) to be converted into an
\code{ATSP} or, for the methods, an object of class \code{ATSP}.}

\item{labels}{optional city labels. If not given, labels are taken from
\code{x}.}

\item{method}{optional name of the distance metric.}

\item{...}{further arguments are passed on.}

\item{order}{order of cities as an integer vector or an object of class
\code{TOUR}.}

\item{col}{color scheme for image.}
}
\value{
\itemize{
\item \code{ATSP()} returns \code{x} as an object of class \code{ATSP}.
\item \code{n_of_cities()} returns the number of cities in \code{x}.
\item \code{labels()} returns a vector with the names of the cities in \code{x}.
}
}
\description{
Constructor to create an instance of the asymmetric traveling salesperson
problem (ATSP) and some auxiliary methods.
}
\details{
Objects of class \code{ATSP} are internally represented by a matrix (use
\code{as.matrix()} to get just the matrix).

ATSPs can be transformed into (larger) symmetric TSPs using
\code{\link[=reformulate_ATSP_as_TSP]{reformulate_ATSP_as_TSP()}}.
}
\examples{
data <- matrix(runif(10^2), ncol = 10, dimnames = list(1:10, 1:10))

atsp <- ATSP(data)
atsp

## use some methods
n_of_cities(atsp)
labels(atsp)

## calculate a tour
tour <- solve_TSP(atsp, method = "nn")
tour

tour_length(tour)

image(atsp, tour)
}
\seealso{
Other TSP: 
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TSP.R
\name{TSP}
\alias{TSP}
\alias{as.TSP}
\alias{as.TSP.dist}
\alias{as.TSP.matrix}
\alias{as.dist.TSP}
\alias{print.TSP}
\alias{n_of_cities}
\alias{n_of_cities.TSP}
\alias{labels.TSP}
\alias{image.TSP}
\title{Class TSP -- Symmetric traveling salesperson problem}
\usage{
TSP(x, labels = NULL, method = NULL)

as.TSP(x)

\method{as.TSP}{dist}(x)

\method{as.TSP}{matrix}(x)

\method{as.dist}{TSP}(m, ...)

\method{print}{TSP}(x, ...)

n_of_cities(x)

\method{n_of_cities}{TSP}(x)

\method{labels}{TSP}(object, ...)

\method{image}{TSP}(x, order, col = gray.colors(64), ...)
}
\arguments{
\item{x, object}{an object (currently \code{dist} or a symmetric matrix) to
be converted into a \code{TSP} or, for the methods, an object of class
\code{TSP}.}

\item{labels}{optional city labels. If not given, labels are taken from
\code{x}.}

\item{method}{optional name of the distance metric. If \code{x} is a
\code{dist} object, then the method is taken from that object.}

\item{m}{a TSP object to be converted to a \link{dist} object.}

\item{...}{further arguments are passed on.}

\item{order}{order of cities for the image as an integer vector or an object
of class \link{TOUR}.}

\item{col}{color scheme for image.}
}
\value{
\itemize{
\item \code{TSP()} returns \code{x} as an object of class \code{TSP}.
\item \code{n_of_cities()} returns the number of cities in \code{x}.
\item \code{labels()} returns a vector with the names of the cities in \code{x}.
}
}
\description{
Constructor to create an instance of a symmetric traveling salesperson
problem (TSP) and some auxiliary methods.
}
\details{
Objects of class \code{TSP} are internally represented as \code{dist}
objects (use \code{\link[=as.dist]{as.dist()}} to get the \code{dist} object).

Not permissible paths can be set to a distance of \code{+Inf}. \code{NA}s are not allowed and \code{-Inf} will lead
to the algorithm only being able to find an admissible tour, but not the best one.
}
\examples{
data("iris")
d <- dist(iris[-5])

## create a TSP
tsp <- TSP(d)
tsp

## use some methods
n_of_cities(tsp)
labels(tsp)
image(tsp)
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/USCA312.R
\docType{data}
\name{USCA}
\alias{USCA}
\alias{USCA312}
\alias{USCA312_GPS}
\alias{USCA50}
\title{USCA312/USCA50 -- 312/50 cities in the US and Canada}
\format{
\code{USCA312} and \code{USCA50} are objects of class \code{TSP}.
\code{USCA312_GPS} is a data.frame with city name, long and lat.
}
\source{
John Burkardt, CITIES -- City Distance Datasets, Florida State
University, Department of Scientific Computing
}
\description{
The \code{USCA312} dataset contains the distances between 312 cities in the
US and Canada as an object of class \code{TSP}.  \code{USCA50} is a subset
of \code{USCA312} containing only the first 50 cities.
}
\details{
The \code{USCA312_GPS} dataset contains the location (long/lat) of the 312
cities.
}
\examples{
data("USCA312")

## calculate a tour
tour <- solve_TSP(USCA312)
tour

# Visualize the tour if package maps is installed
if(require("maps")) {

  library(maps)
  data("USCA312_GPS")
  head(USCA312_GPS)

  plot((USCA312_GPS[, c("long", "lat")]), cex = .3)
  map("world", col = "gray", add = TRUE)
  polygon(USCA312_GPS[, c("long", "lat")][tour,], border = "red")
}
}
\author{
Michael Hahsler
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tour_length.R
\name{tour_length}
\alias{tour_length}
\alias{tour_length.TSP}
\alias{tour_length.ATSP}
\alias{tour_length.ETSP}
\alias{tour_length.TOUR}
\alias{tour_length.integer}
\title{Calculate the length of a tour}
\usage{
tour_length(x, ...)

\method{tour_length}{TSP}(x, order, ...)

\method{tour_length}{ATSP}(x, order, ...)

\method{tour_length}{ETSP}(x, order, ...)

\method{tour_length}{TOUR}(x, tsp = NULL, ...)

\method{tour_length}{integer}(x, tsp = NULL, ...)
}
\arguments{
\item{x}{a TSP problem or a \link{TOUR}.}

\item{...}{further arguments are currently unused.}

\item{order}{an object of class \code{TOUR}}

\item{tsp}{as TSP object.}
}
\description{
Calculate the length of a \link{TOUR} for a \link{TSP}.
}
\details{
If no \code{tsp} is specified, then the tour length stored in \code{x} as
attribute \code{"tour_length"} is returned.  If \code{tsp} is given then the
tour length is recalculated using the specified TSP problem.

If a distance in the tour is infinite, the result is also infinite. If the
tour contains positive and negative infinite distances then the method
returns \code{NA}.
}
\examples{

data("USCA50")

## original order
tour_length(solve_TSP(USCA50, method="identity"))

## length of a manually created (random) tour
tour <- TOUR(sample(seq(n_of_cities(USCA50))))
tour
tour_length(tour)
tour_length(tour, USCA50)
}
\seealso{
Other TOUR: 
\code{\link{TOUR}()},
\code{\link{cut_tour}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TOUR}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA_TSP-package.R
\docType{package}
\name{TSP-package}
\alias{TSP-package}
\title{TSP: Infrastructure for the Traveling Salesperson Problem}
\description{
Basic infrastructure and some algorithms for the traveling salesperson problem (also traveling salesman problem; TSP). The package provides some simple algorithms and an interface to the Concorde TSP solver and its implementation of the Chained-Lin-Kernighan heuristic. The code for Concorde itself is not included in the package and has to be obtained separately. Hahsler and Hornik (2007) \doi{10.18637/jss.v023.i02}.
}
\section{Key functions}{

\itemize{
\item \code{\link[=solve_TSP]{solve_TSP()}}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mhahsler/TSP}
  \item Report bugs at \url{https://github.com/mhahsler/TSP/issues}
}

}
\author{
\strong{Maintainer}: Michael Hahsler \email{mhahsler@lyle.smu.edu} (\href{https://orcid.org/0000-0003-2716-1405}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Kurt Hornik (\href{https://orcid.org/0000-0003-4198-9911}{ORCID}) [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert_dummy.R
\name{insert_dummy}
\alias{insert_dummy}
\alias{insert_dummy.TSP}
\alias{insert_dummy.ATSP}
\alias{insert_dummy.ETSP}
\title{Insert dummy cities into a distance matrix}
\usage{
insert_dummy(x, n = 1, const = 0, inf = Inf, label = "dummy")

\method{insert_dummy}{TSP}(x, n = 1, const = 0, inf = Inf, label = "dummy")

\method{insert_dummy}{ATSP}(x, n = 1, const = 0, inf = Inf, label = "dummy")

\method{insert_dummy}{ETSP}(x, n = 1, const = 0, inf = Inf, label = "dummy")
}
\arguments{
\item{x}{an object with a TSP problem.}

\item{n}{number of dummy cities.}

\item{const}{distance of the dummy cities to all other cities.}

\item{inf}{distance between dummy cities.}

\item{label}{labels for the dummy cities. If only one label is given, it is
reused for all dummy cities.}
}
\value{
returns an object of the same class as \code{x}.
}
\description{
Inserts dummy cities into a TSP problem.  A
dummy city has the same, constant distance (0) to all other cities and is
infinitely far from other dummy cities. A dummy city can be used to
transform a shortest Hamiltonian path problem (i.e., finding an optimal
linear order) into a shortest Hamiltonian cycle problem which can be solved
by a TSP solvers (Garfinkel 1985).
}
\details{
Several dummy cities can be used together with a TSP solvers to perform
rearrangement clustering (Climer and Zhang 2006).

The dummy cities are inserted after the other cities in \code{x}.

A \code{const} of 0 is guaranteed to work if the TSP finds the optimal
solution. For heuristics returning suboptimal solutions, a higher
\code{const} (e.g., \code{2 * max(x)}) might provide better results.
}
\examples{
## Example 1: Find a short Hamiltonian path
set.seed(1000)
x <- data.frame(x = runif(20), y = runif(20), row.names = LETTERS[1:20])

tsp <- TSP(dist(x))

## add a dummy city to cut the tour into a path
tsp <- insert_dummy(tsp, label = "cut")
tour <- solve_TSP(tsp)
tour

plot(x)
lines(x[cut_tour(tour, cut = "cut"),])


## Example 2: Rearrangement clustering of the iris dataset
set.seed(1000)
data("iris")
tsp <- TSP(dist(iris[-5]))

## insert 2 dummy cities to creates 2 clusters
tsp_dummy <- insert_dummy(tsp, n = 3, label = "boundary")

## get a solution for the TSP
tour <- solve_TSP(tsp_dummy)

## plot the reordered distance matrix with the dummy cities as lines separating
## the clusters
image(tsp_dummy, tour)
abline(h = which(labels(tour)=="boundary"), col = "red")
abline(v = which(labels(tour)=="boundary"), col = "red")

## plot the original data with paths connecting the points in each cluster
plot(iris[,c(2,3)], col = iris[,5])
paths <- cut_tour(tour, cut = "boundary")
for(p in paths) lines(iris[p, c(2,3)])

## Note: The clustering is not perfect!
}
\references{
Sharlee Climer, Weixiong Zhang (2006): Rearrangement Clustering:
Pitfalls, Remedies, and Applications, \emph{Journal of Machine Learning
Research} \bold{7}(Jun), pp. 919--943.

R.S. Garfinkel (1985): Motivation and modelling (chapter 2). In: E. L.
Lawler, J. K. Lenstra, A.H.G. Rinnooy Kan, D.  B. Shmoys (eds.) The
traveling salesman problem - A guided tour of combinatorial optimization,
Wiley & Sons.
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{reformulate_ATSP_as_TSP}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reformulare_ATSP_as_TSP.R
\name{reformulate_ATSP_as_TSP}
\alias{reformulate_ATSP_as_TSP}
\alias{filter_ATSP_as_TSP_dummies}
\title{Reformulate a ATSP as a symmetric TSP}
\usage{
reformulate_ATSP_as_TSP(x, infeasible = Inf, cheap = -Inf)

filter_ATSP_as_TSP_dummies(tour, atsp)
}
\arguments{
\item{x}{an \link{ATSP}.}

\item{infeasible}{value for infeasible connections.}

\item{cheap}{value for distance between a city and its corresponding dummy
city.}

\item{tour}{a \link{TOUR} created for a ATSP reformulated as a TSP.}

\item{atsp}{the original \link{ATSP}.}
}
\value{
\code{reformulate_ATSP_as_TSP()} returns a \link{TSP} object.
\code{filter_ATSP_as_TSP_dummies()} returns a \link{TOUR} object.
}
\description{
A ATSP can be formulated as a symmetric TSP by doubling the number of cities
(Jonker and Volgenant 1983). The solution of the TSP also represents the
solution of the original ATSP.
}
\details{
To reformulate a \link{ATSP} as a \link{TSP}, for each city a dummy city (e.g, for 'New
York' a dummy city 'New York*') is added. Between each city and its
corresponding dummy city a very small (or negative) distance with value
\code{cheap} is used.
To ensure that the solver places each cities always occurs in the
solution together with its dummy city, this cost has to be much smaller than
the distances in the TSP.
The original distances are used
between the cities and the dummy cities, where each city is responsible for
the distance going to the city and the dummy city is responsible for the
distance coming from the city. The distances between all cities and the
distances between all dummy cities are set to \code{infeasible}, a very
large value which prevents the solver from using these links.
We use infinite values here and \code{\link[=solve_TSP]{solve_TSP()}} treats them appropriately.

\code{filter_ATSP_as_TSP_dummies()} can be used to extract the solution for the original
ATSP from the tour found for an ATSP reformulated as a TSP. Note that the symmetric TSP
tour does not reveal the direction for the ATSP. The filter function computed the
tour length for both directions and returns the shorter tour.

\code{\link[=solve_TSP]{solve_TSP()}} has a parameter \code{as_TSP} which preforms the reformulation and
filtering the dummy cities automatically.

\strong{Note on performance:} Doubling the problem size is a performance issue especially
has a negative impact on solution quality for heuristics. It should only be used
together with Concorde when the optimal solution is required. Most heuristics can solve
ATSPs directly with good solution quality.
}
\examples{
data("USCA50")

## set the distances from anywhere to Austin to zero which makes it an ATSP
austin <- which(labels(USCA50) == "Austin, TX")
atsp <- as.ATSP(USCA50)
atsp[, austin] <- 0
atsp

## reformulate as a TSP (by doubling the number of cities with dummy cities marked with *)
tsp <- reformulate_ATSP_as_TSP(atsp)
tsp

## create tour for the TSP. You should use Concorde to find the optimal solution.
# tour_tsp <- solve_TSP(tsp, method = "concorde")
# The standard heuristic is bad for this problem. We use it here because
#   Concord may not be installed.
tour_tsp <- solve_TSP(tsp)
head(labels(tour_tsp), n = 10)
tour_tsp
# The tour length is -Inf since it includes cheap links
#  from a city to its dummy city.

## get the solution for the original ATSP by filtering out the dummy cities.
tour_atsp <- filter_ATSP_as_TSP_dummies(tour_tsp, atsp = atsp)
tour_atsp
head(labels(tour_atsp), n = 10)

## This process can also be done automatically by using as_TSP = TRUE:
# solve_TSP(atsp, method = "concorde", as_TSP = TRUE)

## The default heuristic can directly solve ATSPs with results close to the
#  optimal solution of 12715.
solve_TSP(atsp, control = list(rep = 10))
}
\references{
Jonker, R. and Volgenant, T. (1983): Transforming asymmetric
into symmetric traveling salesman problems, \emph{Operations Research
Letters,} 2, 161--163.
}
\seealso{
Other TSP: 
\code{\link{ATSP}()},
\code{\link{Concorde}},
\code{\link{ETSP}()},
\code{\link{TSP}()},
\code{\link{TSPLIB}},
\code{\link{insert_dummy}()},
\code{\link{solve_TSP}()}
}
\author{
Michael Hahsler
}
\concept{TSP}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TOUR.R
\name{TOUR}
\alias{TOUR}
\alias{as.TOUR}
\alias{as.TOUR.numeric}
\alias{as.TOUR.integer}
\alias{print.TOUR}
\title{Class TOUR -- Solution to a traveling salesperson problem}
\usage{
TOUR(x, method = NA, tsp = NULL)

as.TOUR(object)

\method{as.TOUR}{numeric}(object)

\method{as.TOUR}{integer}(object)

\method{print}{TOUR}(x, ...)
}
\arguments{
\item{x}{an integer permutation vector or, for the methods an object of
class \link{TOUR}.}

\item{method}{character string; method used to create the tour.}

\item{tsp}{\code{TSP} object the tour applies to. If available then the tour
will include the tour length. Also the labels of the cities will be
available in the tour (otherwise the labels of \code{x} are used).}

\item{object}{data (an integer vector) which can be coerced to \code{TOUR}.}

\item{...}{further arguments are passed on.}
}
\description{
Class to store the solution of a TSP. Objects of this class are returned by
TSP solvers in this package.  Essentially, an object of class \code{TOUR} is
a permutation vector containing the order of cities to visit.
}
\details{
Since an object of class \code{TOUR} is an integer vector, it can be
subsetted as an ordinary vector or coerced to an integer vector using
\code{as.integer()}. It also contains the names of the objects as labels.
Additionally, \code{TOUR} has the following attributes: \code{"method"},
\code{"tour_length"}.

For most functions, e.g., \code{\link[=tour_length]{tour_length()}} or \code{\link[=image.TSP]{image.TSP()}}, the
\code{TSP/ATSP} object used to find the tour is still needed, since the tour
does not contain the distance information.
}
\examples{
TOUR(1:10)

## calculate a tour
data("USCA50")
tour <- solve_TSP(USCA50)
tour

## get tour length directly from tour
tour_length(tour)

## get permutation vector
as.integer(tour)

## show labels
labels(tour)
}
\seealso{
Other TOUR: 
\code{\link{cut_tour}()},
\code{\link{solve_TSP}()},
\code{\link{tour_length}()}
}
\author{
Michael Hahsler
}
\concept{TOUR}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cut_tour.R
\name{cut_tour}
\alias{cut_tour}
\alias{cut_tour.TOUR}
\title{Cut a tour to form a path}
\usage{
cut_tour(x, cut, exclude_cut = TRUE)

\method{cut_tour}{TOUR}(x, cut, exclude_cut = TRUE)
}
\arguments{
\item{x}{an object of class \link{TOUR}.}

\item{cut}{the index or label of the city/cities to cut the tour.}

\item{exclude_cut}{exclude the city where we cut? If \code{FALSE}, the city
at the cut is included in the path as the first city.}
}
\value{
Returns a named vector with city ids forming the path. If multiple
cuts are used then a list with paths is returned.
}
\description{
Cuts a tour at a specified city to form a path.
}
\examples{
data("USCA50")

## find a path starting at Austin, TX
tour <- solve_TSP(USCA50)
path <- cut_tour(tour, cut = "Austin, TX", exclude_cut = FALSE)
path

## cut the tours at two cities
tour <- solve_TSP(USCA50)
path <- cut_tour(tour, cut = c("Austin, TX", "Cambridge, MA"), exclude_cut = FALSE)
path

## cut a tour at the largest gap using a dummy city
tsp <- insert_dummy(USCA50, label = "cut")
tour <- solve_TSP(tsp)

## cut tour into path at the dummy city
path <- cut_tour(tour, "cut")
path
}
\seealso{
Other TOUR: 
\code{\link{TOUR}()},
\code{\link{solve_TSP}()},
\code{\link{tour_length}()}
}
\author{
Michael Hahsler
}
\concept{TOUR}
\keyword{optimize}
